/**
 * FLL - Level 0
 *
 * Project: IKI
 * API Version: 0.7
 * Licenses: lgpl-2.1-or-later
 *
 * Defines iki common data.
 *
 * This is auto-included by iki.h and should not need to be explicitly included.
 */
#ifndef _F_iki_common_h
#define _F_iki_common_h

#ifdef __cplusplus
extern "C" {
#endif

/**
 * Provide a default allocation step.
 *
 * For a UTF-8 friendly allocation step, set to at least 4.
 *
 * F_iki_default_allocation_*_d:
 *   - console: An allocation step used for small buffers specifically for console parameter.
 *   - large:   The large allocation size.
 *   - small:   The small allocation size, in particular used for allocation iki strings.
 */
#ifndef _di_f_iki_default_d_
  #define F_iki_default_allocation_console_d 8
  #define F_iki_default_allocation_large_d   F_memory_default_allocation_large_d
  #define F_iki_default_allocation_small_d   8
#endif // _di_f_iki_default_d_

/**
 * State flags associated with iki functions.
 *
 * These flags are meant to be bitwise for the 32-bit f_state_t flag property.
 *
 * The f_iki_state_flag_none_e is expected to be 0, therefore it must be safe to use 0 directly.
 *
 * f_iki_state_flag_*_e:
 *   - none:                  No flags are set.
 *   - force_wrap:            For IKI writing, this forces the use of wrapping characters even if this is not required.
 *   - utf_fail_on_valid_not: Immediately fail on invalid UTF-8 character (including incomplete).
 */
#ifndef _di_f_iki_state_flag_e_
  enum {
    f_iki_state_flag_none_e                  = 0x0,
    f_iki_state_flag_force_wrap_e            = 0x1,
    f_iki_state_flag_utf_fail_on_valid_not_e = 0x2,
  }; // enum
#endif // _di_f_iki_state_flag_e_

/**
 * IKI-specific syntax.
 *
 * f_iki_syntax_*_e:
 *   - separator:    The separator character used to separate the Vocabulary from the Content.
 *   - placeholder:  The placeholder character (such as NULL), often used to remove delimits before printing.
 *   - quote_double: The double-quote character used for wrapping the Content.
 *   - quote_grave:  The grave character used for wrapping the Content.
 *   - quote_single: The single-quote character used for wrapping the Content.
 *   - slash:        The slash character used for escaping.
 *   - wrap_open:    The wrapping character used at the start of a Vocabulary.
 *   - wrap_close:   The separator character used at the stop of a Vocabulary.
 */
#ifndef _di_f_iki_syntax_s_
  #define F_iki_syntax_separator_s    F_string_ascii_colon_s
  #define F_iki_syntax_placeholder_s  F_string_placeholder_s
  #define F_iki_syntax_quote_double_s F_string_ascii_quote_double_s
  #define F_iki_syntax_quote_grave_s  F_string_ascii_grave_s
  #define F_iki_syntax_quote_single_s F_string_ascii_quote_single_s
  #define F_iki_syntax_slash_s        F_string_ascii_slash_backward_s
  #define F_iki_syntax_wrap_open_s    F_string_ascii_bracket_open_s
  #define F_iki_syntax_wrap_close_s   F_string_ascii_bracket_close_s

  #define F_iki_syntax_separator_s_length    F_string_ascii_colon_s_length
  #define F_iki_syntax_placeholder_s_length  F_string_placeholder_s_length
  #define F_iki_syntax_quote_double_s_length F_string_ascii_quote_double_s_length
  #define F_iki_syntax_quote_grave_s_length  F_string_ascii_grave_s_length
  #define F_iki_syntax_quote_single_s_length F_string_ascii_quote_single_s_length
  #define F_iki_syntax_slash_s_length        F_string_ascii_slash_backward_s_length
  #define F_iki_syntax_wrap_open_s_length    F_string_ascii_bracket_open_s_length
  #define F_iki_syntax_wrap_close_s_length   F_string_ascii_bracket_close_s_length

  #ifndef _di_f_iki_syntax_separator_s_
    extern const f_string_static_t f_iki_syntax_separator_s;
  #endif // _di_f_iki_syntax_separator_s_

  #ifndef _di_f_iki_syntax_placeholder_s_
    extern const f_string_static_t f_iki_syntax_placeholder_s;
  #endif // _di_f_iki_syntax_placeholder_s_

  #ifndef _di_f_iki_syntax_quote_double_s_
    extern const f_string_static_t f_iki_syntax_quote_double_s;
  #endif // _di_f_iki_syntax_quote_double_s_

  #ifndef _di_f_iki_syntax_quote_grave_s_
    extern const f_string_static_t f_iki_syntax_quote_grave_s;
  #endif // _di_f_iki_syntax_quote_grave_s_

  #ifndef _di_f_iki_syntax_quote_single_s_
    extern const f_string_static_t f_iki_syntax_quote_single_s;
  #endif // _di_f_iki_syntax_quote_single_s_

  #ifndef _di_f_iki_syntax_slash_s_
    extern const f_string_static_t f_iki_syntax_slash_s;
  #endif // _di_f_iki_syntax_slash_s_

  #ifndef _di_f_iki_syntax_wrap_open_s_
    extern const f_string_static_t f_iki_syntax_wrap_open_s;
  #endif // _di_f_iki_syntax_wrap_open_s_

  #ifndef _di_f_iki_syntax_wrap_close_s_
    extern const f_string_static_t f_iki_syntax_wrap_close_s;
  #endif // _di_f_iki_syntax_wrap_close_s_
#endif //_di_f_iki_syntax_s_

#ifndef _di_f_iki_vocabulary_0001_s_
  #define F_iki_vocabulary_0001_address_s  "address"
  #define F_iki_vocabulary_0001_code_s     "code"
  #define F_iki_vocabulary_0001_email_s    "email"
  #define F_iki_vocabulary_0001_name_s     "name"
  #define F_iki_vocabulary_0001_phone_s    "phone"
  #define F_iki_vocabulary_0001_quote_s    "quote"
  #define F_iki_vocabulary_0001_uri_s      "uri"
  #define F_iki_vocabulary_0001_url_s      "url"
  #define F_iki_vocabulary_0001_urn_s      "urn"
  #define F_iki_vocabulary_0001_variable_s "var"

  #define F_iki_vocabulary_0001_address_s_length  7
  #define F_iki_vocabulary_0001_code_s_length     4
  #define F_iki_vocabulary_0001_email_s_length    5
  #define F_iki_vocabulary_0001_name_s_length     4
  #define F_iki_vocabulary_0001_phone_s_length    5
  #define F_iki_vocabulary_0001_quote_s_length    5
  #define F_iki_vocabulary_0001_uri_s_length      3
  #define F_iki_vocabulary_0001_url_s_length      3
  #define F_iki_vocabulary_0001_urn_s_length      3
  #define F_iki_vocabulary_0001_variable_s_length 3

  #ifndef _di_f_iki_vocabulary_0001_address_s_
    extern const f_string_static_t f_iki_vocabulary_0001_address_s;
  #endif // _di_f_iki_vocabulary_0001_address_s_

  #ifndef _di_f_iki_vocabulary_0001_code_s_
    extern const f_string_static_t f_iki_vocabulary_0001_code_s;
  #endif // _di_f_iki_vocabulary_0001_code_s_

  #ifndef _di_f_iki_vocabulary_0001_email_s_
    extern const f_string_static_t f_iki_vocabulary_0001_email_s;
  #endif // _di_f_iki_vocabulary_0001_email_s_

  #ifndef _di_f_iki_vocabulary_0001_name_s_
    extern const f_string_static_t f_iki_vocabulary_0001_name_s;
  #endif // _di_f_iki_vocabulary_0001_name_s_

  #ifndef _di_f_iki_vocabulary_0001_phone_s_
    extern const f_string_static_t f_iki_vocabulary_0001_phone_s;
  #endif // _di_f_iki_vocabulary_0001_phone_s_

  #ifndef _di_f_iki_vocabulary_0001_quote_s_
    extern const f_string_static_t f_iki_vocabulary_0001_quote_s;
  #endif // _di_f_iki_vocabulary_0001_quote_s_

  #ifndef _di_f_iki_vocabulary_0001_uri_s_
    extern const f_string_static_t f_iki_vocabulary_0001_uri_s;
  #endif // _di_f_iki_vocabulary_0001_uri_s_

  #ifndef _di_f_iki_vocabulary_0001_url_s_
    extern const f_string_static_t f_iki_vocabulary_0001_url_s;
  #endif // _di_f_iki_vocabulary_0001_url_s_

  #ifndef _di_f_iki_vocabulary_0001_urn_s_
    extern const f_string_static_t f_iki_vocabulary_0001_urn_s;
  #endif // _di_f_iki_vocabulary_0001_urn_s_

  #ifndef _di_f_iki_vocabulary_0001_variable_s_
    extern const f_string_static_t f_iki_vocabulary_0001_variable_s;
  #endif // _di_f_iki_vocabulary_0001_variable_s_
#endif // _di_f_iki_vocabulary_0001_s_

#ifndef _di_f_iki_vocabulary_0002_s_
  #define F_iki_vocabulary_0002_context_s   "context"
  #define F_iki_vocabulary_0002_define_s    "define"
  #define F_iki_vocabulary_0002_parameter_s "parameter"

  #define F_iki_vocabulary_0002_context_s_length   7
  #define F_iki_vocabulary_0002_define_s_length    6
  #define F_iki_vocabulary_0002_parameter_s_length 9

  #ifndef _di_f_iki_vocabulary_0002_context_s_
    extern const f_string_static_t f_iki_vocabulary_0002_context_s;
  #endif // _di_f_iki_vocabulary_0002_context_s_

  #ifndef _di_f_iki_vocabulary_0002_define_s_
    extern const f_string_static_t f_iki_vocabulary_0002_define_s;
  #endif // _di_f_iki_vocabulary_0002_define_s_

  #ifndef _di_f_iki_vocabulary_0002_parameter_s_
    extern const f_string_static_t f_iki_vocabulary_0002_parameter_s;
  #endif // _di_f_iki_vocabulary_0002_parameter_s_
#endif // _di_f_iki_vocabulary_0002_s_

/**
 * This containing all of the IKI data.
 *
 * Properties:
 *   - content:    A set of ranges representing the content (variable value) list to store the content associated with the found vocabulary name.
 *   - delimits:   A delimits array representing where delimits exist within the buffer.
 *   - variable:   A set of ranges representing the entire vocabulary (variable name), content, and the syntax.
 *   - vocabulary: A set of ranges representing the vocabulary (variable name) list to store the found vocabulary name.
 */
#ifndef _di_f_iki_data_t_
  typedef struct {
    f_ranges_t content;
    f_number_unsigneds_t delimits;
    f_ranges_t variable;
    f_ranges_t vocabulary;
  } f_iki_data_t;

  #define f_iki_data_t_initialize { \
    f_ranges_t_initialize, \
    f_number_unsigneds_t_initialize, \
    f_ranges_t_initialize, \
    f_ranges_t_initialize, \
  }

  #define macro_f_iki_data_t_initialize_1(content, delimits, variable, vocabulary) { content, delimits, variable, vocabulary }

  #define macro_f_iki_data_t_clear(data) \
    f_ranges_t_clear(data.content) \
    f_number_unsigneds_t_clear(data.delimits) \
    f_ranges_t_clear(data.variable) \
    f_ranges_t_clear(data.vocabulary)
#endif // _di_f_iki_data_t_

/**
 * An array of f_iki_data_t.
 *
 * Properties:
 *   - array: The array of IKI data.
 *   - size:  Total amount of allocated space.
 *   - used:  Total number of allocated spaces used.
 */
#ifndef _di_f_iki_datas_t_
  typedef struct {
    f_iki_data_t *array;

    f_number_unsigned_t size;
    f_number_unsigned_t used;
  } f_iki_datas_t;

  #define f_iki_datas_t_initialize { 0, 0, 0 }

  #define macro_f_iki_datas_t_initialize_1(content, size, used) { array, size, used }
  #define macro_f_iki_datas_t_initialize_2(array, length) { array, length, length }
#endif // _di_f_iki_datas_t_

/**
 * An array of f_iki_datas_t.
 *
 * Properties:
 *   - array: The array of IKI datas.
 *   - size:  Total amount of allocated space.
 *   - used:  Total number of allocated spaces used.
 */
#ifndef _di_f_iki_datass_t_
  typedef struct {
    f_iki_datas_t *array;

    f_number_unsigned_t size;
    f_number_unsigned_t used;
  } f_iki_datass_t;

  #define f_iki_datass_t_initialize { 0, 0, 0 }

  #define macro_f_iki_datass_t_initialize_1(content, size, used) { array, size, used }
  #define macro_f_iki_datass_t_initialize_2(array, length) { array, length, length }
#endif // _di_f_iki_datass_t_

/**
 * This containing all of the EKI data (extended IKI data).
 *
 * Properties:
 *   - content:     A set of ranges representing the content (variable value) list to store the content associated with the found vocabulary name.
 *   - delimits:    A delimits array representing where delimits exist within the buffer.
 *   - variable:    A set of ranges representing the entire vocabulary (variable name), content, and the syntax.
 *   - vocabularys: A set of ranges representing the vocabularies (variable names) list to store each found vocabulary name.
 */
#ifndef _di_f_iki_eki_t_
  typedef struct {
    f_ranges_t content;
    f_number_unsigneds_t delimits;
    f_ranges_t variable;
    f_rangess_t vocabularys;
  } f_iki_eki_t;

  #define f_iki_eki_t_initialize { \
    f_ranges_t_initialize, \
    f_number_unsigneds_t_initialize, \
    f_ranges_t_initialize, \
    f_rangess_t_initialize, \
  }

  #define macro_f_iki_eki_t_initialize_1(content, delimits, variable, vocabularys) { content, delimits, variable, vocabularys }

  #define macro_f_iki_eki_t_clear(data) \
    f_ranges_t_clear(data.content) \
    f_number_unsigneds_t_clear(data.delimits) \
    f_ranges_t_clear(data.variable) \
    f_ranges_t_clear(data.vocabulary)
#endif // _di_f_iki_eki_t_

/**
 * An array of f_iki_eki_t.
 *
 * Properties:
 *   - array: The array of IKI EKI.
 *   - size:  Total amount of allocated space.
 *   - used:  Total number of allocated spaces used.
 */
#ifndef _di_f_iki_ekis_t_
  typedef struct {
    f_iki_eki_t *array;

    f_number_unsigned_t size;
    f_number_unsigned_t used;
  } f_iki_ekis_t;

  #define f_iki_ekis_t_initialize { 0, 0, 0 }

  #define macro_f_iki_ekis_t_initialize_1(content, size, used) { array, size, used }
  #define macro_f_iki_ekis_t_initialize_2(array, length) { array, length, length }
#endif // _di_f_iki_ekis_t_

/**
 * An array of f_iki_ekis_t.
 *
 * Properties:
 *   - array: The array of IKI EKIs.
 *   - size:  Total amount of allocated space.
 *   - used:  Total number of allocated spaces used.
 */
#ifndef _di_f_iki_ekiss_t_
  typedef struct {
    f_iki_ekis_t *array;

    f_number_unsigned_t size;
    f_number_unsigned_t used;
  } f_iki_ekiss_t;

  #define f_iki_ekiss_t_initialize { 0, 0, 0 }

  #define macro_f_iki_ekiss_t_initialize_1(content, size, used) { array, size, used }
  #define macro_f_iki_ekiss_t_initialize_2(array, length) { array, length, length }
#endif // _di_f_iki_ekiss_t_

#ifdef __cplusplus
} // extern "C"
#endif

#endif // _F_iki_common_h
