#include "time.h"

#ifdef __cplusplus
extern "C" {
#endif

#ifndef _di_f_time_calendar_string_
  f_status_t f_time_calendar_string(const time_t time, f_string_dynamic_t * const destination) {
    #ifndef _di_level_0_parameter_checking_
      if (!destination) return F_status_set_error(F_parameter);
    #endif // _di_level_0_parameter_checking_

    f_char_t string[F_time_calendar_string_length_d] = { 0 };

    if (!ctime_r(&time, string)) {
      if (errno == EOVERFLOW) return F_status_set_error(F_buffer_overflow);

      return F_status_set_error(F_failure);
    }

    const f_status_t status = f_string_append(string, strnlen(string, F_time_calendar_string_length_d), destination);
    if (F_status_is_error(status)) return status;

    return F_okay;
  }
#endif // _di_f_time_calendar_string_

#ifndef _di_f_time_calendar_string_part_
  f_status_t f_time_calendar_string_part(const f_time_part_t part, f_string_dynamic_t * const destination) {
    #ifndef _di_level_0_parameter_checking_
      if (!destination) return F_status_set_error(F_parameter);
    #endif // _di_level_0_parameter_checking_

    f_char_t string[F_time_calendar_string_length_d] = { 0 };

    if (!asctime_r(&part, string)) {
      if (errno == EOVERFLOW) return F_status_set_error(F_buffer_overflow);

      return F_status_set_error(F_failure);
    }

    const f_status_t status = f_string_append(string, strnlen(string, F_time_calendar_string_length_d), destination);
    if (F_status_is_error(status)) return status;

    return F_okay;
  }
#endif // _di_f_time_calendar_string_part_

#ifndef _di_f_time_clock_get_
  f_status_t f_time_clock_get(const clockid_t code, f_time_spec_t * const time) {
    #ifndef _di_level_0_parameter_checking_
      if (!time) return F_status_set_error(F_parameter);
    #endif // _di_level_0_parameter_checking_

    if (clock_gettime(code, time)) {
      if (errno == EACCES) return F_status_set_error(F_access_denied);
      if (errno == EFAULT) return F_status_set_error(F_buffer);
      if (errno == ENODEV) return F_status_set_error(F_device_not);
      if (errno == ENOTSUP) return F_status_set_error(F_support_not);
      if (errno == EINVAL) return F_status_set_error(F_parameter);
      if (errno == EOVERFLOW) return F_status_set_error(F_buffer_overflow);
      if (errno == EPERM) return F_status_set_error(F_prohibited);

      return F_status_set_error(F_failure);
    }

    return F_okay;
  }
#endif // _di_f_time_clock_get_

#ifndef _di_f_time_clock_precision_
  f_status_t f_time_clock_precision(const clockid_t code, f_time_spec_t * const precision) {
    #ifndef _di_level_0_parameter_checking_
      if (!precision) return F_status_set_error(F_parameter);
    #endif // _di_level_0_parameter_checking_

    if (clock_getres(code, precision)) {
      if (errno == EACCES) return F_status_set_error(F_access_denied);
      if (errno == EFAULT) return F_status_set_error(F_buffer);
      if (errno == ENODEV) return F_status_set_error(F_device_not);
      if (errno == ENOTSUP) return F_status_set_error(F_support_not);
      if (errno == EINVAL) return F_status_set_error(F_parameter);
      if (errno == EOVERFLOW) return F_status_set_error(F_buffer_overflow);
      if (errno == EPERM) return F_status_set_error(F_prohibited);

      return F_status_set_error(F_failure);
    }

    return F_okay;
  }
#endif // _di_f_time_clock_precision_

#ifndef _di_f_time_clock_set_
  f_status_t f_time_clock_set(const clockid_t code, const f_time_spec_t time) {

    if (clock_settime(code, &time)) {
      if (errno == EACCES) return F_status_set_error(F_access_denied);
      if (errno == EFAULT) return F_status_set_error(F_buffer);
      if (errno == ENODEV) return F_status_set_error(F_device_not);
      if (errno == ENOTSUP) return F_status_set_error(F_support_not);
      if (errno == EINVAL) return F_status_set_error(F_parameter);
      if (errno == EOVERFLOW) return F_status_set_error(F_buffer_overflow);
      if (errno == EPERM) return F_status_set_error(F_prohibited);

      return F_status_set_error(F_failure);
    }

    return F_okay;
  }
#endif // _di_f_time_clock_set_

#ifndef _di_f_time_epoch_get_
  f_status_t f_time_epoch_get(time_t * const time_value) {
    #ifndef _di_level_0_parameter_checking_
      if (!time_value) return F_status_set_error(F_parameter);
    #endif // _di_level_0_parameter_checking_

    const time_t result = time(0);

    if (result == ((time_t) -1)) {
      if (errno == EFAULT) return F_status_set_error(F_buffer);

      return F_status_set_error(F_failure);
    }

    *time_value = result;

    return F_okay;
  }
#endif // _di_f_time_epoch_get_

#ifndef _di_f_time_local_get_
  f_status_t f_time_local_get(const time_t time, f_time_part_t * const destination) {
    #ifndef _di_level_0_parameter_checking_
      if (!destination) return F_status_set_error(F_parameter);
    #endif // _di_level_0_parameter_checking_

    if (!localtime_r(&time, destination)) {
      if (errno == EOVERFLOW) return F_status_set_error(F_buffer_overflow);

      return F_status_set_error(F_failure);
    }

    return F_okay;
  }
#endif // _di_f_time_local_get_

#ifndef _di_f_time_of_day_get_
  f_status_t f_time_of_day_get(f_time_value_t * const time) {
    #ifndef _di_level_0_parameter_checking_
      if (!time) return F_status_set_error(F_parameter);
    #endif // _di_level_0_parameter_checking_

    if (gettimeofday(time, 0) == -1) {
      if (errno == EFAULT) return F_status_set_error(F_buffer);
      if (errno == EINVAL) return F_status_set_error(F_parameter);
      if (errno == EPERM) return F_status_set_error(F_prohibited);

      return F_status_set_error(F_failure);
    }

    return F_okay;
  }
#endif // _di_f_time_of_day_get_

#ifndef _di_f_time_of_day_set_
  f_status_t f_time_of_day_set(const f_time_value_t time) {

    if (settimeofday(&time, 0) == -1) {
      if (errno == EFAULT) return F_status_set_error(F_buffer);
      if (errno == EINVAL) return F_status_set_error(F_parameter);
      if (errno == EPERM) return F_status_set_error(F_prohibited);

      return F_status_set_error(F_failure);
    }

    return F_okay;
  }
#endif // _di_f_time_of_day_set_

#ifndef _di_f_time_sleep_spec_
  f_status_t f_time_sleep_spec(const f_time_spec_t time, f_time_spec_t * const remaining) {

    if (nanosleep(&time, remaining) == -1) {
      if (errno == EFAULT) return F_status_set_error(F_buffer);
      if (errno == EINTR) return F_interrupt;
      if (errno == EINVAL) return F_status_set_error(F_parameter);

      return F_status_set_error(F_failure);
    }

    return F_okay;
  }
#endif // _di_f_time_sleep_spec_

#ifndef _di_f_time_spec_millisecond_
  extern f_status_t f_time_spec_millisecond(const f_number_unsigned_t second, const f_number_unsigned_t millisecond, f_time_spec_t * const time) {
    #ifndef _di_level_0_parameter_checking_
      if (!time) return F_status_set_error(F_parameter);
    #endif // _di_level_0_parameter_checking_

    if (((time_t) second) < 0) {
      if ((long) millisecond < 0) {
        time->tv_sec = 0;
        time->tv_nsec = 0;
      }
      else {
        time->tv_sec = (long) millisecond > 1000 ? ((long) millisecond) / 1000 : 0;
        time->tv_nsec = (time->tv_sec ? ((long) millisecond) % 1000 : ((long) millisecond)) * 1000;
      }

      return F_status_set_error(F_number_underflow);
    }

    if ((long) millisecond < 0) {
      time->tv_sec = (time_t) second;
      time->tv_nsec = 0;

      return F_status_set_error(F_number_underflow);
    }

    if (millisecond) {
      time->tv_sec = ((long) millisecond) / 1000;
      time->tv_nsec = (time->tv_sec ? ((long) millisecond) % 1000 : ((long) millisecond)) * 1000;
    }
    else {
      time->tv_sec = 0;
      time->tv_nsec = 0;
    }

    time->tv_sec += (time_t) second;

    return time->tv_sec < (time_t) second ? F_status_set_error(F_number_overflow) : F_okay;
  }
#endif // _di_f_time_spec_millisecond_

#ifndef _di_f_time_spec_nanosecond_
  extern f_status_t f_time_spec_nanosecond(const f_number_unsigned_t second, const f_number_unsigned_t nanosecond, f_time_spec_t * const time) {
    #ifndef _di_level_0_parameter_checking_
      if (!time) return F_status_set_error(F_parameter);
    #endif // _di_level_0_parameter_checking_

    if (((time_t) second) < 0) {
      time->tv_sec = 0;
      time->tv_nsec = ((long) nanosecond) < 0 ? 0 : (long) nanosecond;

      return F_status_set_error(F_number_underflow);
    }

    time->tv_sec = (time_t) second;

    if ((long) nanosecond < 0) {
      time->tv_nsec = 0;

      return F_status_set_error(F_number_underflow);
    }

    time->tv_nsec = (long) nanosecond;

    // If tv_nsec is 1 second or greater, then increment second.
    if (time->tv_nsec >= 1000000000) {
      ++(time->tv_sec);

      time->tv_nsec -= 1000000000;

      if (time->tv_sec < (time_t) second) return F_status_set_error(F_number_overflow);
    }

    return F_okay;
  }
#endif // _di_f_time_spec_nanosecond_

#ifndef _di_f_time_utc_get_
  f_status_t f_time_utc_get(const time_t time, f_time_part_t * const destination) {
    #ifndef _di_level_0_parameter_checking_
      if (!destination) return F_status_set_error(F_parameter);
    #endif // _di_level_0_parameter_checking_

    if (!gmtime_r(&time, destination)) {
      if (errno == EOVERFLOW) return F_status_set_error(F_buffer_overflow);

      return F_status_set_error(F_failure);
    }

    return F_okay;
  }
#endif // _di_f_time_utc_get_

#ifdef __cplusplus
} // extern "C"
#endif
