#include "directory.h"
#include "private-directory.h"

#ifdef __cplusplus
extern "C" {
#endif

#ifndef _di_fl_directory_create_
  f_status_t fl_directory_create(const f_string_static_t path, const mode_t mode) {

    if (!path.used) return F_data_not;

    f_status_t status = f_directory_exists(path);
    if (F_status_is_error(status)) return status;
    if (status == F_true) return F_directory_found;

    {
      f_number_unsigned_t at_tree = 0;
      f_number_unsigned_t at_path = 0;
      f_string_static_t tree = f_string_static_t_initialize;

      f_char_t tree_string[path.used + 1];
      tree.string = tree_string;
      tree_string[path.used] = 0;

      for (; at_path < path.used; ++at_path) {

        if (at_path && path.string[at_path] == f_path_separator_s.string[0]) {
          memcpy(tree.string + at_tree, path.string + at_tree, sizeof(f_char_t) * (at_path - at_tree));
          tree.string[at_path] = 0;
          tree.used = at_path;

          status = f_directory_exists(tree);
          if (F_status_is_error(status)) return status;

          if (status == F_false || status == F_file_found_not) {
            status = f_directory_create(tree, mode);
            if (F_status_is_error(status)) return status;
          }

          at_tree = at_path;
        }
      } // for
    }

    return f_directory_create(path, mode);
  }
#endif // _di_fl_directory_create_

#ifndef _di_fl_directory_do_
  void fl_directory_do(const f_string_static_t path, f_directory_recurse_do_t * const recurse) {
    #ifndef _di_level_1_parameter_checking_
      if (!recurse) return;
    #endif // _di_level_1_parameter_checking_

    recurse->path.used = 0;
    recurse->path_cache.used = 0;
    recurse->path_top = &path;
    recurse->depth = 0;
    recurse->state.status = F_okay;
    recurse->state.data = (void *) recurse;

    // Guarantee initialization even for parameter checking failures.
    #ifndef _di_level_1_parameter_checking_
      if (!recurse->action) {
        recurse->state.status = F_status_set_error(F_parameter);

        private_inline_fl_directory_do_handle(recurse, path, f_directory_recurse_do_flag_top_e);
        if (F_status_is_error(recurse->state.status)) return;
      }
    #endif // _di_level_1_parameter_checking_

    if (!path.used) {
      recurse->state.status = F_data_not;

      private_inline_fl_directory_do_handle(recurse, path, f_directory_recurse_do_flag_top_e | f_directory_recurse_do_flag_path_e);
      if (F_status_is_error(recurse->state.status)) return;
    }

    recurse->state.status = f_string_dynamic_append_nulless(path, &recurse->path);

    if (F_status_is_error_not(recurse->state.status)) {

      // Do not allow trailing path separators in the string's length calculation, except root directory '/'.
      for (; recurse->path.used; --recurse->path.used) {
        if (recurse->path.string[recurse->path.used - 1] != f_path_separator_s.string[0]) break;
      } // for

      recurse->path.string[recurse->path.used] = 0;

      recurse->state.status = f_directory_exists(path);

      if (recurse->state.status == F_false) {
        recurse->state.status = F_status_set_error(F_directory_not);
      }
    }

    if (F_status_is_error(recurse->state.status)) {
      private_inline_fl_directory_do_handle(recurse, path, f_directory_recurse_do_flag_top_e | f_directory_recurse_do_flag_path_e);
      if (F_status_is_error(recurse->state.status)) return;
    }

    if ((recurse->flag & f_directory_recurse_do_flag_top_e) && (recurse->flag & f_directory_recurse_do_flag_before_e)) {
      recurse->state.status = F_okay;

      recurse->action((void *) recurse, path, f_directory_recurse_do_flag_top_e | f_directory_recurse_do_flag_path_e | f_directory_recurse_do_flag_before_e);

      if (F_status_is_error(recurse->state.status)) {
        private_inline_fl_directory_do_handle(recurse, path, f_directory_recurse_do_flag_top_e | f_directory_recurse_do_flag_path_e | f_directory_recurse_do_flag_before_e);
        if (F_status_is_error(recurse->state.status)) return;
      }

      if (recurse->state.status == F_done) {
        recurse->state.status = F_okay;

        return;
      }
    }

    if (recurse->depth_max) {
      recurse->depth = 1;

      private_fl_directory_do_recurse(recurse);

      recurse->path.used = path.used;
      recurse->depth = 0;

      if (F_status_is_error(recurse->state.status)) {
        private_inline_fl_directory_do_handle(recurse, path, f_directory_recurse_do_flag_top_e | f_directory_recurse_do_flag_path_e | f_directory_recurse_do_flag_before_e);
        if (F_status_is_error(recurse->state.status)) return;
      }
    }
    else {
      recurse->state.status = f_directory_exists(path);

      if (F_status_is_error_not(recurse->state.status) && recurse->state.status != F_true) {
        if (recurse->state.status == F_false) {
          recurse->state.status = F_status_set_error(F_directory_not);
        }
        else {
          recurse->state.status = F_status_set_error(recurse->state.status);
        }
      }

      if (F_status_is_error(recurse->state.status)) {
        private_inline_fl_directory_do_handle(recurse, path, f_directory_recurse_do_flag_top_e | f_directory_recurse_do_flag_directory_e);
        if (F_status_is_error(recurse->state.status)) return;
      }

      recurse->state.status = F_okay;

      recurse->action((void *) recurse, path, f_directory_recurse_do_flag_top_e | f_directory_recurse_do_flag_directory_e);

      if (F_status_is_error(recurse->state.status)) {
        private_inline_fl_directory_do_handle(recurse, path, f_directory_recurse_do_flag_top_e | f_directory_recurse_do_flag_directory_e);
        if (F_status_is_error(recurse->state.status)) return;
      }

      if (recurse->state.status == F_done) {
        recurse->state.status = F_okay;

        return;
      }
    }

    if ((recurse->flag & f_directory_recurse_do_flag_top_e) && (recurse->flag & f_directory_recurse_do_flag_after_e)) {
      recurse->state.status = F_okay;

      recurse->action((void *) recurse, path, f_directory_recurse_do_flag_top_e | f_directory_recurse_do_flag_path_e | f_directory_recurse_do_flag_after_e);
    }

    if (F_status_is_error(recurse->state.status)) {
      private_inline_fl_directory_do_handle(recurse, path, f_directory_recurse_do_flag_top_e | f_directory_recurse_do_flag_path_e | f_directory_recurse_do_flag_after_e);
      if (F_status_is_error(recurse->state.status)) return;
    }

    recurse->state.status = F_okay;
  }
#endif // _di_fl_directory_do_

#ifndef _di_fl_directory_list_
  f_status_t fl_directory_list(const f_string_static_t path, int (*filter)(const struct dirent *), int (*sort)(const struct dirent **, const struct dirent **), const bool dereference, f_directory_listing_t * const listing) {
    #ifndef _di_level_1_parameter_checking_
      if (!listing) return F_status_set_error(F_parameter);
    #endif // _di_level_1_parameter_checking_

    if (!path.used) return F_data_not;

    return private_fl_directory_list(path, filter, sort, dereference, listing);
  }
#endif // _di_fl_directory_list_

#ifndef _di_fl_directory_path_pop_
  f_status_t fl_directory_path_pop(f_string_static_t * const path) {
    #ifndef _di_level_1_parameter_checking_
      if (!path) return F_status_set_error(F_parameter);
    #endif // _di_level_1_parameter_checking_

    if (!path->used) return F_data_not;

    const bool null_terminated = path->string[path->used] == 0;
    bool first_nulless = F_false;

    f_number_unsigned_t i = path->used - 1;
    f_number_unsigned_t j = 0;

    f_status_t status = F_okay;

    for (; i > 0; --i) {

      if (!path->string[i]) continue;

      status = f_utf_is_control(path->string + i, path->used - i);
      if (status == F_true) continue;

      if (F_status_is_error(status)) {
        if (F_status_set_fine(status) == F_complete_not_utf) continue;

        return status;
      }

      if (first_nulless) {
        if (path->string[i] == f_path_separator_s.string[0]) {
          if (null_terminated) {
            path->string[i + 1] = 0;
            path->used = i + 2;
          }
          else {
            path->used = i + 1;
          }

          return F_okay;
        }
      }
      else {
        first_nulless = F_true;

        for (j = i; j > 0; --j) {
          if (!path->string[j]) continue;

          status = f_utf_is_control(path->string + j, path->used - j);
          if (status == F_true) continue;

          if (F_status_is_error(status)) {
            if (F_status_set_fine(status) == F_complete_not_utf) continue;

            return status;
          }

          if (path->string[j] != f_path_separator_s.string[0]) {
            i = j + 1;

            break;
          }
        } // for
      }
    } // for

    path->used = 0;

    return F_okay;
  }
#endif // _di_fl_directory_path_pop_

#ifndef _di_fl_directory_path_push_
  f_status_t fl_directory_path_push(const f_string_static_t source, f_string_dynamic_t * const destination) {
    #ifndef _di_level_1_parameter_checking_
      if (!destination) return F_status_set_error(F_parameter);
    #endif // _di_level_1_parameter_checking_

    if (!source.used) return F_data_not;

    return private_fl_directory_path_push(source, destination);
  }
#endif // _di_fl_directory_path_push_

#ifdef __cplusplus
} // extern "C"
#endif
