#include "embedded_list.h"
#include "../private-fss.h"
#include "../private-fss-list.h"

#ifdef __cplusplus
extern "C" {
#endif

#ifndef _di_fl_fss_embedded_list_content_read_
  void fl_fss_embedded_list_content_read(const f_string_static_t buffer, f_range_t * const range, f_fss_nest_t * const found, f_number_unsigneds_t * const delimits, f_ranges_t * const comments, f_state_t * const state) {
    #ifndef _di_level_1_parameter_checking_
      if (!state) return;

      if (!range || !found || !delimits || !comments || !state->data) {
        state->status = F_status_set_error(F_parameter);

        return;
      }
    #endif // _di_level_1_parameter_checking_

    f_fss_skip_past_delimit(buffer, range, state);
    if (F_status_is_error(state->status) || state->status == F_data_not || state->status == F_okay_eos || state->status == F_okay_stop) return;

    fl_fss_data_embedded_list_t * const cache = (fl_fss_data_embedded_list_t *) state->data;

    if (!cache->positions || !cache->objects || !cache->slashes) {
      state->status = F_status_set_error(F_parameter);

      return;
    }

    cache->positions->used = 0;
    cache->objects->used = 0;
    cache->slashes->used = 0;

    state->status = f_memory_array_increase(state->step_small, sizeof(f_fss_nest_t), (void **) &found->depth, &found->used, &found->size);

    if (F_status_is_error_not(state->status)) {
      state->status = f_memory_array_increase(state->step_small, sizeof(f_range_t), (void **) &cache->objects->array, &cache->objects->used, &cache->objects->size);
    }

    if (F_status_is_error_not(state->status)) {
      state->status = f_memory_array_increase(state->step_small, sizeof(f_number_unsigned_t), (void **) &cache->positions->array, &cache->positions->used, &cache->positions->size);
    }

    if (F_status_is_error_not(state->status)) {
      state->status = f_memory_array_increase(state->step_small, sizeof(f_number_unsigned_t), (void **) &cache->slashes->array, &cache->slashes->used, &cache->slashes->size);
    }

    if (F_status_is_error(state->status)) return;

    const f_number_unsigned_t delimits_used = delimits->used;
    const f_number_unsigned_t comments_used = comments->used;

    f_number_unsigned_t depth = 0;
    f_number_unsigned_t position = 0;
    f_number_unsigned_t position_previous = range->start;
    f_number_unsigned_t line_start = range->start;
    f_number_unsigned_t newline_last = range->start;
    f_number_unsigned_t comment_delimit = 0;
    f_number_unsigned_t slash_first = 0;
    f_number_unsigned_t slash_last = 0;
    f_number_unsigned_t before_list_open = position_previous;
    f_number_unsigned_t close_location = 0;

    // 0x0 = is false for all, 0x1 = is first graph, 0x2 = is delimited comment, 0x4 = is open, 0x8 = is object, 0x10 = is delimited close.
    uint8_t is_state = 0x1;

    // Initialize depth 1 start position.
    // Positions_start.used is used as a max depth (such that cache->positions->used == max depth + 1).
    cache->positions->array[0] = range->start;
    cache->positions->used = 1;

    cache->slashes->array[0] = 0;
    cache->slashes->used = 1;

    while (range->start <= range->stop && range->start < buffer.used) {

      if (state->interrupt) {
        state->interrupt((void *) state, 0);
        if (F_status_set_fine(state->status) == F_interrupt) break;
      }

      if (buffer.string[range->start] == f_fss_eol_s.string[0]) {
        if (is_state & 0x2) {
          state->status = f_memory_array_increase(state->step_small, sizeof(f_number_unsigned_t), (void **) &delimits->array, &delimits->used, &delimits->size);
          if (F_status_is_error(state->status)) break;

          delimits->array[delimits->used++] = comment_delimit;
        }

        newline_last = position_previous = range->start++;
        line_start = range->start;
        is_state = 0x1;
      }
      else if (buffer.string[range->start] == f_fss_slash_s.string[0] && (is_state & 0x1)) {
        slash_first = slash_last = position_previous = range->start++;
        cache->slashes->array[depth] = 1;

        for (; range->start <= range->stop && range->start < buffer.used && (buffer.string[range->start] == f_fss_placeholder_s.string[0] || buffer.string[range->start] == f_fss_slash_s.string[0]); ++range->start) {

          if (state->interrupt) {
            state->interrupt((void *) state, 0);
            if (F_status_set_fine(state->status) == F_interrupt) break;
          }

          position_previous = range->start;

          if (buffer.string[position_previous] == f_fss_slash_s.string[0]) {
            slash_last = position_previous;
            ++cache->slashes->array[depth];
          }
        } // for

        if (F_status_is_error(state->status) || state->status == F_interrupt || range->start >= buffer.used || range->start > range->stop) break;

        // All slashes for an open are delimited (because it could represent a slash in the object name).
        // For example 'object {' = valid open, name 'object', 'object \{' represents 'object {', 'object \\{' = valid open, name 'object \', 'object \\\{' represents 'object \{', etc..
        // Only the first slash before a close is delimited, all others are maintained.
        // For example '}' = valid close, '\}' represents '}', '\\}' represents '\}', '\\\}' represents '\\}', '\\\\}' represents '\\\}', and so on..
        // When slash is odd and a (delimited) valid open/close is found, then save delimited positions and continue.
        if (buffer.string[range->start] == f_fss_eol_s.string[0]) {
          if (is_state & 0x2) {
            state->status = f_memory_array_increase(state->step_small, sizeof(f_number_unsigned_t), (void **) &delimits->array, &delimits->used, &delimits->size);
            if (F_status_is_error(state->status)) break;

            delimits->array[delimits->used++] = comment_delimit;
          }

          newline_last = position_previous = range->start++;
          line_start = range->start;
          is_state = 0x1;
        }
        else if (buffer.string[range->start] == f_fss_embedded_list_open_s.string[0] || buffer.string[range->start] == f_fss_embedded_list_close_s.string[0]) {
          before_list_open = position_previous;
          is_state &= ~0x15;

          if (buffer.string[range->start] == f_fss_embedded_list_open_s.string[0]) {
            is_state |= 0x4;
          }

          position_previous = range->start++;

          while (range->start <= range->stop && range->start < buffer.used) {

            if (state->interrupt) {
              state->interrupt((void *) state, 0);
              if (F_status_set_fine(state->status) == F_interrupt) break;
            }

            if (buffer.string[range->start] == f_fss_eol_s.string[0]) break;

            if (buffer.string[range->start] != f_fss_placeholder_s.string[0]) {
              if (f_fss_is_space(buffer, *range, state) == F_false) break;
            }

            position_previous = range->start;

            f_utf_buffer_increment(buffer, range, 1);
            if (F_status_is_error(state->status)) break;
          } // while

          if (F_status_is_error(state->status) || state->status == F_interrupt || range->start >= buffer.used || range->start > range->stop) break;

          // This is a valid object open/close that has been delimited, save the slash delimit positions.
          if (buffer.string[range->start] == f_fss_eol_s.string[0]) {
            newline_last = range->start;
            line_start = newline_last + 1;

            if (is_state & 0x4) {
              if (cache->slashes->array[depth] % 2 == 0) {
                is_state |= 0x8;
              }
              else {
                is_state &= ~0x8;
              }

              range->start = slash_first;

              state->status = f_memory_array_increase_by((cache->slashes->array[depth] / 2) + 1, sizeof(f_number_unsigned_t), (void **) &delimits->array, &delimits->used, &delimits->size);
              if (F_status_is_error(state->status)) break;

              // Apply slash delimits, only slashes and placeholders should be present.
              while (cache->slashes->array[depth]) {

                if (buffer.string[range->start] == f_fss_slash_s.string[0]) {
                  if (cache->slashes->array[depth] % 2 == 1) {
                    delimits->array[delimits->used++] = range->start;
                  }

                  --cache->slashes->array[depth];
                }

                // Delimit slashes and placeholders are required to be in the ASCII range.
                position_previous = range->start++;
              } // while

              // When slashes are even, the object is valid and needs to be processed.
              if (is_state & 0x8) {
                if (++depth >= cache->objects->size) {
                  state->status = f_memory_array_resize(depth + 2, sizeof(f_range_t), (void **) &cache->objects->array, &cache->objects->used, &cache->objects->size);
                }

                if (depth >= cache->positions->size) {
                  if (F_status_is_error_not(state->status)) {
                    state->status = f_memory_array_resize(depth + 2, sizeof(f_number_unsigned_t), (void **) &cache->positions->array, &cache->positions->used, &cache->positions->size);
                  }
                }

                if (depth >= cache->slashes->size) {
                  if (F_status_is_error_not(state->status)) {
                    state->status = f_memory_array_resize(depth + 2, sizeof(f_number_unsigned_t), (void **) &cache->slashes->array, &cache->slashes->used, &cache->slashes->size);
                  }
                }

                if (F_status_is_error(state->status)) break;

                cache->objects->array[depth].start = line_start;
                cache->objects->array[depth].stop = before_list_open;
                cache->positions->array[depth] = newline_last + 1;
                cache->slashes->array[depth] = 0;

                if (cache->objects->used < depth) {
                  cache->objects->used = depth;
                }

                if (cache->positions->used < depth) {
                  cache->positions->used = depth;
                }

                if (cache->slashes->used < depth) {
                  cache->slashes->used = depth;
                }
              }
            }
            else {
              is_state |= 0x10;
            }

            if (is_state & 0x12) {
              state->status = f_memory_array_increase(state->step_small, sizeof(f_number_unsigned_t), (void **) &delimits->array, &delimits->used, &delimits->size);
              if (F_status_is_error(state->status)) break;

              delimits->array[delimits->used++] = slash_last;
            }

            range->start = line_start;
            is_state = 0x1;
          }
        }
        else if (buffer.string[range->start] == f_fss_comment_s.string[0] && (is_state & 0x1)) {
          is_state |= 0x2;
          is_state &= ~0x1;
          comment_delimit = slash_first;
        }
        else {
          is_state &= ~0x1;
        }
      }
      else if (buffer.string[range->start] == f_fss_embedded_list_open_s.string[0]) {
        is_state &= ~0x1;
        before_list_open = position_previous;
        position_previous = range->start;

        state->status = f_utf_buffer_increment(buffer, range, 1);
        if (F_status_is_error(state->status)) break;

        while (range->start <= range->stop && range->start < buffer.used) {

          if (state->interrupt) {
            state->interrupt((void *) state, 0);
            if (F_status_set_fine(state->status) == F_interrupt) break;
          }

          if (buffer.string[range->start] == f_fss_eol_s.string[0]) break;

          if (buffer.string[range->start] != f_fss_placeholder_s.string[0]) {
            if (f_fss_is_space(buffer, *range, state) == F_false) break;
          }

          position_previous = range->start;

          f_utf_buffer_increment(buffer, range, 1);
          if (F_status_is_error(state->status)) break;
        } // while

        if (F_status_is_error(state->status) || state->status == F_interrupt || range->start >= buffer.used || range->start > range->stop) break;

        if (buffer.string[range->start] == f_fss_eol_s.string[0]) {
          if (++depth >= cache->objects->size) {
            state->status = f_memory_array_resize(depth + 2, sizeof(f_range_t), (void **) &cache->objects->array, &cache->objects->used, &cache->objects->size);
          }

          if (depth >= cache->positions->size) {
            if (F_status_is_error_not(state->status)) {
              state->status = f_memory_array_resize(depth + 2, sizeof(f_number_unsigned_t), (void **) &cache->positions->array, &cache->positions->used, &cache->positions->size);
            }
          }

          if (depth >= cache->slashes->size) {
            if (F_status_is_error_not(state->status)) {
              state->status = f_memory_array_resize(depth + 2, sizeof(f_number_unsigned_t), (void **) &cache->slashes->array, &cache->slashes->used, &cache->slashes->size);
            }
          }

          if (F_status_is_error(state->status)) break;

          cache->objects->array[depth].start = line_start;
          cache->objects->array[depth].stop = before_list_open;
          cache->positions->array[depth] = range->start + 1;
          cache->slashes->array[depth] = 0;

          if (cache->objects->used <= depth) {
            cache->objects->used = depth + 1;
          }

          if (cache->positions->used <= depth) {
            cache->positions->used = depth + 1;
          }

          if (cache->slashes->used <= depth) {
            cache->slashes->used = depth + 1;
          }

          if (is_state & 0x2) {
            state->status = f_memory_array_increase(state->step_small, sizeof(f_number_unsigned_t), (void **) &delimits->array, &delimits->used, &delimits->size);
            if (F_status_is_error(state->status)) break;

            delimits->array[delimits->used++] = comment_delimit;
          }

          newline_last = range->start;
          line_start = newline_last + 1;
          is_state = 0x1;
        }
        else {

          // No valid object close found, seek until EOL.
          f_fss_seek_to_eol(buffer, range, state);
          if (F_status_is_error(state->status)) break;

          if (is_state & 0x2) {
            state->status = f_memory_array_increase(state->step_small, sizeof(f_number_unsigned_t), (void **) &delimits->array, &delimits->used, &delimits->size);
            if (F_status_is_error(state->status)) break;

            delimits->array[delimits->used++] = comment_delimit;
          }

          newline_last = range->start;
          line_start = newline_last + 1;
          is_state = 0x1;
        }
      }
      else if (buffer.string[range->start] == f_fss_embedded_list_close_s.string[0] && (is_state & 0x1)) {
        close_location = range->start;

        while (range->start <= range->stop && range->start < buffer.used) {

          if (state->interrupt) {
            state->interrupt((void *) state, 0);
            if (F_status_set_fine(state->status) == F_interrupt) break;
          }

          position_previous = range->start;

          state->status = f_utf_buffer_increment(buffer, range, 1);
          if (F_status_is_error(state->status) || buffer.string[range->start] == f_fss_eol_s.string[0]) break;

          if (buffer.string[range->start] != f_fss_placeholder_s.string[0]) {
            if (f_fss_is_space(buffer, *range, state) == F_false || F_status_is_error(state->status)) break;
          }
        } // while

        if (F_status_is_error(state->status) || state->status == F_interrupt || range->start >= buffer.used || range->start > range->stop) break;

        if (buffer.string[range->start] == f_fss_eol_s.string[0]) {
          if (depth >= found->used) {
            state->status = f_memory_array_resize(depth + 2, sizeof(f_fss_nest_t), (void **) &found->depth, &found->used, &found->size);
          }

          if (found->depth[depth].used >= found->depth[depth].size) {
            if (F_status_is_error_not(state->status)) {
              state->status = f_memory_array_increase(state->step_small, sizeof(f_fss_item_t), (void **) &found->depth[depth].array, &found->depth[depth].used, &found->depth[depth].size);
            }
          }

          position = found->depth[depth].used;

          if (found->depth[depth].array[position].content.size != 1) {
            if (F_status_is_error_not(state->status)) {
              state->status = f_memory_array_resize(1, sizeof(f_range_t), (void **) &found->depth[depth].array[position].content.array, &found->depth[depth].array[position].content.used, &found->depth[depth].array[position].content.size);
            }
          }

          if (F_status_is_error(state->status)) break;

          if (depth) {
            found->depth[depth].array[position].parent = found->depth[depth - 1].used;

            // Only assign object positions for nested objects.
            found->depth[depth].array[position].object.start = cache->objects->array[depth].start;
            found->depth[depth].array[position].object.stop = cache->objects->array[depth].stop;
          }

          // The Content is empty when line_start is the same as the start position.
          if (line_start == cache->positions->array[depth]) {
            found->depth[depth].array[position].content.array[0].start = 1;
            found->depth[depth].array[position].content.array[0].stop = 0;
          }
          else {
            found->depth[depth].array[position].content.array[0].start = cache->positions->array[depth];
            found->depth[depth].array[position].content.array[0].stop = newline_last;
          }

          found->depth[depth].array[position].content.used = 1;

          if (position >= found->depth[depth].used) {
            found->depth[depth].used++;
          }

          if (found->used < depth + 1) {
            found->used = depth + 1;
          }

          if (is_state & 0x2) {
            state->status = f_memory_array_increase(state->step_small, sizeof(f_number_unsigned_t), (void **) &delimits->array, &delimits->used, &delimits->size);
            if (F_status_is_error(state->status)) break;

            delimits->array[delimits->used++] = comment_delimit;
          }

          found->depth[depth].array[position].close.start = line_start;
          found->depth[depth].array[position].close.stop = close_location;

          newline_last = range->start;
          line_start = newline_last + 1;
          is_state = 0x1;

          if (!depth) {
            state->status = f_utf_buffer_increment(buffer, range, 1);

            if (F_status_is_error(state->status))  {
              delimits->used = delimits_used;
              comments->used = comments_used;

              return;
            }

            if (range->start >= buffer.used) {
              state->status = F_okay_eos;
            } else if (range->start > range->stop) {
              state->status = F_okay_stop;
            }
            else {
              state->status = F_fss_found_content;
            }

            return;
          }

          --depth;
        }
        else {

          // No valid object close found, seek until EOL.
          while (range->start <= range->stop && range->start < buffer.used) {

            if (state->interrupt) {
              state->interrupt((void *) state, 0);
              if (F_status_set_fine(state->status) == F_interrupt) break;
            }

            if (buffer.string[range->start] == f_fss_eol_s.string[0]) {
              if (is_state & 0x2) {
                state->status = f_memory_array_increase(state->step_small, sizeof(f_number_unsigned_t), (void **) &delimits->array, &delimits->used, &delimits->size);
                if (F_status_is_error(state->status)) break;

                delimits->array[delimits->used++] = comment_delimit;
              }

              newline_last = range->start;
              line_start = newline_last + 1;
              is_state = 0x1;

              break;
            }

            position_previous = range->start;

            state->status = f_utf_buffer_increment(buffer, range, 1);
            if (F_status_is_error(state->status)) break;
          } // while

          if (F_status_is_error(state->status) || state->status == F_interrupt) break;
        }
      }
      else if (buffer.string[range->start] == f_fss_comment_s.string[0] && (is_state & 0x1)) {

        // The newline_last is initialized to the range->start, which may not actually be a new line.
        position = (buffer.string[newline_last] == f_string_eol_s.string[0]) ? newline_last + 1 : newline_last;

        f_fss_seek_to_eol(buffer, range, state);
        if (F_status_is_error(state->status)) break;

        if (range->start > range->stop || range->start >= buffer.used) {
          --range->start;
        }
        else {
          if (is_state & 0x2) {
            state->status = f_memory_array_increase(state->step_small, sizeof(f_number_unsigned_t), (void **) &delimits->array, &delimits->used, &delimits->size);
            if (F_status_is_error(state->status)) break;

            delimits->array[delimits->used++] = comment_delimit;
          }

          newline_last = range->start;
          line_start = newline_last + 1;
          is_state = 0x1;
        }

        state->status = f_memory_array_increase(state->step_small, sizeof(f_range_t), (void **) &comments->array, &comments->used, &comments->size);
        if (F_status_is_error(state->status)) break;

        comments->array[comments->used].start = position;
        comments->array[comments->used++].stop = range->start++;
      }
      else if (buffer.string[range->start] != f_fss_eol_s.string[0]) {
        position_previous = range->start;

        if (is_state & 0x1) {
          if (f_fss_is_space(buffer, *range, state) == F_false) {
            is_state &= ~0x1;
          }
        }

        if (F_status_is_error_not(state->status)) {
          state->status = f_utf_buffer_increment(buffer, range, 1);
        }

        if (F_status_is_error(state->status)) break;
      }
    } // while

    delimits->used = delimits_used;
    comments->used = comments_used;

    if (F_status_is_error_not(state->status) && state->status != F_interrupt) {
      if (range->start > range->stop) {
        state->status = F_status_set_error(depth ? F_end_not_nest_stop : F_end_not_stop);
      }
      else {
        state->status = F_status_set_error(depth ? F_end_not_nest_eos : F_end_not_eos);
      }
    }
  }
#endif // _di_fl_fss_embedded_list_content_read_

#ifndef _di_fl_fss_embedded_list_content_write_
  void fl_fss_embedded_list_content_write(const f_string_static_t content, const uint8_t complete, const f_string_static_t * const prepend, const f_ranges_t * const ignore, f_range_t * const range, f_string_dynamic_t * const destination, f_state_t * const state) {
    #ifndef _di_level_1_parameter_checking_
      if (!state) return;

      if (!range || !destination) {
        state->status = F_status_set_error(F_parameter);

        return;
      }
    #endif // _di_level_1_parameter_checking_

    f_fss_skip_past_delimit(content, range, state);
    if (F_status_is_error(state->status)) return;
    if (state->status == F_data_not) return;

    if (state->status == F_okay_eos) {
      state->status = F_data_not_eos;
    }
    else if (state->status == F_okay_stop) {
      state->status = F_data_not_stop;
    }

    if (range->start > range->stop || range->start >= content.used) {
      if (complete == f_fss_complete_full_e || complete == f_fss_complete_full_trim_e || complete == f_fss_complete_end_e) {
        const f_status_t status = f_memory_array_increase_by(2, sizeof(f_char_t), (void **) &destination->string, &destination->used, &destination->size);

        if (F_status_is_error(status)) {
          state->status = status;

          return;
        }

        destination->string[destination->used++] = f_fss_embedded_list_close_s.string[0];
        destination->string[destination->used++] = f_fss_embedded_list_close_end_s.string[0];
      }

      return;
    }

    const f_number_unsigned_t destination_used = destination->used;

    uint8_t is_comment = F_false;
    uint8_t ends_on_eol = F_false;
    uint8_t has_graph = F_false;
    uint8_t do_prepend = prepend ? F_true : F_false;
    uint8_t width = 0;

    f_number_unsigned_t i = 0;
    f_number_unsigned_t r = 0;
    f_number_unsigned_t slash_count = 0;
    f_number_unsigned_t start = 0;

    while (range->start <= range->stop && range->start < content.used) {

      if (state->interrupt) {
        state->interrupt((void *) state, 0);
        if (F_status_set_fine(state->status) == F_interrupt) break;
      }

      if (content.string[range->start] == f_fss_slash_s.string[0] && !is_comment) {
        slash_count = 1;

        if (do_prepend) {
          state->status = f_string_dynamic_append(*prepend, destination);
          if (F_status_is_error(state->status)) break;

          do_prepend = F_false;
        }

        state->status = f_memory_array_increase(state->step_large, sizeof(f_char_t), (void **) &destination->string, &destination->used, &destination->size);
        if (F_status_is_error(state->status)) break;

        destination->string[destination->used++] = content.string[range->start];

        for (++range->start; range->start <= range->stop && range->start < content.used; ++range->start) {

          if (state->interrupt) {
            state->interrupt((void *) state, 0);
            if (F_status_set_fine(state->status) == F_interrupt) {
              destination->used = destination_used;

              return;
            }
          }

          if (content.string[range->start] == f_fss_placeholder_s.string[0]) continue;
          if (content.string[range->start] != f_fss_slash_s.string[0]) break;

          state->status = f_memory_array_increase(state->step_large, sizeof(f_char_t), (void **) &destination->string, &destination->used, &destination->size);

          if (F_status_is_error(state->status)) {
            destination->used = destination_used;

            return;
          }

          destination->string[destination->used++] = f_fss_slash_s.string[0];
          ++slash_count;
        } // for

        if (content.string[range->start] == f_fss_embedded_list_open_s.string[0] || content.string[range->start] == f_fss_embedded_list_close_s.string[0]) {
          start = range->start++;

          f_fss_skip_past_space(content, range, state);
          if (F_status_is_error(state->status)) break;

          if (range->start >= content.used || range->start > range->stop || content.string[range->start] == f_fss_eol_s.string[0]) {

            // Increase by total slashes + 1 embedded list open/close.
            state->status = f_memory_array_increase_by(slash_count + 2, sizeof(f_char_t), (void **) &destination->string, &destination->used, &destination->size);
            if (F_status_is_error(state->status)) break;

            if (content.string[range->start] == f_fss_embedded_list_open_s.string[0]) {
              while (--slash_count) {
                destination->string[destination->used++] = f_fss_slash_s.string[0];
              } // while
            }

            destination->string[destination->used++] = f_fss_slash_s.string[0];
            destination->string[destination->used++] = content.string[start];

            range->start = start + 1;

            private_fl_fss_list_write_add_until_end(content, range, destination, state);
            if (F_status_is_error(state->status)) break;

            if (content.string[range->start] != f_fss_eol_s.string[0]) {
              has_graph = F_true;
            }

            continue;
          }

          // Increase by character at "start" and possible newline.
          state->status = f_memory_array_increase_by(2, sizeof(f_char_t), (void **) &destination->string, &destination->used, &destination->size);
          if (F_status_is_error(state->status)) break;

          destination->string[destination->used++] = content.string[start];

          if (content.string[range->start] == f_fss_eol_s.string[0]) {
            destination->string[destination->used++] = f_fss_eol_s.string[0];
            ends_on_eol = F_true;
          }
          else {
            ends_on_eol = F_false;
          }

          range->start = start + 1;

          continue;
        }
      }
      else if ((content.string[range->start] == f_fss_embedded_list_open_s.string[0] || (!has_graph && content.string[range->start] == f_fss_embedded_list_close_s.string[0])) && !is_comment) {
        start = range->start++;

        if (do_prepend) {
          state->status = f_string_dynamic_append(*prepend, destination);
          if (F_status_is_error(state->status)) break;

          do_prepend = F_false;
        }

        has_graph = F_true;

        f_fss_skip_past_space(content, range, state);
        if (F_status_is_error(state->status)) break;

        if (range->start >= content.used || range->start > range->stop || content.string[range->start] == f_fss_eol_s.string[0]) {

          if (content.string[range->start] == f_fss_eol_s.string[0] && prepend) {
            do_prepend = F_true;
            ends_on_eol = F_true;
          }
          else {
            ends_on_eol = F_false;
          }

          if (ignore && ignore->used) {

            for (r = 0; r < ignore->used; ++r) {
              if (start >= ignore->array[r].start && start <= ignore->array[r].stop) break;
            } // for

            if (r < ignore->used) {
              state->status = f_memory_array_increase(state->step_large, sizeof(f_char_t), (void **) &destination->string, &destination->used, &destination->size);
              if (F_status_is_error(state->status)) break;

              destination->string[destination->used++] = content.string[start];
              range->start = start + 1;

              continue;
            }
          }

          // Increase by slash and extended list open and possible newline.
          state->status = f_memory_array_increase_by(3, sizeof(f_char_t), (void **) &destination->string, &destination->used, &destination->size);
          if (F_status_is_error(state->status)) break;

          destination->string[destination->used++] = f_fss_slash_s.string[0];
          destination->string[destination->used++] = content.string[start];

          range->start = start + 1;

          private_fl_fss_list_write_add_until_end(content, range, destination, state);
          if (F_status_is_error(state->status)) break;

          continue;
        }

        state->status = f_memory_array_increase(state->step_large, sizeof(f_char_t), (void **) &destination->string, &destination->used, &destination->size);
        if (F_status_is_error(state->status)) break;

        destination->string[destination->used++] = content.string[start];
        range->start = start + 1;

        continue;
      }
      else if (content.string[range->start] == f_fss_comment_s.string[0] && !has_graph) {
        is_comment = F_true;
      }
      else if (content.string[range->start] == f_fss_eol_s.string[0]) {
        has_graph = F_false;
        is_comment = F_false;
      }
      else if (f_fss_is_graph(content, *range, state) == F_false) {
        if (F_status_is_error(state->status)) break;
      }
      else {
        has_graph = F_true;
      }

      if (content.string[range->start] != f_fss_placeholder_s.string[0]) {
        if (do_prepend) {
          state->status = f_string_dynamic_append(*prepend, destination);
          if (F_status_is_error(state->status)) break;

          do_prepend = F_false;
        }

        if (content.string[range->start] == f_fss_eol_s.string[0] && prepend) {
          do_prepend = F_true;
          ends_on_eol = F_true;
        }
        else {
          ends_on_eol = F_false;
        }

        width = macro_f_utf_byte_width(content.string[range->start]);

        state->status = f_memory_array_increase_by(width, sizeof(f_char_t), (void **) &destination->string, &destination->used, &destination->size);
        if (F_status_is_error(state->status)) break;

        for (i = 0; i < width; ++i) {
          destination->string[destination->used++] = content.string[range->start + i];
        } // for
      }

      state->status = f_utf_buffer_increment(content, range, 1);
      if (F_status_is_error(state->status)) break;
    } // while

    if (F_status_is_error(state->status)) {
      destination->used = destination_used;

      return;
    }

    if (complete == f_fss_complete_full_e || complete == f_fss_complete_full_trim_e || complete == f_fss_complete_end_e) {
      state->status = f_memory_array_increase_by(3, sizeof(f_char_t), (void **) &destination->string, &destination->used, &destination->size);
      if (F_status_is_error(state->status)) return;

      if (!ends_on_eol) {
        destination->string[destination->used++] = f_fss_eol_s.string[0];
      }

      destination->string[destination->used++] = f_fss_embedded_list_close_s.string[0];
      destination->string[destination->used++] = f_fss_embedded_list_close_end_s.string[0];
    }

    if (range->start > range->stop) {
      state->status = F_okay_stop;
    }
    else if (range->start >= content.used) {
      state->status = F_okay_eos;
    }
    else {
      state->status = F_okay;
    }
  }
#endif // _di_fl_fss_embedded_list_content_write_

#ifndef _di_fl_fss_embedded_list_object_read_
  void fl_fss_embedded_list_object_read(const f_string_static_t buffer, f_range_t * const range, f_range_t * const found, f_number_unsigneds_t * const delimits, f_state_t * const state) {
    #ifndef _di_level_1_parameter_checking_
      if (!state) return;

      if (!range || !found || !delimits) {
        state->status = F_status_set_error(F_parameter);

        return;
      }
    #endif // _di_level_1_parameter_checking_

    return private_fl_fss_list_object_read(f_fss_embedded_list_open_s, buffer, range, found, delimits, state);
  }
#endif // _di_fl_fss_embedded_list_object_read_

#ifndef _di_fl_fss_embedded_list_object_write_
  void fl_fss_embedded_list_object_write(const f_string_static_t object, const uint8_t complete, f_range_t * const range, f_string_dynamic_t * const destination, f_state_t * const state) {
    #ifndef _di_level_1_parameter_checking_
      if (!state) return;

      if (!range || !destination) {
        state->status = F_status_set_error(F_parameter);

        return;
      }
    #endif // _di_level_1_parameter_checking_

    f_fss_skip_past_delimit(object, range, state);
    if (F_status_is_error(state->status)) return;
    if (state->status == F_data_not) return;

    if (state->status == F_okay_eos) {
      state->status = F_data_not_eos;
    }
    else if (state->status == F_okay_stop) {
      state->status = F_data_not_stop;
    }

    if (state->status == F_data_not_stop || state->status == F_data_not_eos) {
      if (complete == f_fss_complete_partial_e || complete == f_fss_complete_partial_trim_e || complete == f_fss_complete_full_e || complete == f_fss_complete_full_trim_e) {
        {
          const f_status_t status = f_memory_array_increase_by(2, sizeof(f_char_t), (void **) &destination->string, &destination->used, &destination->size);

          if (F_status_is_error(status)) {
            state->status = status;

            return;
          }
        }

        destination->string[destination->used++] = f_fss_embedded_list_open_s.string[0];

        if (complete == f_fss_complete_full_e || complete == f_fss_complete_full_trim_e) {
          destination->string[destination->used++] = f_fss_embedded_list_open_end_s.string[0];
        }
      }

      return;
    }

    // Ensure that there is room for a slash delimit, the object open character, and the end of line character.
    state->status = f_memory_array_increase_by(4, sizeof(f_char_t), (void **) &destination->string, &destination->used, &destination->size);
    if (F_status_is_error(state->status)) return;

    const f_number_unsigned_t destination_used = destination->used;
    f_number_unsigned_t slash_count = 0;

    bool ends_on_space = F_false;

    uint8_t width = 0;

    // Find the first graph character.
    while (range->start <= range->stop && range->start < object.used) {

      if (state->interrupt) {
        state->interrupt((void *) state, 0);
        if (F_status_set_fine(state->status) == F_interrupt) break;
      }

      if (object.string[range->start] == f_fss_comment_s.string[0]) {

        // When a comment is found, escape it.
        state->status = f_memory_array_increase(state->step_large, sizeof(f_char_t), (void **) &destination->string, &destination->used, &destination->size);
        if (F_status_is_error(state->status)) break;

        destination->string[destination->used++] = f_fss_slash_s.string[0];

        break;
      }

      if (f_fss_is_graph(object, *range, state) == F_false) {
        if (F_status_is_error(state->status)) break;
      }
      else {
        break;
      }

      // Objects will not have leading white spaces, but having this does not result in an invalid object, so just write the provided spaces.
      if (object.string[range->start] != f_fss_placeholder_s.string[0]) {
        if (object.string[range->start] == f_fss_eol_s.string[0]) {
          state->status = F_status_set_error(F_okay_eol);

          break;
        }

        if (f_fss_is_space(object, *range, state) == F_false) {
          if (F_status_is_error(state->status)) break;
        }
        else {
          if (object.string[range->start] == f_fss_eol_s.string[0]) {
            state->status = F_status_set_error(F_okay_eol);

            break;
          }

          width = macro_f_utf_byte_width(object.string[range->start]);

          state->status = f_memory_array_increase_by(width, sizeof(f_char_t), (void **) &destination->string, &destination->used, &destination->size);
          if (F_status_is_error(state->status)) break;

          memcpy(destination->string + destination->used, object.string + range->start, width);

          destination->used += width;
        }
      }

      state->status = f_utf_buffer_increment(object, range, 1);
      if (F_status_is_error(state->status)) break;
    } // while

    if (F_status_is_error(state->status)) {
      destination->used = destination_used;

      return;
    }

    while (range->start <= range->stop && range->start < object.used) {

      if (state->interrupt) {
        state->interrupt((void *) state, 0);
        if (F_status_set_fine(state->status) == F_interrupt) break;
      }

      if (object.string[range->start] == f_fss_slash_s.string[0]) {
        slash_count = 1;

        for (++range->start; range->start <= range->stop && range->start < object.used; ++range->start) {

          if (state->interrupt) {
            state->interrupt((void *) state, 0);

            if (F_status_set_fine(state->status) == F_interrupt) {
              destination->used = destination_used;

              return;
            }
          }

          if (object.string[range->start] == f_fss_placeholder_s.string[0]) continue;
          if (object.string[range->start] != f_fss_slash_s.string[0]) break;

          ++slash_count;
        } // for

        if (range->start > range->stop || range->start >= object.used) {

          // Slashes at the end of the object must be delimited to avoid delimiting the object close character.
          slash_count *= 2;
        }

        state->status = f_memory_array_increase_by(slash_count, sizeof(f_char_t), (void **) &destination->string, &destination->used, &destination->size);
        if (F_status_is_error(state->status)) break;

        while (--slash_count) {
          destination->string[destination->used++] = f_fss_slash_s.string[0];
        } // while

        if (range->start > range->stop || range->start >= object.used) {
          ends_on_space = F_false;

          break;
        }
      }

      if (object.string[range->start] != f_fss_placeholder_s.string[0]) {
        if (object.string[range->start] == f_fss_eol_s.string[0]) {
          state->status = F_status_set_error(F_okay_eol);

          break;
        }

        if (f_fss_is_space(object, *range, state) == F_false) {
          if (F_status_is_error(state->status)) break;
        }
        else {
          ends_on_space = F_true;

          if (object.string[range->start] == f_fss_eol_s.string[0]) {
            state->status = F_status_set_error(F_okay_eol);

            break;
          }
        }

        width = macro_f_utf_byte_width(object.string[range->start]);

        state->status = f_memory_array_increase_by(width, sizeof(f_char_t), (void **) &destination->string, &destination->used, &destination->size);
        if (F_status_is_error(state->status)) break;

        memcpy(destination->string + destination->used, object.string + range->start, width);

        destination->used += width;
      }

      state->status = f_utf_buffer_increment(object, range, 1);
      if (F_status_is_error(state->status)) break;
    } // while

    if (F_status_is_error(state->status)) {
      destination->used = destination_used;

      return;
    }

    if (complete == f_fss_complete_partial_e || complete == f_fss_complete_partial_trim_e || complete == f_fss_complete_full_e || complete == f_fss_complete_full_trim_e || complete == f_fss_complete_trim_e) {
      if (complete == f_fss_complete_full_trim_e || complete == f_fss_complete_trim_e) {
        private_fl_fss_list_write_object_trim(destination_used, destination, state);

        if (F_status_is_error(state->status)) {
          destination->used = destination_used;

          return;
        }

        // Prevent a space from being added post-trimming.
        ends_on_space = F_true;
      }

      if (complete != f_fss_complete_trim_e) {
        state->status = f_memory_array_increase_by(3, sizeof(f_char_t), (void **) &destination->string, &destination->used, &destination->size);

        if (F_status_is_error(state->status)) {
          destination->used = destination_used;

          return;
        }

        if (!ends_on_space) {
          destination->string[destination->used++] = f_fss_space_s.string[0];
        }

        destination->string[destination->used++] = f_fss_embedded_list_open_s.string[0];

        if (complete == f_fss_complete_full_e || complete == f_fss_complete_full_trim_e) {
          destination->string[destination->used++] = f_fss_embedded_list_open_end_s.string[0];
        }
      }
    }

    if (range->start > range->stop) {
      state->status = F_okay_stop;
    }
    else if (range->start >= object.used) {
      state->status = F_okay_eos;
    }
    else {
      state->status = F_okay;
    }
  }
#endif // _di_fl_fss_embedded_list_object_write_

#ifdef __cplusplus
} // extern "C"
#endif
