#include "fss.h"
#include "private-fss-list.h"

#ifdef __cplusplus
extern "C" {
#endif

#if !defined(_di_fl_fss_basic_list_object_read_) || !defined(_di_fl_fss_embedded_list_object_read_) || !defined(_di_fl_fss_extended_list_object_read_)
  void private_fl_fss_list_object_read(const f_string_static_t list_open, const f_string_static_t buffer, f_range_t * const range, f_range_t * const found, f_number_unsigneds_t * const delimits, f_state_t * const state) {

    f_fss_skip_past_space(buffer, range, state);
    if (F_status_is_error(state->status) || state->status == F_data_not) return;

    if (state->status == F_okay_eol) {

      // Move the start position to after the EOL.
      ++range->start;
      state->status = F_fss_found_object_not;

      return;
    }

    if (state->status == F_okay_eos || state->status == F_okay_stop) {
      state->status = state->status == F_okay_eos ? F_data_not_eos : F_data_not_stop;

      return;
    }

    // Return found nothing if this line only contains white space and delimit placeholders.
    if (buffer.string[range->start] == f_fss_eol_s.string[0]) {

      // Move the start position to after the EOL.
      ++range->start;
      state->status = F_fss_found_object_not;

      return;
    }

    // Ignore all comment lines.
    if (buffer.string[range->start] == f_fss_comment_s.string[0]) {
      f_fss_seek_to_eol(buffer, range, state);
      if (F_status_is_error(state->status)) return;

      if (state->status == F_okay_eos || state->status == F_okay_stop) {
        state->status = state->status == F_okay_eos ? F_data_not_eos : F_data_not_stop;

        return;
      }

      // Move the start position to after the EOL.
      ++range->start;
      state->status = F_fss_found_object_not;

      return;
    }

    const f_number_unsigned_t delimits_used = delimits->used;

    f_number_unsigned_t slash_first = 0;
    f_number_unsigned_t slash_count = 0;
    f_number_unsigned_t start = 1;
    f_number_unsigned_t stop = 0;

    uint8_t graph_first = F_true;

    found->start = range->start;

    // Identify where the object ends.
    while (range->start <= range->stop && range->start < buffer.used && buffer.string[range->start] != f_fss_eol_s.string[0]) {

      if (state->interrupt) {
        state->interrupt((void *) state, 0);

        if (F_status_is_error(state->status)) {
          delimits->used = delimits_used;

          return;
        }
      }

      if (buffer.string[range->start] == f_fss_slash_s.string[0]) {
        slash_first = range->start++;
        slash_count = 1;

        for (; range->start <= range->stop && range->start < buffer.used && (buffer.string[range->start] == f_fss_placeholder_s.string[0] || buffer.string[range->start] == f_fss_slash_s.string[0]); ++range->start) {

          if (state->interrupt) {
            state->interrupt((void *) state, 0);

            if (F_status_is_error(state->status)) {
              delimits->used = delimits_used;

              return;
            }
          }

          if (buffer.string[range->start] == f_fss_slash_s.string[0]) ++slash_count;
        } // for

        if (range->start >= buffer.used || range->start > range->stop) {
          delimits->used = delimits_used;

          if (range->start >= buffer.used) {
            found->stop = buffer.used - 1;
            state->status = F_data_not_eos;
          }
          else {
            found->stop = range->stop;
            state->status = F_data_not_stop;
          }

          return;
        }

        if (buffer.string[range->start] == list_open.string[0]) {
          graph_first = F_false;
          stop = range->start++;

          while (range->start <= range->stop && range->start < buffer.used) {

            if (state->interrupt) {
              state->interrupt((void *) state, 0);

              if (F_status_is_error(state->status)) {
                delimits->used = delimits_used;

                return;
              }
            }

            if (buffer.string[range->start] == f_fss_eol_s.string[0] || f_fss_is_space(buffer, *range, state) == F_false) break;

            state->status = f_utf_buffer_increment(buffer, range, 1);

            if (F_status_is_error(state->status)) {
              delimits->used = delimits_used;

              return;
            }
          } // while

          if (range->start >= buffer.used || range->start > range->stop) {
            delimits->used = delimits_used;

            if (range->start >= buffer.used) {
              found->stop = buffer.used - 1;
              state->status = F_data_not_eos;
            }
            else {
              found->stop = range->stop;
              state->status = F_data_not_stop;
            }

            return;
          }

          if (buffer.string[range->start] == f_fss_eol_s.string[0]) {
            found->stop = range->start;
            range->start = slash_first;

            state->status = f_memory_array_increase_by((slash_count / 2) + 1, sizeof(f_number_unsigned_t), (void **) &delimits->array, &delimits->used, &delimits->size);

            if (F_status_is_error(state->status)) {
              range->start = found->stop;
              delimits->used = delimits_used;

              return;
            }

            if (slash_count % 2 == 0) {
              while (slash_count > 0) {

                if (buffer.string[range->start] == f_fss_slash_s.string[0]) {
                  if (slash_count % 2 == 1) {
                    delimits->array[delimits->used++] = range->start;
                  }

                  --slash_count;
                  ++range->start;
                }
                else {
                  state->status = f_utf_buffer_increment(buffer, range, 1);

                  if (F_status_is_error(state->status)) {
                    range->start = found->stop;
                    delimits->used = delimits_used;

                    return;
                  }
                }
              } // while

              range->start = found->stop + 1;
              found->start = stop;
              found->stop = stop;

              state->status = f_utf_buffer_decrement(buffer, found, 1);

              if (F_status_is_error(state->status)) {
                --range->start;
                delimits->used = delimits_used;

                return;
              }

              found->stop = found->start;
              found->start = start;
              state->status = F_fss_found_object;
            }
            else {
              range->start = found->stop + 1;
              state->status = F_fss_found_object_not;
            }

            return;
          }
        }
        else if (graph_first && buffer.string[range->start] == f_fss_comment_s.string[0]) {
          graph_first = F_false;
          start = slash_first;

          // Comments may only have white space before the '#', therefore only the first slash needs to be delimited.
          state->status = f_memory_array_increase(state->step_small, sizeof(f_number_unsigned_t), (void **) &delimits->array, &delimits->used, &delimits->size);

          if (F_status_is_error(state->status)) {
            range->start = found->stop;
            delimits->used = delimits_used;

            return;
          }

          delimits->array[delimits->used++] = slash_first;
          stop = range->start++;
        }
        else {
          graph_first = F_false;
          stop = range->start;
        }

        continue;
      }

      if (buffer.string[range->start] == list_open.string[0]) {
        ++range->start;

        while (range->start <= range->stop && range->start < buffer.used) {

          if (state->interrupt) {
            state->interrupt((void *) state, 0);

            if (F_status_is_error(state->status)) {
              delimits->used = delimits_used;

              return;
            }
          }

          if (buffer.string[range->start] == f_fss_eol_s.string[0] || f_fss_is_space(buffer, *range, state) == F_false) break;

          state->status = f_utf_buffer_increment(buffer, range, 1);

          if (F_status_is_error(state->status)) {
            delimits->used = delimits_used;

            return;
          }
        } // while

        if (range->start >= buffer.used) {
          found->stop = buffer.used - 1;
          state->status = F_okay_eos;

          return;
        }

        if (range->start > range->stop) {
          found->stop = range->stop;
          state->status = F_okay_stop;

          return;
        }

        if (buffer.string[range->start] == f_fss_eol_s.string[0]) {
          found->start = start;
          found->stop = stop + macro_f_utf_byte_width(buffer.string[stop]) - 1;
          ++range->start;
          state->status = F_fss_found_object;

          return;
        }

        if (graph_first) {
          graph_first = F_false;
          start = 1;
          stop = 0;
        }
        else {
          stop = range->start;
        }

        continue;
      }

      if (f_fss_is_space(buffer, *range, state) == F_false) {
        if (F_status_is_error(state->status)) {
          delimits->used = delimits_used;

          return;
        }

        if (graph_first) {
          graph_first = F_false;
          start = range->start;
        }

        stop = range->start;
      }

      state->status = f_utf_buffer_increment(buffer, range, 1);

      if (F_status_is_error(state->status)) {
        delimits->used = delimits_used;

        return;
      }
    } // while

    if (range->start >= buffer.used) {
      found->stop = buffer.used - 1;
      state->status = F_data_not_eos;

      return;
    }

    if (range->start > range->stop) {
      found->stop = range->stop;
      state->status = F_data_not_stop;

      return;
    }

    // Seek to the end of the line when no valid object is found.
    f_fss_seek_to_eol(buffer, range, state);

    if (F_status_is_error(state->status)) {
      delimits->used = delimits_used;
    }
    else {
      state->status = F_fss_found_object_not;

      // Move the start position to after the EOL.
      ++range->start;
    }
  }
#endif // !defined(_di_fl_fss_basic_list_object_read_) || !defined(_di_fl_fss_embedded_list_object_read_) || !defined(_di_fl_fss_extended_list_object_read_)

#if !defined(_di_fl_fss_basic_list_content_write_) || !defined(_di_fl_fss_extended_list_content_write_) || !defined(_di_fl_fss_embedded_list_content_write_)
  void private_fl_fss_list_write_add_until_end(const f_string_static_t buffer, f_range_t * const range, f_string_dynamic_t * const destination, f_state_t * const state) {

    state->status = F_okay;

    for (; range->start <= range->stop && range->start < buffer.used; ++range->start) {

      if (state->interrupt) {
        state->interrupt((void *) state, 0);
        if (F_status_set_fine(state->status) == F_interrupt) return;
      }

      if (buffer.string[range->start] == f_fss_placeholder_s.string[0]) continue;
      if (buffer.string[range->start] == f_fss_eol_s.string[0]) break;

      state->status = f_memory_array_increase(state->step_large, sizeof(f_char_t), (void **) &destination->string, &destination->used, &destination->size);
      if (F_status_is_error(state->status)) break;

      destination->string[destination->used++] = buffer.string[range->start];
    } // for
  }
#endif // !defined(_di_fl_fss_basic_list_content_write_) || !defined(_di_fl_fss_extended_list_content_write_) || !defined(_di_fl_fss_embedded_list_content_write_)

#if !defined(_di_fl_fss_basic_list_object_write_) || !defined(_di_fl_fss_extended_list_object_write_)
  void private_fl_fss_list_write_object_trim(const f_number_unsigned_t used_start, f_string_dynamic_t * const destination, f_state_t * const state) {

    f_range_t destination_range = macro_f_range_t_initialize_2(destination->used);
    f_number_unsigned_t i = 0;
    uint8_t width = 0;

    state->status = F_okay;

    for (; destination_range.start < destination->used; ++destination_range.start) {

      if (state->interrupt) {
        state->interrupt((void *) state, 0);
        if (F_status_set_fine(state->status) == F_interrupt) return;
      }

      if (destination->string[destination_range.start] == f_fss_placeholder_s.string[0]) continue;

      if (f_fss_is_space(*destination, destination_range, state) == F_false) {
        if (F_status_is_error(state->status)) {
          destination->used = used_start;

          return;
        }

        break;
      }

      width = macro_f_utf_byte_width(destination->string[destination_range.start]);

      for (i = 0; i < width; ++i) {
        destination->string[destination_range.start + i] = f_fss_placeholder_s.string[0];
      } // for
    } // for

    for (destination_range.start = destination->used - 1; destination_range.start > 0; --destination_range.start) {

      if (state->interrupt) {
        state->interrupt((void *) state, 0);
        if (F_status_set_fine(state->status) == F_interrupt) return;
      }

      if (destination->string[destination_range.start] == f_fss_placeholder_s.string[0]) {
        --destination->used;

        continue;
      }

      if (f_fss_is_space(*destination, destination_range, state) == F_false) {

        // When going backwards, getting incomplete UTF-8 sequences is not an error.
        if (F_status_set_fine(state->status) == F_complete_not_utf) continue;

        if (F_status_is_error(state->status)) {
          destination->used = used_start;

          return;
        }

        break;
      }

      destination->used -= macro_f_utf_byte_width(destination->string[destination_range.start]);
    } // for

    if (destination_range.start == 0) {
      if (f_fss_is_space(*destination, destination_range, state) == F_false) {
        if (F_status_is_error(state->status)) {
          destination->used = used_start;

          return;
        }
      }
      else {
        destination->used = 0;
      }
    }

    state->status = F_okay;
  }
#endif // !defined(_di_fl_fss_basic_list_object_write_) || !defined(_di_fl_fss_extended_list_object_write_)

#ifdef __cplusplus
} // extern "C"
#endif
