#include "test-iki.h"
#include "test-iki-eki_read.h"

#ifdef __cplusplus
extern "C" {
#endif

void test__fl_iki_eki_read__parameter_checking(void **state) {

  f_string_static_t buffer = f_string_static_t_initialize;
  f_range_t range = f_range_t_initialize;
  f_iki_eki_t data = f_iki_eki_t_initialize;
  f_state_t the_state = f_state_t_initialize;

  // The data parameter is not directly accessed by fl_iki_eki_read(), and so there is no parameter checking for that parameter.

  {
    fl_iki_eki_read(0, 0, 0, &the_state);

    assert_int_equal(F_status_set_fine(the_state.status), F_parameter);
  }

  {
    fl_iki_eki_read(&buffer, 0, 0, &the_state);

    assert_int_equal(F_status_set_fine(the_state.status), F_parameter);
  }

  {
    fl_iki_eki_read(&buffer, 0, &data, &the_state);

    assert_int_equal(F_status_set_fine(the_state.status), F_parameter);
  }

  {
    fl_iki_eki_read(&buffer, 0, &data, &the_state);

    assert_int_equal(F_status_set_fine(the_state.status), F_parameter);
  }

  {
    fl_iki_eki_read(0, &range, 0, &the_state);

    assert_int_equal(F_status_set_fine(the_state.status), F_parameter);
  }

  {
    fl_iki_eki_read(0, &range, &data, &the_state);

    assert_int_equal(F_status_set_fine(the_state.status), F_parameter);
  }
}

void test__fl_iki_eki_read__works(void **state) {

  f_string_static_t buffer = f_string_static_t_initialize;
  f_range_t range = f_range_t_initialize;
  f_iki_eki_t data = f_iki_eki_t_initialize;
  f_state_t the_state = f_state_t_initialize;

  {
    the_state.status = F_false;
    range.start = 0;
    range.stop = 2;
    buffer.used = 10;

    will_return(__wrap_f_iki_eki_read, false);
    will_return(__wrap_f_iki_eki_read, 1);
    will_return(__wrap_f_iki_eki_read, F_okay);

    will_return(__wrap_f_iki_eki_read, false);
    will_return(__wrap_f_iki_eki_read, 2);
    will_return(__wrap_f_iki_eki_read, F_okay);

    will_return(__wrap_f_iki_eki_read, false);
    will_return(__wrap_f_iki_eki_read, 3);
    will_return(__wrap_f_iki_eki_read, F_okay_stop);

    fl_iki_eki_read(&buffer, &range, &data, &the_state);

    assert_int_equal(the_state.status, F_okay_stop);
    assert_int_equal(range.start, range.stop + 1);
  }
}

#ifdef __cplusplus
} // extern "C"
#endif
