#include "test-iki.h"
#include "test-iki-write.h"

#ifdef __cplusplus
extern "C" {
#endif

void test__f_iki_eki_write__parameter_checking(void **state) {

  f_state_t state_data = f_state_t_initialize;
  f_string_dynamic_t destination = f_string_dynamic_t_initialize;
  const f_string_static_t empty = macro_f_string_static_t_initialize_1(f_string_empty_s.string, f_string_empty_s.size, f_string_empty_s.used);
  const f_string_statics_t vocabularys = f_string_statics_t_initialize;
  const f_string_static_t content = macro_f_string_static_t_initialize_1("content", 0, 7);

  state_data.status = F_known_not;

  {
    f_iki_eki_write(vocabularys, content, f_iki_syntax_quote_double_s, &destination, 0);

    // The state cannot be set to F_parameter because the state is not passed to the function.
    assert_int_equal(state_data.status, F_known_not);
  }

  {
    f_iki_eki_write(vocabularys, content, f_iki_syntax_quote_grave_s, &destination, 0);

    // The state cannot be set to F_parameter because the state is not passed to the function.
    assert_int_equal(state_data.status, F_known_not);
  }

  {
    f_iki_eki_write(vocabularys, content, f_iki_syntax_quote_single_s, &destination, 0);

    // The state cannot be set to F_parameter because the state is not passed to the function.
    assert_int_equal(state_data.status, F_known_not);
  }

  {
    f_iki_eki_write(vocabularys, content, f_iki_syntax_quote_double_s, 0, &state_data);

    assert_int_equal(state_data.status, F_status_set_error(F_parameter));
  }

  {
    f_iki_eki_write(vocabularys, content, empty, &destination, &state_data);

    assert_int_equal(state_data.status, F_status_set_error(F_parameter));
  }

  {
    f_iki_eki_write(vocabularys, content, content, &destination, &state_data);

    assert_int_equal(state_data.status, F_status_set_error(F_parameter));
  }
}

void test__f_iki_eki_write__returns_data_not(void **state) {

  f_state_t state_data = f_state_t_initialize;
  f_string_dynamic_t destination = f_string_dynamic_t_initialize;
  f_string_static_t empty_children_array[] = { f_string_empty_s };
  const f_string_statics_t empty_children = macro_f_string_statics_t_initialize_2(empty_children_array, 1);
  const f_string_statics_t empty_vocabs = f_string_statics_t_initialize;
  const f_string_static_t content = macro_f_string_static_t_initialize_1("content", 0, 7);

  {
    f_iki_eki_write(empty_vocabs, content, f_iki_syntax_quote_double_s, &destination, &state_data);

    assert_int_equal(F_status_set_fine(state_data.status), F_data_not);
  }

  {
    f_iki_eki_write(empty_vocabs, content, f_iki_syntax_quote_grave_s, &destination, &state_data);

    assert_int_equal(F_status_set_fine(state_data.status), F_data_not);
  }

  {
    f_iki_eki_write(empty_vocabs, content, f_iki_syntax_quote_single_s, &destination, &state_data);

    assert_int_equal(F_status_set_fine(state_data.status), F_data_not);
  }

  {
    f_iki_eki_write(empty_children, content, f_iki_syntax_quote_double_s, &destination, &state_data);

    assert_int_equal(F_status_set_fine(state_data.status), F_data_not);
  }

  {
    f_iki_eki_write(empty_children, content, f_iki_syntax_quote_grave_s, &destination, &state_data);

    assert_int_equal(F_status_set_fine(state_data.status), F_data_not);
  }

  {
    f_iki_eki_write(empty_children, content, f_iki_syntax_quote_single_s, &destination, &state_data);

    assert_int_equal(F_status_set_fine(state_data.status), F_data_not);
  }
}

void test__f_iki_eki_write__returns_syntax(void **state) {

  f_state_t state_data = f_state_t_initialize;
  f_string_dynamic_t destination = f_string_dynamic_t_initialize;
  f_string_static_t content = macro_f_string_static_t_initialize_1("content", 0, 7);
  f_string_static_t str_colon = macro_f_string_static_t_initialize_1(":", 0, 1);
  f_string_static_t str_space = macro_f_string_static_t_initialize_1(" ", 0, 1);
  f_string_static_t str_at = macro_f_string_static_t_initialize_1("@", 0, 1);
  f_string_static_t str_bracket_open = macro_f_string_static_t_initialize_1("[", 0, 1);
  f_string_static_t str_bracket_close = macro_f_string_static_t_initialize_1("]", 0, 1);
  f_string_static_t str_lots = macro_f_string_static_t_initialize_1("This * is [never] going && to be...>%'`\" valid!", 0, 47);

  f_string_static_t str_colon_array[] = { str_colon };
  f_string_static_t str_space_array[] = { str_space };
  f_string_static_t str_at_array[] = { str_at };
  f_string_static_t str_bracket_open_array[] = { str_bracket_open };
  f_string_static_t str_bracket_close_array[] = { str_bracket_close };
  f_string_static_t str_lots_array[] = { str_lots };

  const f_string_statics_t str_colon_vocabs = macro_f_string_statics_t_initialize_2(str_colon_array, 1);
  const f_string_statics_t str_space_vocabs = macro_f_string_statics_t_initialize_2(str_space_array, 1);
  const f_string_statics_t str_at_vocabs = macro_f_string_statics_t_initialize_2(str_at_array, 1);
  const f_string_statics_t str_bracket_open_vocabs = macro_f_string_statics_t_initialize_2(str_bracket_open_array, 1);
  const f_string_statics_t str_bracket_close_vocabs = macro_f_string_statics_t_initialize_2(str_bracket_close_array, 1);
  const f_string_statics_t str_lots_vocabs = macro_f_string_statics_t_initialize_2(str_lots_array, 1);

  {
    f_iki_eki_write(str_colon_vocabs, content, f_iki_syntax_quote_double_s, &destination, &state_data);

    assert_int_equal(F_status_set_fine(state_data.status), F_syntax);
  }

  {
    f_iki_eki_write(str_space_vocabs, content, f_iki_syntax_quote_double_s, &destination, &state_data);

    assert_int_equal(F_status_set_fine(state_data.status), F_syntax);
  }

  {
    f_iki_eki_write(str_at_vocabs, content, f_iki_syntax_quote_double_s, &destination, &state_data);

    assert_int_equal(F_status_set_fine(state_data.status), F_syntax);
  }

  {
    f_iki_eki_write(str_bracket_open_vocabs, content, f_iki_syntax_quote_double_s, &destination, &state_data);

    assert_int_equal(F_status_set_fine(state_data.status), F_syntax);
  }

  {
    f_iki_eki_write(str_bracket_close_vocabs, content, f_iki_syntax_quote_double_s, &destination, &state_data);

    assert_int_equal(F_status_set_fine(state_data.status), F_syntax);
  }

  {
    f_iki_eki_write(str_lots_vocabs, content, f_iki_syntax_quote_double_s, &destination, &state_data);

    assert_int_equal(F_status_set_fine(state_data.status), F_syntax);
  }
}

void test__f_iki_eki_write__works(void **state) {

  mock_unwrap = 1;

  f_state_t state_data = f_state_t_initialize;
  f_string_dynamic_t destination = f_string_dynamic_t_initialize;

  f_string_static_t empty = macro_f_string_static_t_initialize_1(f_string_empty_s.string, f_string_empty_s.size, f_string_empty_s.used);
  f_string_static_t str_a = macro_f_string_static_t_initialize_1("a", 0, 1);
  f_string_static_t str_b = macro_f_string_static_t_initialize_1("b", 0, 1);
  f_string_static_t str_iki = macro_f_string_static_t_initialize_1("iki", 0, 3);
  f_string_static_t str_has = macro_f_string_static_t_initialize_1("has_CHARS+-", 0, 11);

  f_string_static_t str_a_array[] = { str_a };
  f_string_static_t str_iki_array[] = { str_iki };
  f_string_static_t str_has_array[] = { str_has };

  const f_string_statics_t str_a_vocabs = macro_f_string_statics_t_initialize_2(str_a_array, 1);
  const f_string_statics_t str_iki_vocabs = macro_f_string_statics_t_initialize_2(str_iki_array, 1);
  const f_string_statics_t str_has_vocabs = macro_f_string_statics_t_initialize_2(str_has_array, 1);

  f_string_static_t str_content = macro_f_string_static_t_initialize_1("content", 0, 7);
  f_string_static_t str_content_quote_double = macro_f_string_static_t_initialize_1("\"content\"", 0, 9);
  f_string_static_t str_content_quote_grave = macro_f_string_static_t_initialize_1("`content`", 0, 9);
  f_string_static_t str_content_quote_single = macro_f_string_static_t_initialize_1("'content'", 0, 9);
  f_string_static_t str_quote_double = macro_f_string_static_t_initialize_1("quote\"s", 0, 7);
  f_string_static_t str_quote_grave = macro_f_string_static_t_initialize_1("quote`s", 0, 7);
  f_string_static_t str_quote_single = macro_f_string_static_t_initialize_1("quote's", 0, 7);
  f_string_static_t str_slash_1 = macro_f_string_static_t_initialize_1("slash\\", 0, 6);
  f_string_static_t str_slash_2 = macro_f_string_static_t_initialize_1("slash\\\\", 0, 7);
  f_string_static_t str_slash_3 = macro_f_string_static_t_initialize_1("slash\\\\\\", 0, 8);
  f_string_static_t str_slash_4 = macro_f_string_static_t_initialize_1("slash\\\\\\\\", 0, 9);

  #define _inline_macro_total_rows 198

  f_string_statics_t vocabularyss[] = {
    str_a_vocabs,
    str_a_vocabs,
    str_a_vocabs,
    str_a_vocabs,
    str_a_vocabs,
    str_a_vocabs,
    str_a_vocabs,
    str_a_vocabs,
    str_a_vocabs,
    str_a_vocabs,
    str_a_vocabs,
    str_iki_vocabs,
    str_iki_vocabs,
    str_iki_vocabs,
    str_iki_vocabs,
    str_iki_vocabs,
    str_iki_vocabs,
    str_iki_vocabs,
    str_iki_vocabs,
    str_iki_vocabs,
    str_iki_vocabs,
    str_iki_vocabs,
    str_has_vocabs,
    str_has_vocabs,
    str_has_vocabs,
    str_has_vocabs,
    str_has_vocabs,
    str_has_vocabs,
    str_has_vocabs,
    str_has_vocabs,
    str_has_vocabs,
    str_has_vocabs,
    str_has_vocabs,
    str_a_vocabs,
    str_a_vocabs,
    str_a_vocabs,
    str_a_vocabs,
    str_a_vocabs,
    str_a_vocabs,
    str_a_vocabs,
    str_a_vocabs,
    str_a_vocabs,
    str_a_vocabs,
    str_a_vocabs,
    str_iki_vocabs,
    str_iki_vocabs,
    str_iki_vocabs,
    str_iki_vocabs,
    str_iki_vocabs,
    str_iki_vocabs,
    str_iki_vocabs,
    str_iki_vocabs,
    str_iki_vocabs,
    str_iki_vocabs,
    str_iki_vocabs,
    str_has_vocabs,
    str_has_vocabs,
    str_has_vocabs,
    str_has_vocabs,
    str_has_vocabs,
    str_has_vocabs,
    str_has_vocabs,
    str_has_vocabs,
    str_has_vocabs,
    str_has_vocabs,
    str_has_vocabs,
    str_a_vocabs,
    str_a_vocabs,
    str_a_vocabs,
    str_a_vocabs,
    str_a_vocabs,
    str_a_vocabs,
    str_a_vocabs,
    str_a_vocabs,
    str_a_vocabs,
    str_a_vocabs,
    str_a_vocabs,
    str_iki_vocabs,
    str_iki_vocabs,
    str_iki_vocabs,
    str_iki_vocabs,
    str_iki_vocabs,
    str_iki_vocabs,
    str_iki_vocabs,
    str_iki_vocabs,
    str_iki_vocabs,
    str_iki_vocabs,
    str_iki_vocabs,
    str_has_vocabs,
    str_has_vocabs,
    str_has_vocabs,
    str_has_vocabs,
    str_has_vocabs,
    str_has_vocabs,
    str_has_vocabs,
    str_has_vocabs,
    str_has_vocabs,
    str_has_vocabs,
    str_has_vocabs,
    str_a_vocabs,
    str_a_vocabs,
    str_a_vocabs,
    str_a_vocabs,
    str_a_vocabs,
    str_a_vocabs,
    str_a_vocabs,
    str_a_vocabs,
    str_a_vocabs,
    str_a_vocabs,
    str_a_vocabs,
    str_iki_vocabs,
    str_iki_vocabs,
    str_iki_vocabs,
    str_iki_vocabs,
    str_iki_vocabs,
    str_iki_vocabs,
    str_iki_vocabs,
    str_iki_vocabs,
    str_iki_vocabs,
    str_iki_vocabs,
    str_iki_vocabs,
    str_has_vocabs,
    str_has_vocabs,
    str_has_vocabs,
    str_has_vocabs,
    str_has_vocabs,
    str_has_vocabs,
    str_has_vocabs,
    str_has_vocabs,
    str_has_vocabs,
    str_has_vocabs,
    str_has_vocabs,
    str_a_vocabs,
    str_a_vocabs,
    str_a_vocabs,
    str_a_vocabs,
    str_a_vocabs,
    str_a_vocabs,
    str_a_vocabs,
    str_a_vocabs,
    str_a_vocabs,
    str_a_vocabs,
    str_a_vocabs,
    str_iki_vocabs,
    str_iki_vocabs,
    str_iki_vocabs,
    str_iki_vocabs,
    str_iki_vocabs,
    str_iki_vocabs,
    str_iki_vocabs,
    str_iki_vocabs,
    str_iki_vocabs,
    str_iki_vocabs,
    str_iki_vocabs,
    str_has_vocabs,
    str_has_vocabs,
    str_has_vocabs,
    str_has_vocabs,
    str_has_vocabs,
    str_has_vocabs,
    str_has_vocabs,
    str_has_vocabs,
    str_has_vocabs,
    str_has_vocabs,
    str_has_vocabs,
    str_a_vocabs,
    str_a_vocabs,
    str_a_vocabs,
    str_a_vocabs,
    str_a_vocabs,
    str_a_vocabs,
    str_a_vocabs,
    str_a_vocabs,
    str_a_vocabs,
    str_a_vocabs,
    str_a_vocabs,
    str_iki_vocabs,
    str_iki_vocabs,
    str_iki_vocabs,
    str_iki_vocabs,
    str_iki_vocabs,
    str_iki_vocabs,
    str_iki_vocabs,
    str_iki_vocabs,
    str_iki_vocabs,
    str_iki_vocabs,
    str_iki_vocabs,
    str_has_vocabs,
    str_has_vocabs,
    str_has_vocabs,
    str_has_vocabs,
    str_has_vocabs,
    str_has_vocabs,
    str_has_vocabs,
    str_has_vocabs,
    str_has_vocabs,
    str_has_vocabs,
    str_has_vocabs,
  };

  f_string_static_t contents[] = {
    empty,
    str_b,
    str_content,
    str_content_quote_single,
    str_quote_single,
    str_quote_grave,
    str_quote_double,
    str_slash_1,
    str_slash_2,
    str_slash_3,
    str_slash_4,
    empty,
    str_b,
    str_content,
    str_content_quote_single,
    str_quote_single,
    str_quote_grave,
    str_quote_double,
    str_slash_1,
    str_slash_2,
    str_slash_3,
    str_slash_4,
    empty,
    str_b,
    str_content,
    str_content_quote_single,
    str_quote_single,
    str_quote_grave,
    str_quote_double,
    str_slash_1,
    str_slash_2,
    str_slash_3,
    str_slash_4,
    empty,
    str_b,
    str_content,
    str_content_quote_grave,
    str_quote_single,
    str_quote_grave,
    str_quote_double,
    str_slash_1,
    str_slash_2,
    str_slash_3,
    str_slash_4,
    empty,
    str_b,
    str_content,
    str_content_quote_grave,
    str_quote_single,
    str_quote_grave,
    str_quote_double,
    str_slash_1,
    str_slash_2,
    str_slash_3,
    str_slash_4,
    empty,
    str_b,
    str_content,
    str_content_quote_grave,
    str_quote_single,
    str_quote_grave,
    str_quote_double,
    str_slash_1,
    str_slash_2,
    str_slash_3,
    str_slash_4,
    empty,
    str_b,
    str_content,
    str_content_quote_double,
    str_quote_single,
    str_quote_grave,
    str_quote_double,
    str_slash_1,
    str_slash_2,
    str_slash_3,
    str_slash_4,
    empty,
    str_b,
    str_content,
    str_content_quote_double,
    str_quote_single,
    str_quote_grave,
    str_quote_double,
    str_slash_1,
    str_slash_2,
    str_slash_3,
    str_slash_4,
    empty,
    str_b,
    str_content,
    str_content_quote_double,
    str_quote_single,
    str_quote_grave,
    str_quote_double,
    str_slash_1,
    str_slash_2,
    str_slash_3,
    str_slash_4,
    empty,
    str_b,
    str_content,
    str_content_quote_single,
    str_quote_single,
    str_quote_grave,
    str_quote_double,
    str_slash_1,
    str_slash_2,
    str_slash_3,
    str_slash_4,
    empty,
    str_b,
    str_content,
    str_content_quote_single,
    str_quote_single,
    str_quote_grave,
    str_quote_double,
    str_slash_1,
    str_slash_2,
    str_slash_3,
    str_slash_4,
    empty,
    str_b,
    str_content,
    str_content_quote_single,
    str_quote_single,
    str_quote_grave,
    str_quote_double,
    str_slash_1,
    str_slash_2,
    str_slash_3,
    str_slash_4,
    empty,
    str_b,
    str_content,
    str_content_quote_grave,
    str_quote_single,
    str_quote_grave,
    str_quote_double,
    str_slash_1,
    str_slash_2,
    str_slash_3,
    str_slash_4,
    empty,
    str_b,
    str_content,
    str_content_quote_grave,
    str_quote_single,
    str_quote_grave,
    str_quote_double,
    str_slash_1,
    str_slash_2,
    str_slash_3,
    str_slash_4,
    empty,
    str_b,
    str_content,
    str_content_quote_grave,
    str_quote_single,
    str_quote_grave,
    str_quote_double,
    str_slash_1,
    str_slash_2,
    str_slash_3,
    str_slash_4,
    empty,
    str_b,
    str_content,
    str_content_quote_double,
    str_quote_single,
    str_quote_grave,
    str_quote_double,
    str_slash_1,
    str_slash_2,
    str_slash_3,
    str_slash_4,
    empty,
    str_b,
    str_content,
    str_content_quote_double,
    str_quote_single,
    str_quote_grave,
    str_quote_double,
    str_slash_1,
    str_slash_2,
    str_slash_3,
    str_slash_4,
    empty,
    str_b,
    str_content,
    str_content_quote_double,
    str_quote_single,
    str_quote_grave,
    str_quote_double,
    str_slash_1,
    str_slash_2,
    str_slash_3,
    str_slash_4,
  };

  f_string_static_t quotes[] = {
    f_iki_syntax_quote_single_s,
    f_iki_syntax_quote_single_s,
    f_iki_syntax_quote_single_s,
    f_iki_syntax_quote_single_s,
    f_iki_syntax_quote_single_s,
    f_iki_syntax_quote_single_s,
    f_iki_syntax_quote_single_s,
    f_iki_syntax_quote_single_s,
    f_iki_syntax_quote_single_s,
    f_iki_syntax_quote_single_s,
    f_iki_syntax_quote_single_s,
    f_iki_syntax_quote_single_s,
    f_iki_syntax_quote_single_s,
    f_iki_syntax_quote_single_s,
    f_iki_syntax_quote_single_s,
    f_iki_syntax_quote_single_s,
    f_iki_syntax_quote_single_s,
    f_iki_syntax_quote_single_s,
    f_iki_syntax_quote_single_s,
    f_iki_syntax_quote_single_s,
    f_iki_syntax_quote_single_s,
    f_iki_syntax_quote_single_s,
    f_iki_syntax_quote_single_s,
    f_iki_syntax_quote_single_s,
    f_iki_syntax_quote_single_s,
    f_iki_syntax_quote_single_s,
    f_iki_syntax_quote_single_s,
    f_iki_syntax_quote_single_s,
    f_iki_syntax_quote_single_s,
    f_iki_syntax_quote_single_s,
    f_iki_syntax_quote_single_s,
    f_iki_syntax_quote_single_s,
    f_iki_syntax_quote_single_s,
    f_iki_syntax_quote_grave_s,
    f_iki_syntax_quote_grave_s,
    f_iki_syntax_quote_grave_s,
    f_iki_syntax_quote_grave_s,
    f_iki_syntax_quote_grave_s,
    f_iki_syntax_quote_grave_s,
    f_iki_syntax_quote_grave_s,
    f_iki_syntax_quote_grave_s,
    f_iki_syntax_quote_grave_s,
    f_iki_syntax_quote_grave_s,
    f_iki_syntax_quote_grave_s,
    f_iki_syntax_quote_grave_s,
    f_iki_syntax_quote_grave_s,
    f_iki_syntax_quote_grave_s,
    f_iki_syntax_quote_grave_s,
    f_iki_syntax_quote_grave_s,
    f_iki_syntax_quote_grave_s,
    f_iki_syntax_quote_grave_s,
    f_iki_syntax_quote_grave_s,
    f_iki_syntax_quote_grave_s,
    f_iki_syntax_quote_grave_s,
    f_iki_syntax_quote_grave_s,
    f_iki_syntax_quote_grave_s,
    f_iki_syntax_quote_grave_s,
    f_iki_syntax_quote_grave_s,
    f_iki_syntax_quote_grave_s,
    f_iki_syntax_quote_grave_s,
    f_iki_syntax_quote_grave_s,
    f_iki_syntax_quote_grave_s,
    f_iki_syntax_quote_grave_s,
    f_iki_syntax_quote_grave_s,
    f_iki_syntax_quote_grave_s,
    f_iki_syntax_quote_grave_s,
    f_iki_syntax_quote_double_s,
    f_iki_syntax_quote_double_s,
    f_iki_syntax_quote_double_s,
    f_iki_syntax_quote_double_s,
    f_iki_syntax_quote_double_s,
    f_iki_syntax_quote_double_s,
    f_iki_syntax_quote_double_s,
    f_iki_syntax_quote_double_s,
    f_iki_syntax_quote_double_s,
    f_iki_syntax_quote_double_s,
    f_iki_syntax_quote_double_s,
    f_iki_syntax_quote_double_s,
    f_iki_syntax_quote_double_s,
    f_iki_syntax_quote_double_s,
    f_iki_syntax_quote_double_s,
    f_iki_syntax_quote_double_s,
    f_iki_syntax_quote_double_s,
    f_iki_syntax_quote_double_s,
    f_iki_syntax_quote_double_s,
    f_iki_syntax_quote_double_s,
    f_iki_syntax_quote_double_s,
    f_iki_syntax_quote_double_s,
    f_iki_syntax_quote_double_s,
    f_iki_syntax_quote_double_s,
    f_iki_syntax_quote_double_s,
    f_iki_syntax_quote_double_s,
    f_iki_syntax_quote_double_s,
    f_iki_syntax_quote_double_s,
    f_iki_syntax_quote_double_s,
    f_iki_syntax_quote_double_s,
    f_iki_syntax_quote_double_s,
    f_iki_syntax_quote_double_s,
    f_iki_syntax_quote_double_s,
    f_iki_syntax_quote_single_s,
    f_iki_syntax_quote_single_s,
    f_iki_syntax_quote_single_s,
    f_iki_syntax_quote_single_s,
    f_iki_syntax_quote_single_s,
    f_iki_syntax_quote_single_s,
    f_iki_syntax_quote_single_s,
    f_iki_syntax_quote_single_s,
    f_iki_syntax_quote_single_s,
    f_iki_syntax_quote_single_s,
    f_iki_syntax_quote_single_s,
    f_iki_syntax_quote_single_s,
    f_iki_syntax_quote_single_s,
    f_iki_syntax_quote_single_s,
    f_iki_syntax_quote_single_s,
    f_iki_syntax_quote_single_s,
    f_iki_syntax_quote_single_s,
    f_iki_syntax_quote_single_s,
    f_iki_syntax_quote_single_s,
    f_iki_syntax_quote_single_s,
    f_iki_syntax_quote_single_s,
    f_iki_syntax_quote_single_s,
    f_iki_syntax_quote_single_s,
    f_iki_syntax_quote_single_s,
    f_iki_syntax_quote_single_s,
    f_iki_syntax_quote_single_s,
    f_iki_syntax_quote_single_s,
    f_iki_syntax_quote_single_s,
    f_iki_syntax_quote_single_s,
    f_iki_syntax_quote_single_s,
    f_iki_syntax_quote_single_s,
    f_iki_syntax_quote_single_s,
    f_iki_syntax_quote_single_s,
    f_iki_syntax_quote_grave_s,
    f_iki_syntax_quote_grave_s,
    f_iki_syntax_quote_grave_s,
    f_iki_syntax_quote_grave_s,
    f_iki_syntax_quote_grave_s,
    f_iki_syntax_quote_grave_s,
    f_iki_syntax_quote_grave_s,
    f_iki_syntax_quote_grave_s,
    f_iki_syntax_quote_grave_s,
    f_iki_syntax_quote_grave_s,
    f_iki_syntax_quote_grave_s,
    f_iki_syntax_quote_grave_s,
    f_iki_syntax_quote_grave_s,
    f_iki_syntax_quote_grave_s,
    f_iki_syntax_quote_grave_s,
    f_iki_syntax_quote_grave_s,
    f_iki_syntax_quote_grave_s,
    f_iki_syntax_quote_grave_s,
    f_iki_syntax_quote_grave_s,
    f_iki_syntax_quote_grave_s,
    f_iki_syntax_quote_grave_s,
    f_iki_syntax_quote_grave_s,
    f_iki_syntax_quote_grave_s,
    f_iki_syntax_quote_grave_s,
    f_iki_syntax_quote_grave_s,
    f_iki_syntax_quote_grave_s,
    f_iki_syntax_quote_grave_s,
    f_iki_syntax_quote_grave_s,
    f_iki_syntax_quote_grave_s,
    f_iki_syntax_quote_grave_s,
    f_iki_syntax_quote_grave_s,
    f_iki_syntax_quote_grave_s,
    f_iki_syntax_quote_grave_s,
    f_iki_syntax_quote_double_s,
    f_iki_syntax_quote_double_s,
    f_iki_syntax_quote_double_s,
    f_iki_syntax_quote_double_s,
    f_iki_syntax_quote_double_s,
    f_iki_syntax_quote_double_s,
    f_iki_syntax_quote_double_s,
    f_iki_syntax_quote_double_s,
    f_iki_syntax_quote_double_s,
    f_iki_syntax_quote_double_s,
    f_iki_syntax_quote_double_s,
    f_iki_syntax_quote_double_s,
    f_iki_syntax_quote_double_s,
    f_iki_syntax_quote_double_s,
    f_iki_syntax_quote_double_s,
    f_iki_syntax_quote_double_s,
    f_iki_syntax_quote_double_s,
    f_iki_syntax_quote_double_s,
    f_iki_syntax_quote_double_s,
    f_iki_syntax_quote_double_s,
    f_iki_syntax_quote_double_s,
    f_iki_syntax_quote_double_s,
    f_iki_syntax_quote_double_s,
    f_iki_syntax_quote_double_s,
    f_iki_syntax_quote_double_s,
    f_iki_syntax_quote_double_s,
    f_iki_syntax_quote_double_s,
    f_iki_syntax_quote_double_s,
    f_iki_syntax_quote_double_s,
    f_iki_syntax_quote_double_s,
    f_iki_syntax_quote_double_s,
    f_iki_syntax_quote_double_s,
    f_iki_syntax_quote_double_s,
  };

  uint64_t flags[] = {
    0,
    0,
    0,
    0,
    0,
    0,
    0,
    0,
    0,
    0,
    0,
    0,
    0,
    0,
    0,
    0,
    0,
    0,
    0,
    0,
    0,
    0,
    0,
    0,
    0,
    0,
    0,
    0,
    0,
    0,
    0,
    0,
    0,
    0,
    0,
    0,
    0,
    0,
    0,
    0,
    0,
    0,
    0,
    0,
    0,
    0,
    0,
    0,
    0,
    0,
    0,
    0,
    0,
    0,
    0,
    0,
    0,
    0,
    0,
    0,
    0,
    0,
    0,
    0,
    0,
    0,
    0,
    0,
    0,
    0,
    0,
    0,
    0,
    0,
    0,
    0,
    0,
    0,
    0,
    0,
    0,
    0,
    0,
    0,
    0,
    0,
    0,
    0,
    0,
    0,
    0,
    0,
    0,
    0,
    0,
    0,
    0,
    0,
    0,
    f_iki_state_flag_force_wrap_e,
    f_iki_state_flag_force_wrap_e,
    f_iki_state_flag_force_wrap_e,
    f_iki_state_flag_force_wrap_e,
    f_iki_state_flag_force_wrap_e,
    f_iki_state_flag_force_wrap_e,
    f_iki_state_flag_force_wrap_e,
    f_iki_state_flag_force_wrap_e,
    f_iki_state_flag_force_wrap_e,
    f_iki_state_flag_force_wrap_e,
    f_iki_state_flag_force_wrap_e,
    f_iki_state_flag_force_wrap_e,
    f_iki_state_flag_force_wrap_e,
    f_iki_state_flag_force_wrap_e,
    f_iki_state_flag_force_wrap_e,
    f_iki_state_flag_force_wrap_e,
    f_iki_state_flag_force_wrap_e,
    f_iki_state_flag_force_wrap_e,
    f_iki_state_flag_force_wrap_e,
    f_iki_state_flag_force_wrap_e,
    f_iki_state_flag_force_wrap_e,
    f_iki_state_flag_force_wrap_e,
    f_iki_state_flag_force_wrap_e,
    f_iki_state_flag_force_wrap_e,
    f_iki_state_flag_force_wrap_e,
    f_iki_state_flag_force_wrap_e,
    f_iki_state_flag_force_wrap_e,
    f_iki_state_flag_force_wrap_e,
    f_iki_state_flag_force_wrap_e,
    f_iki_state_flag_force_wrap_e,
    f_iki_state_flag_force_wrap_e,
    f_iki_state_flag_force_wrap_e,
    f_iki_state_flag_force_wrap_e,
    f_iki_state_flag_force_wrap_e,
    f_iki_state_flag_force_wrap_e,
    f_iki_state_flag_force_wrap_e,
    f_iki_state_flag_force_wrap_e,
    f_iki_state_flag_force_wrap_e,
    f_iki_state_flag_force_wrap_e,
    f_iki_state_flag_force_wrap_e,
    f_iki_state_flag_force_wrap_e,
    f_iki_state_flag_force_wrap_e,
    f_iki_state_flag_force_wrap_e,
    f_iki_state_flag_force_wrap_e,
    f_iki_state_flag_force_wrap_e,
    f_iki_state_flag_force_wrap_e,
    f_iki_state_flag_force_wrap_e,
    f_iki_state_flag_force_wrap_e,
    f_iki_state_flag_force_wrap_e,
    f_iki_state_flag_force_wrap_e,
    f_iki_state_flag_force_wrap_e,
    f_iki_state_flag_force_wrap_e,
    f_iki_state_flag_force_wrap_e,
    f_iki_state_flag_force_wrap_e,
    f_iki_state_flag_force_wrap_e,
    f_iki_state_flag_force_wrap_e,
    f_iki_state_flag_force_wrap_e,
    f_iki_state_flag_force_wrap_e,
    f_iki_state_flag_force_wrap_e,
    f_iki_state_flag_force_wrap_e,
    f_iki_state_flag_force_wrap_e,
    f_iki_state_flag_force_wrap_e,
    f_iki_state_flag_force_wrap_e,
    f_iki_state_flag_force_wrap_e,
    f_iki_state_flag_force_wrap_e,
    f_iki_state_flag_force_wrap_e,
    f_iki_state_flag_force_wrap_e,
    f_iki_state_flag_force_wrap_e,
    f_iki_state_flag_force_wrap_e,
    f_iki_state_flag_force_wrap_e,
    f_iki_state_flag_force_wrap_e,
    f_iki_state_flag_force_wrap_e,
    f_iki_state_flag_force_wrap_e,
    f_iki_state_flag_force_wrap_e,
    f_iki_state_flag_force_wrap_e,
    f_iki_state_flag_force_wrap_e,
    f_iki_state_flag_force_wrap_e,
    f_iki_state_flag_force_wrap_e,
    f_iki_state_flag_force_wrap_e,
    f_iki_state_flag_force_wrap_e,
    f_iki_state_flag_force_wrap_e,
    f_iki_state_flag_force_wrap_e,
    f_iki_state_flag_force_wrap_e,
    f_iki_state_flag_force_wrap_e,
    f_iki_state_flag_force_wrap_e,
    f_iki_state_flag_force_wrap_e,
    f_iki_state_flag_force_wrap_e,
    f_iki_state_flag_force_wrap_e,
    f_iki_state_flag_force_wrap_e,
    f_iki_state_flag_force_wrap_e,
    f_iki_state_flag_force_wrap_e,
    f_iki_state_flag_force_wrap_e,
    f_iki_state_flag_force_wrap_e,
    f_iki_state_flag_force_wrap_e,
    f_iki_state_flag_force_wrap_e,
    f_iki_state_flag_force_wrap_e,
    f_iki_state_flag_force_wrap_e,
    f_iki_state_flag_force_wrap_e,
    f_iki_state_flag_force_wrap_e,
  };

  f_string_static_t ikis[] = {
    macro_f_string_static_t_initialize_1("a:''", 0, 4),
    macro_f_string_static_t_initialize_1("a:'b'", 0, 5),
    macro_f_string_static_t_initialize_1("a:'content'", 0, 11),
    macro_f_string_static_t_initialize_1("a:'\\'content\\''", 0, 15),
    macro_f_string_static_t_initialize_1("a:'quote\\'s'", 0, 12),
    macro_f_string_static_t_initialize_1("a:'quote`s'", 0, 11),
    macro_f_string_static_t_initialize_1("a:'quote\"s'", 0, 11),
    macro_f_string_static_t_initialize_1("a:'slash\\\\'", 0, 11),
    macro_f_string_static_t_initialize_1("a:'slash\\\\\\\\'", 0, 13),
    macro_f_string_static_t_initialize_1("a:'slash\\\\\\\\\\\\'", 0, 15),
    macro_f_string_static_t_initialize_1("a:'slash\\\\\\\\\\\\\\\\'", 0, 17),
    macro_f_string_static_t_initialize_1("iki:''", 0, 6),
    macro_f_string_static_t_initialize_1("iki:'b'", 0, 7),
    macro_f_string_static_t_initialize_1("iki:'content'", 0, 13),
    macro_f_string_static_t_initialize_1("iki:'\\'content\\''", 0, 17),
    macro_f_string_static_t_initialize_1("iki:'quote\\'s'", 0, 14),
    macro_f_string_static_t_initialize_1("iki:'quote`s'", 0, 13),
    macro_f_string_static_t_initialize_1("iki:'quote\"s'", 0, 13),
    macro_f_string_static_t_initialize_1("iki:'slash\\\\'", 0, 13),
    macro_f_string_static_t_initialize_1("iki:'slash\\\\\\\\'", 0, 15),
    macro_f_string_static_t_initialize_1("iki:'slash\\\\\\\\\\\\'", 0, 17),
    macro_f_string_static_t_initialize_1("iki:'slash\\\\\\\\\\\\\\\\'", 0, 19),
    macro_f_string_static_t_initialize_1("has_CHARS+-:''", 0, 14),
    macro_f_string_static_t_initialize_1("has_CHARS+-:'b'", 0, 15),
    macro_f_string_static_t_initialize_1("has_CHARS+-:'content'", 0, 21),
    macro_f_string_static_t_initialize_1("has_CHARS+-:'\\'content\\''", 0, 25),
    macro_f_string_static_t_initialize_1("has_CHARS+-:'quote\\'s'", 0, 22),
    macro_f_string_static_t_initialize_1("has_CHARS+-:'quote`s'", 0, 21),
    macro_f_string_static_t_initialize_1("has_CHARS+-:'quote\"s'", 0, 21),
    macro_f_string_static_t_initialize_1("has_CHARS+-:'slash\\\\'", 0, 21),
    macro_f_string_static_t_initialize_1("has_CHARS+-:'slash\\\\\\\\'", 0, 23),
    macro_f_string_static_t_initialize_1("has_CHARS+-:'slash\\\\\\\\\\\\'", 0, 25),
    macro_f_string_static_t_initialize_1("has_CHARS+-:'slash\\\\\\\\\\\\\\\\'", 0, 27),
    macro_f_string_static_t_initialize_1("a:``", 0, 4),
    macro_f_string_static_t_initialize_1("a:`b`", 0, 5),
    macro_f_string_static_t_initialize_1("a:`content`", 0, 11),
    macro_f_string_static_t_initialize_1("a:`\\`content\\``", 0, 15),
    macro_f_string_static_t_initialize_1("a:`quote's`", 0, 11),
    macro_f_string_static_t_initialize_1("a:`quote\\`s`", 0, 12),
    macro_f_string_static_t_initialize_1("a:`quote\"s`", 0, 11),
    macro_f_string_static_t_initialize_1("a:`slash\\\\`", 0, 11),
    macro_f_string_static_t_initialize_1("a:`slash\\\\\\\\`", 0, 13),
    macro_f_string_static_t_initialize_1("a:`slash\\\\\\\\\\\\`", 0, 15),
    macro_f_string_static_t_initialize_1("a:`slash\\\\\\\\\\\\\\\\`", 0, 17),
    macro_f_string_static_t_initialize_1("iki:``", 0, 6),
    macro_f_string_static_t_initialize_1("iki:`b`", 0, 7),
    macro_f_string_static_t_initialize_1("iki:`content`", 0, 13),
    macro_f_string_static_t_initialize_1("iki:`\\`content\\``", 0, 17),
    macro_f_string_static_t_initialize_1("iki:`quote's`", 0, 13),
    macro_f_string_static_t_initialize_1("iki:`quote\\`s`", 0, 14),
    macro_f_string_static_t_initialize_1("iki:`quote\"s`", 0, 13),
    macro_f_string_static_t_initialize_1("iki:`slash\\\\`", 0, 13),
    macro_f_string_static_t_initialize_1("iki:`slash\\\\\\\\`", 0, 15),
    macro_f_string_static_t_initialize_1("iki:`slash\\\\\\\\\\\\`", 0, 17),
    macro_f_string_static_t_initialize_1("iki:`slash\\\\\\\\\\\\\\\\`", 0, 19),
    macro_f_string_static_t_initialize_1("has_CHARS+-:``", 0, 14),
    macro_f_string_static_t_initialize_1("has_CHARS+-:`b`", 0, 15),
    macro_f_string_static_t_initialize_1("has_CHARS+-:`content`", 0, 21),
    macro_f_string_static_t_initialize_1("has_CHARS+-:`\\`content\\``", 0, 25),
    macro_f_string_static_t_initialize_1("has_CHARS+-:`quote's`", 0, 21),
    macro_f_string_static_t_initialize_1("has_CHARS+-:`quote\\`s`", 0, 22),
    macro_f_string_static_t_initialize_1("has_CHARS+-:`quote\"s`", 0, 21),
    macro_f_string_static_t_initialize_1("has_CHARS+-:`slash\\\\`", 0, 21),
    macro_f_string_static_t_initialize_1("has_CHARS+-:`slash\\\\\\\\`", 0, 23),
    macro_f_string_static_t_initialize_1("has_CHARS+-:`slash\\\\\\\\\\\\`", 0, 25),
    macro_f_string_static_t_initialize_1("has_CHARS+-:`slash\\\\\\\\\\\\\\\\`", 0, 27),
    macro_f_string_static_t_initialize_1("a:\"\"", 0, 4),
    macro_f_string_static_t_initialize_1("a:\"b\"", 0, 5),
    macro_f_string_static_t_initialize_1("a:\"content\"", 0, 11),
    macro_f_string_static_t_initialize_1("a:\"\\\"content\\\"\"", 0, 15),
    macro_f_string_static_t_initialize_1("a:\"quote's\"", 0, 11),
    macro_f_string_static_t_initialize_1("a:\"quote`s\"", 0, 11),
    macro_f_string_static_t_initialize_1("a:\"quote\\\"s\"", 0, 12),
    macro_f_string_static_t_initialize_1("a:\"slash\\\\\"", 0, 11),
    macro_f_string_static_t_initialize_1("a:\"slash\\\\\\\\\"", 0, 13),
    macro_f_string_static_t_initialize_1("a:\"slash\\\\\\\\\\\\\"", 0, 15),
    macro_f_string_static_t_initialize_1("a:\"slash\\\\\\\\\\\\\\\\\"", 0, 17),
    macro_f_string_static_t_initialize_1("iki:\"\"", 0, 6),
    macro_f_string_static_t_initialize_1("iki:\"b\"", 0, 7),
    macro_f_string_static_t_initialize_1("iki:\"content\"", 0, 13),
    macro_f_string_static_t_initialize_1("iki:\"\\\"content\\\"\"", 0, 17),
    macro_f_string_static_t_initialize_1("iki:\"quote's\"", 0, 13),
    macro_f_string_static_t_initialize_1("iki:\"quote`s\"", 0, 13),
    macro_f_string_static_t_initialize_1("iki:\"quote\\\"s\"", 0, 14),
    macro_f_string_static_t_initialize_1("iki:\"slash\\\\\"", 0, 13),
    macro_f_string_static_t_initialize_1("iki:\"slash\\\\\\\\\"", 0, 15),
    macro_f_string_static_t_initialize_1("iki:\"slash\\\\\\\\\\\\\"", 0, 17),
    macro_f_string_static_t_initialize_1("iki:\"slash\\\\\\\\\\\\\\\\\"", 0, 19),
    macro_f_string_static_t_initialize_1("has_CHARS+-:\"\"", 0, 14),
    macro_f_string_static_t_initialize_1("has_CHARS+-:\"b\"", 0, 15),
    macro_f_string_static_t_initialize_1("has_CHARS+-:\"content\"", 0, 21),
    macro_f_string_static_t_initialize_1("has_CHARS+-:\"\\\"content\\\"\"", 0, 25),
    macro_f_string_static_t_initialize_1("has_CHARS+-:\"quote's\"", 0, 21),
    macro_f_string_static_t_initialize_1("has_CHARS+-:\"quote`s\"", 0, 21),
    macro_f_string_static_t_initialize_1("has_CHARS+-:\"quote\\\"s\"", 0, 22),
    macro_f_string_static_t_initialize_1("has_CHARS+-:\"slash\\\\\"", 0, 21),
    macro_f_string_static_t_initialize_1("has_CHARS+-:\"slash\\\\\\\\\"", 0, 23),
    macro_f_string_static_t_initialize_1("has_CHARS+-:\"slash\\\\\\\\\\\\\"", 0, 25),
    macro_f_string_static_t_initialize_1("has_CHARS+-:\"slash\\\\\\\\\\\\\\\\\"", 0, 27),
    macro_f_string_static_t_initialize_1("[a]:''", 0, 6),
    macro_f_string_static_t_initialize_1("[a]:'b'", 0, 7),
    macro_f_string_static_t_initialize_1("[a]:'content'", 0, 13),
    macro_f_string_static_t_initialize_1("[a]:'\\'content\\''", 0, 17),
    macro_f_string_static_t_initialize_1("[a]:'quote\\'s'", 0, 14),
    macro_f_string_static_t_initialize_1("[a]:'quote`s'", 0, 13),
    macro_f_string_static_t_initialize_1("[a]:'quote\"s'", 0, 13),
    macro_f_string_static_t_initialize_1("[a]:'slash\\\\'", 0, 13),
    macro_f_string_static_t_initialize_1("[a]:'slash\\\\\\\\'", 0, 15),
    macro_f_string_static_t_initialize_1("[a]:'slash\\\\\\\\\\\\'", 0, 17),
    macro_f_string_static_t_initialize_1("[a]:'slash\\\\\\\\\\\\\\\\'", 0, 19),
    macro_f_string_static_t_initialize_1("[iki]:''", 0, 8),
    macro_f_string_static_t_initialize_1("[iki]:'b'", 0, 9),
    macro_f_string_static_t_initialize_1("[iki]:'content'", 0, 15),
    macro_f_string_static_t_initialize_1("[iki]:'\\'content\\''", 0, 19),
    macro_f_string_static_t_initialize_1("[iki]:'quote\\'s'", 0, 16),
    macro_f_string_static_t_initialize_1("[iki]:'quote`s'", 0, 15),
    macro_f_string_static_t_initialize_1("[iki]:'quote\"s'", 0, 15),
    macro_f_string_static_t_initialize_1("[iki]:'slash\\\\'", 0, 15),
    macro_f_string_static_t_initialize_1("[iki]:'slash\\\\\\\\'", 0, 17),
    macro_f_string_static_t_initialize_1("[iki]:'slash\\\\\\\\\\\\'", 0, 19),
    macro_f_string_static_t_initialize_1("[iki]:'slash\\\\\\\\\\\\\\\\'", 0, 21),
    macro_f_string_static_t_initialize_1("[has_CHARS+-]:''", 0, 16),
    macro_f_string_static_t_initialize_1("[has_CHARS+-]:'b'", 0, 17),
    macro_f_string_static_t_initialize_1("[has_CHARS+-]:'content'", 0, 23),
    macro_f_string_static_t_initialize_1("[has_CHARS+-]:'\\'content\\''", 0, 27),
    macro_f_string_static_t_initialize_1("[has_CHARS+-]:'quote\\'s'", 0, 24),
    macro_f_string_static_t_initialize_1("[has_CHARS+-]:'quote`s'", 0, 23),
    macro_f_string_static_t_initialize_1("[has_CHARS+-]:'quote\"s'", 0, 23),
    macro_f_string_static_t_initialize_1("[has_CHARS+-]:'slash\\\\'", 0, 23),
    macro_f_string_static_t_initialize_1("[has_CHARS+-]:'slash\\\\\\\\'", 0, 25),
    macro_f_string_static_t_initialize_1("[has_CHARS+-]:'slash\\\\\\\\\\\\'", 0, 27),
    macro_f_string_static_t_initialize_1("[has_CHARS+-]:'slash\\\\\\\\\\\\\\\\'", 0, 29),
    macro_f_string_static_t_initialize_1("[a]:``", 0, 6),
    macro_f_string_static_t_initialize_1("[a]:`b`", 0, 7),
    macro_f_string_static_t_initialize_1("[a]:`content`", 0, 13),
    macro_f_string_static_t_initialize_1("[a]:`\\`content\\``", 0, 17),
    macro_f_string_static_t_initialize_1("[a]:`quote's`", 0, 13),
    macro_f_string_static_t_initialize_1("[a]:`quote\\`s`", 0, 14),
    macro_f_string_static_t_initialize_1("[a]:`quote\"s`", 0, 13),
    macro_f_string_static_t_initialize_1("[a]:`slash\\\\`", 0, 13),
    macro_f_string_static_t_initialize_1("[a]:`slash\\\\\\\\`", 0, 15),
    macro_f_string_static_t_initialize_1("[a]:`slash\\\\\\\\\\\\`", 0, 17),
    macro_f_string_static_t_initialize_1("[a]:`slash\\\\\\\\\\\\\\\\`", 0, 19),
    macro_f_string_static_t_initialize_1("[iki]:``", 0, 8),
    macro_f_string_static_t_initialize_1("[iki]:`b`", 0, 9),
    macro_f_string_static_t_initialize_1("[iki]:`content`", 0, 15),
    macro_f_string_static_t_initialize_1("[iki]:`\\`content\\``", 0, 19),
    macro_f_string_static_t_initialize_1("[iki]:`quote's`", 0, 15),
    macro_f_string_static_t_initialize_1("[iki]:`quote\\`s`", 0, 16),
    macro_f_string_static_t_initialize_1("[iki]:`quote\"s`", 0, 15),
    macro_f_string_static_t_initialize_1("[iki]:`slash\\\\`", 0, 15),
    macro_f_string_static_t_initialize_1("[iki]:`slash\\\\\\\\`", 0, 17),
    macro_f_string_static_t_initialize_1("[iki]:`slash\\\\\\\\\\\\`", 0, 19),
    macro_f_string_static_t_initialize_1("[iki]:`slash\\\\\\\\\\\\\\\\`", 0, 21),
    macro_f_string_static_t_initialize_1("[has_CHARS+-]:``", 0, 16),
    macro_f_string_static_t_initialize_1("[has_CHARS+-]:`b`", 0, 17),
    macro_f_string_static_t_initialize_1("[has_CHARS+-]:`content`", 0, 23),
    macro_f_string_static_t_initialize_1("[has_CHARS+-]:`\\`content\\``", 0, 27),
    macro_f_string_static_t_initialize_1("[has_CHARS+-]:`quote's`", 0, 23),
    macro_f_string_static_t_initialize_1("[has_CHARS+-]:`quote\\`s`", 0, 24),
    macro_f_string_static_t_initialize_1("[has_CHARS+-]:`quote\"s`", 0, 23),
    macro_f_string_static_t_initialize_1("[has_CHARS+-]:`slash\\\\`", 0, 23),
    macro_f_string_static_t_initialize_1("[has_CHARS+-]:`slash\\\\\\\\`", 0, 25),
    macro_f_string_static_t_initialize_1("[has_CHARS+-]:`slash\\\\\\\\\\\\`", 0, 27),
    macro_f_string_static_t_initialize_1("[has_CHARS+-]:`slash\\\\\\\\\\\\\\\\`", 0, 29),
    macro_f_string_static_t_initialize_1("[a]:\"\"", 0, 6),
    macro_f_string_static_t_initialize_1("[a]:\"b\"", 0, 7),
    macro_f_string_static_t_initialize_1("[a]:\"content\"", 0, 13),
    macro_f_string_static_t_initialize_1("[a]:\"\\\"content\\\"\"", 0, 17),
    macro_f_string_static_t_initialize_1("[a]:\"quote's\"", 0, 13),
    macro_f_string_static_t_initialize_1("[a]:\"quote`s\"", 0, 13),
    macro_f_string_static_t_initialize_1("[a]:\"quote\\\"s\"", 0, 14),
    macro_f_string_static_t_initialize_1("[a]:\"slash\\\\\"", 0, 13),
    macro_f_string_static_t_initialize_1("[a]:\"slash\\\\\\\\\"", 0, 15),
    macro_f_string_static_t_initialize_1("[a]:\"slash\\\\\\\\\\\\\"", 0, 17),
    macro_f_string_static_t_initialize_1("[a]:\"slash\\\\\\\\\\\\\\\\\"", 0, 19),
    macro_f_string_static_t_initialize_1("[iki]:\"\"", 0, 8),
    macro_f_string_static_t_initialize_1("[iki]:\"b\"", 0, 9),
    macro_f_string_static_t_initialize_1("[iki]:\"content\"", 0, 15),
    macro_f_string_static_t_initialize_1("[iki]:\"\\\"content\\\"\"", 0, 19),
    macro_f_string_static_t_initialize_1("[iki]:\"quote's\"", 0, 15),
    macro_f_string_static_t_initialize_1("[iki]:\"quote`s\"", 0, 15),
    macro_f_string_static_t_initialize_1("[iki]:\"quote\\\"s\"", 0, 16),
    macro_f_string_static_t_initialize_1("[iki]:\"slash\\\\\"", 0, 15),
    macro_f_string_static_t_initialize_1("[iki]:\"slash\\\\\\\\\"", 0, 17),
    macro_f_string_static_t_initialize_1("[iki]:\"slash\\\\\\\\\\\\\"", 0, 19),
    macro_f_string_static_t_initialize_1("[iki]:\"slash\\\\\\\\\\\\\\\\\"", 0, 21),
    macro_f_string_static_t_initialize_1("[has_CHARS+-]:\"\"", 0, 16),
    macro_f_string_static_t_initialize_1("[has_CHARS+-]:\"b\"", 0, 17),
    macro_f_string_static_t_initialize_1("[has_CHARS+-]:\"content\"", 0, 23),
    macro_f_string_static_t_initialize_1("[has_CHARS+-]:\"\\\"content\\\"\"", 0, 27),
    macro_f_string_static_t_initialize_1("[has_CHARS+-]:\"quote's\"", 0, 23),
    macro_f_string_static_t_initialize_1("[has_CHARS+-]:\"quote`s\"", 0, 23),
    macro_f_string_static_t_initialize_1("[has_CHARS+-]:\"quote\\\"s\"", 0, 24),
    macro_f_string_static_t_initialize_1("[has_CHARS+-]:\"slash\\\\\"", 0, 23),
    macro_f_string_static_t_initialize_1("[has_CHARS+-]:\"slash\\\\\\\\\"", 0, 25),
    macro_f_string_static_t_initialize_1("[has_CHARS+-]:\"slash\\\\\\\\\\\\\"", 0, 27),
    macro_f_string_static_t_initialize_1("[has_CHARS+-]:\"slash\\\\\\\\\\\\\\\\\"", 0, 29),
  };

  const f_status_t statuss[] = {
    F_okay,
    F_okay,
    F_okay,
    F_okay,
    F_okay,
    F_okay,
    F_okay,
    F_okay,
    F_okay,
    F_okay,
    F_okay,
    F_okay,
    F_okay,
    F_okay,
    F_okay,
    F_okay,
    F_okay,
    F_okay,
    F_okay,
    F_okay,
    F_okay,
    F_okay,
    F_okay,
    F_okay,
    F_okay,
    F_okay,
    F_okay,
    F_okay,
    F_okay,
    F_okay,
    F_okay,
    F_okay,
    F_okay,
    F_okay,
    F_okay,
    F_okay,
    F_okay,
    F_okay,
    F_okay,
    F_okay,
    F_okay,
    F_okay,
    F_okay,
    F_okay,
    F_okay,
    F_okay,
    F_okay,
    F_okay,
    F_okay,
    F_okay,
    F_okay,
    F_okay,
    F_okay,
    F_okay,
    F_okay,
    F_okay,
    F_okay,
    F_okay,
    F_okay,
    F_okay,
    F_okay,
    F_okay,
    F_okay,
    F_okay,
    F_okay,
    F_okay,
    F_okay,
    F_okay,
    F_okay,
    F_okay,
    F_okay,
    F_okay,
    F_okay,
    F_okay,
    F_okay,
    F_okay,
    F_okay,
    F_okay,
    F_okay,
    F_okay,
    F_okay,
    F_okay,
    F_okay,
    F_okay,
    F_okay,
    F_okay,
    F_okay,
    F_okay,
    F_okay,
    F_okay,
    F_okay,
    F_okay,
    F_okay,
    F_okay,
    F_okay,
    F_okay,
    F_okay,
    F_okay,
    F_okay,
    F_okay,
    F_okay,
    F_okay,
    F_okay,
    F_okay,
    F_okay,
    F_okay,
    F_okay,
    F_okay,
    F_okay,
    F_okay,
    F_okay,
    F_okay,
    F_okay,
    F_okay,
    F_okay,
    F_okay,
    F_okay,
    F_okay,
    F_okay,
    F_okay,
    F_okay,
    F_okay,
    F_okay,
    F_okay,
    F_okay,
    F_okay,
    F_okay,
    F_okay,
    F_okay,
    F_okay,
    F_okay,
    F_okay,
    F_okay,
    F_okay,
    F_okay,
    F_okay,
    F_okay,
    F_okay,
    F_okay,
    F_okay,
    F_okay,
    F_okay,
    F_okay,
    F_okay,
    F_okay,
    F_okay,
    F_okay,
    F_okay,
    F_okay,
    F_okay,
    F_okay,
    F_okay,
    F_okay,
    F_okay,
    F_okay,
    F_okay,
    F_okay,
    F_okay,
    F_okay,
    F_okay,
    F_okay,
    F_okay,
    F_okay,
    F_okay,
    F_okay,
    F_okay,
    F_okay,
    F_okay,
    F_okay,
    F_okay,
    F_okay,
    F_okay,
    F_okay,
    F_okay,
    F_okay,
    F_okay,
    F_okay,
    F_okay,
    F_okay,
    F_okay,
    F_okay,
    F_okay,
    F_okay,
    F_okay,
    F_okay,
    F_okay,
    F_okay,
    F_okay,
    F_okay,
    F_okay,
    F_okay,
    F_okay,
    F_okay,
    F_okay,
    F_okay,
    F_okay,
    F_okay,
    F_okay,
  };

  for (uint8_t i = 0; i < _inline_macro_total_rows; ++i) {

    state_data.flag = flags[i];
    destination.used = 0;

    f_iki_eki_write(vocabularyss[i], contents[i], quotes[i], &destination, &state_data);

    assert_int_equal(state_data.status, statuss[i]);
    assert_int_equal(destination.used, ikis[i].used);
    assert_string_equal(destination.string, ikis[i].string);

    // Clear the string to make any debugging easier.
    memset(destination.string, 0, sizeof(destination.used));
  } // for

  if (destination.string) free(destination.string);

  #undef _inline_macro_total_rows
}

void test__f_iki_eki_write__works_for_multiple(void **state) {

  mock_unwrap = 1;

  f_state_t state_data = f_state_t_initialize;
  f_string_dynamic_t destination = f_string_dynamic_t_initialize;

  f_string_static_t empty = macro_f_string_static_t_initialize_1(f_string_empty_s.string, f_string_empty_s.size, f_string_empty_s.used);
  f_string_static_t str_a = macro_f_string_static_t_initialize_1("a", 0, 1);
  f_string_static_t str_b = macro_f_string_static_t_initialize_1("b", 0, 1);
  f_string_static_t str_sea = macro_f_string_static_t_initialize_1("sea", 0, 3);
  f_string_static_t str_iki = macro_f_string_static_t_initialize_1("iki", 0, 3);
  f_string_static_t str_has = macro_f_string_static_t_initialize_1("has_CHARS+-", 0, 11);

  f_string_static_t str_a_array[] = { str_a, str_b };
  f_string_static_t str_iki_array[] = { str_iki, str_sea };
  f_string_static_t str_has_array[] = { str_has, str_b, str_sea };

  const f_string_statics_t str_a_vocabs = macro_f_string_statics_t_initialize_2(str_a_array, 2);
  const f_string_statics_t str_iki_vocabs = macro_f_string_statics_t_initialize_2(str_iki_array, 2);
  const f_string_statics_t str_has_vocabs = macro_f_string_statics_t_initialize_2(str_has_array, 3);

  f_string_static_t str_content = macro_f_string_static_t_initialize_1("content", 0, 7);
  f_string_static_t str_content_quote_double = macro_f_string_static_t_initialize_1("\"content\"", 0, 9);
  f_string_static_t str_content_quote_grave = macro_f_string_static_t_initialize_1("`content`", 0, 9);
  f_string_static_t str_content_quote_single = macro_f_string_static_t_initialize_1("'content'", 0, 9);
  f_string_static_t str_quote_double = macro_f_string_static_t_initialize_1("quote\"s", 0, 7);
  f_string_static_t str_quote_grave = macro_f_string_static_t_initialize_1("quote`s", 0, 7);
  f_string_static_t str_quote_single = macro_f_string_static_t_initialize_1("quote's", 0, 7);
  f_string_static_t str_slash_1 = macro_f_string_static_t_initialize_1("slash\\", 0, 6);
  f_string_static_t str_slash_2 = macro_f_string_static_t_initialize_1("slash\\\\", 0, 7);
  f_string_static_t str_slash_3 = macro_f_string_static_t_initialize_1("slash\\\\\\", 0, 8);
  f_string_static_t str_slash_4 = macro_f_string_static_t_initialize_1("slash\\\\\\\\", 0, 9);

  #define _inline_macro_total_rows 198

  f_string_statics_t vocabularyss[] = {
    str_a_vocabs,
    str_a_vocabs,
    str_a_vocabs,
    str_a_vocabs,
    str_a_vocabs,
    str_a_vocabs,
    str_a_vocabs,
    str_a_vocabs,
    str_a_vocabs,
    str_a_vocabs,
    str_a_vocabs,
    str_iki_vocabs,
    str_iki_vocabs,
    str_iki_vocabs,
    str_iki_vocabs,
    str_iki_vocabs,
    str_iki_vocabs,
    str_iki_vocabs,
    str_iki_vocabs,
    str_iki_vocabs,
    str_iki_vocabs,
    str_iki_vocabs,
    str_has_vocabs,
    str_has_vocabs,
    str_has_vocabs,
    str_has_vocabs,
    str_has_vocabs,
    str_has_vocabs,
    str_has_vocabs,
    str_has_vocabs,
    str_has_vocabs,
    str_has_vocabs,
    str_has_vocabs,
    str_a_vocabs,
    str_a_vocabs,
    str_a_vocabs,
    str_a_vocabs,
    str_a_vocabs,
    str_a_vocabs,
    str_a_vocabs,
    str_a_vocabs,
    str_a_vocabs,
    str_a_vocabs,
    str_a_vocabs,
    str_iki_vocabs,
    str_iki_vocabs,
    str_iki_vocabs,
    str_iki_vocabs,
    str_iki_vocabs,
    str_iki_vocabs,
    str_iki_vocabs,
    str_iki_vocabs,
    str_iki_vocabs,
    str_iki_vocabs,
    str_iki_vocabs,
    str_has_vocabs,
    str_has_vocabs,
    str_has_vocabs,
    str_has_vocabs,
    str_has_vocabs,
    str_has_vocabs,
    str_has_vocabs,
    str_has_vocabs,
    str_has_vocabs,
    str_has_vocabs,
    str_has_vocabs,
    str_a_vocabs,
    str_a_vocabs,
    str_a_vocabs,
    str_a_vocabs,
    str_a_vocabs,
    str_a_vocabs,
    str_a_vocabs,
    str_a_vocabs,
    str_a_vocabs,
    str_a_vocabs,
    str_a_vocabs,
    str_iki_vocabs,
    str_iki_vocabs,
    str_iki_vocabs,
    str_iki_vocabs,
    str_iki_vocabs,
    str_iki_vocabs,
    str_iki_vocabs,
    str_iki_vocabs,
    str_iki_vocabs,
    str_iki_vocabs,
    str_iki_vocabs,
    str_has_vocabs,
    str_has_vocabs,
    str_has_vocabs,
    str_has_vocabs,
    str_has_vocabs,
    str_has_vocabs,
    str_has_vocabs,
    str_has_vocabs,
    str_has_vocabs,
    str_has_vocabs,
    str_has_vocabs,
    str_a_vocabs,
    str_a_vocabs,
    str_a_vocabs,
    str_a_vocabs,
    str_a_vocabs,
    str_a_vocabs,
    str_a_vocabs,
    str_a_vocabs,
    str_a_vocabs,
    str_a_vocabs,
    str_a_vocabs,
    str_iki_vocabs,
    str_iki_vocabs,
    str_iki_vocabs,
    str_iki_vocabs,
    str_iki_vocabs,
    str_iki_vocabs,
    str_iki_vocabs,
    str_iki_vocabs,
    str_iki_vocabs,
    str_iki_vocabs,
    str_iki_vocabs,
    str_has_vocabs,
    str_has_vocabs,
    str_has_vocabs,
    str_has_vocabs,
    str_has_vocabs,
    str_has_vocabs,
    str_has_vocabs,
    str_has_vocabs,
    str_has_vocabs,
    str_has_vocabs,
    str_has_vocabs,
    str_a_vocabs,
    str_a_vocabs,
    str_a_vocabs,
    str_a_vocabs,
    str_a_vocabs,
    str_a_vocabs,
    str_a_vocabs,
    str_a_vocabs,
    str_a_vocabs,
    str_a_vocabs,
    str_a_vocabs,
    str_iki_vocabs,
    str_iki_vocabs,
    str_iki_vocabs,
    str_iki_vocabs,
    str_iki_vocabs,
    str_iki_vocabs,
    str_iki_vocabs,
    str_iki_vocabs,
    str_iki_vocabs,
    str_iki_vocabs,
    str_iki_vocabs,
    str_has_vocabs,
    str_has_vocabs,
    str_has_vocabs,
    str_has_vocabs,
    str_has_vocabs,
    str_has_vocabs,
    str_has_vocabs,
    str_has_vocabs,
    str_has_vocabs,
    str_has_vocabs,
    str_has_vocabs,
    str_a_vocabs,
    str_a_vocabs,
    str_a_vocabs,
    str_a_vocabs,
    str_a_vocabs,
    str_a_vocabs,
    str_a_vocabs,
    str_a_vocabs,
    str_a_vocabs,
    str_a_vocabs,
    str_a_vocabs,
    str_iki_vocabs,
    str_iki_vocabs,
    str_iki_vocabs,
    str_iki_vocabs,
    str_iki_vocabs,
    str_iki_vocabs,
    str_iki_vocabs,
    str_iki_vocabs,
    str_iki_vocabs,
    str_iki_vocabs,
    str_iki_vocabs,
    str_has_vocabs,
    str_has_vocabs,
    str_has_vocabs,
    str_has_vocabs,
    str_has_vocabs,
    str_has_vocabs,
    str_has_vocabs,
    str_has_vocabs,
    str_has_vocabs,
    str_has_vocabs,
    str_has_vocabs,
  };

  f_string_static_t contents[] = {
    empty,
    str_b,
    str_content,
    str_content_quote_single,
    str_quote_single,
    str_quote_grave,
    str_quote_double,
    str_slash_1,
    str_slash_2,
    str_slash_3,
    str_slash_4,
    empty,
    str_b,
    str_content,
    str_content_quote_single,
    str_quote_single,
    str_quote_grave,
    str_quote_double,
    str_slash_1,
    str_slash_2,
    str_slash_3,
    str_slash_4,
    empty,
    str_b,
    str_content,
    str_content_quote_single,
    str_quote_single,
    str_quote_grave,
    str_quote_double,
    str_slash_1,
    str_slash_2,
    str_slash_3,
    str_slash_4,
    empty,
    str_b,
    str_content,
    str_content_quote_grave,
    str_quote_single,
    str_quote_grave,
    str_quote_double,
    str_slash_1,
    str_slash_2,
    str_slash_3,
    str_slash_4,
    empty,
    str_b,
    str_content,
    str_content_quote_grave,
    str_quote_single,
    str_quote_grave,
    str_quote_double,
    str_slash_1,
    str_slash_2,
    str_slash_3,
    str_slash_4,
    empty,
    str_b,
    str_content,
    str_content_quote_grave,
    str_quote_single,
    str_quote_grave,
    str_quote_double,
    str_slash_1,
    str_slash_2,
    str_slash_3,
    str_slash_4,
    empty,
    str_b,
    str_content,
    str_content_quote_double,
    str_quote_single,
    str_quote_grave,
    str_quote_double,
    str_slash_1,
    str_slash_2,
    str_slash_3,
    str_slash_4,
    empty,
    str_b,
    str_content,
    str_content_quote_double,
    str_quote_single,
    str_quote_grave,
    str_quote_double,
    str_slash_1,
    str_slash_2,
    str_slash_3,
    str_slash_4,
    empty,
    str_b,
    str_content,
    str_content_quote_double,
    str_quote_single,
    str_quote_grave,
    str_quote_double,
    str_slash_1,
    str_slash_2,
    str_slash_3,
    str_slash_4,
    empty,
    str_b,
    str_content,
    str_content_quote_single,
    str_quote_single,
    str_quote_grave,
    str_quote_double,
    str_slash_1,
    str_slash_2,
    str_slash_3,
    str_slash_4,
    empty,
    str_b,
    str_content,
    str_content_quote_single,
    str_quote_single,
    str_quote_grave,
    str_quote_double,
    str_slash_1,
    str_slash_2,
    str_slash_3,
    str_slash_4,
    empty,
    str_b,
    str_content,
    str_content_quote_single,
    str_quote_single,
    str_quote_grave,
    str_quote_double,
    str_slash_1,
    str_slash_2,
    str_slash_3,
    str_slash_4,
    empty,
    str_b,
    str_content,
    str_content_quote_grave,
    str_quote_single,
    str_quote_grave,
    str_quote_double,
    str_slash_1,
    str_slash_2,
    str_slash_3,
    str_slash_4,
    empty,
    str_b,
    str_content,
    str_content_quote_grave,
    str_quote_single,
    str_quote_grave,
    str_quote_double,
    str_slash_1,
    str_slash_2,
    str_slash_3,
    str_slash_4,
    empty,
    str_b,
    str_content,
    str_content_quote_grave,
    str_quote_single,
    str_quote_grave,
    str_quote_double,
    str_slash_1,
    str_slash_2,
    str_slash_3,
    str_slash_4,
    empty,
    str_b,
    str_content,
    str_content_quote_double,
    str_quote_single,
    str_quote_grave,
    str_quote_double,
    str_slash_1,
    str_slash_2,
    str_slash_3,
    str_slash_4,
    empty,
    str_b,
    str_content,
    str_content_quote_double,
    str_quote_single,
    str_quote_grave,
    str_quote_double,
    str_slash_1,
    str_slash_2,
    str_slash_3,
    str_slash_4,
    empty,
    str_b,
    str_content,
    str_content_quote_double,
    str_quote_single,
    str_quote_grave,
    str_quote_double,
    str_slash_1,
    str_slash_2,
    str_slash_3,
    str_slash_4,
  };

  f_string_static_t quotes[] = {
    f_iki_syntax_quote_single_s,
    f_iki_syntax_quote_single_s,
    f_iki_syntax_quote_single_s,
    f_iki_syntax_quote_single_s,
    f_iki_syntax_quote_single_s,
    f_iki_syntax_quote_single_s,
    f_iki_syntax_quote_single_s,
    f_iki_syntax_quote_single_s,
    f_iki_syntax_quote_single_s,
    f_iki_syntax_quote_single_s,
    f_iki_syntax_quote_single_s,
    f_iki_syntax_quote_single_s,
    f_iki_syntax_quote_single_s,
    f_iki_syntax_quote_single_s,
    f_iki_syntax_quote_single_s,
    f_iki_syntax_quote_single_s,
    f_iki_syntax_quote_single_s,
    f_iki_syntax_quote_single_s,
    f_iki_syntax_quote_single_s,
    f_iki_syntax_quote_single_s,
    f_iki_syntax_quote_single_s,
    f_iki_syntax_quote_single_s,
    f_iki_syntax_quote_single_s,
    f_iki_syntax_quote_single_s,
    f_iki_syntax_quote_single_s,
    f_iki_syntax_quote_single_s,
    f_iki_syntax_quote_single_s,
    f_iki_syntax_quote_single_s,
    f_iki_syntax_quote_single_s,
    f_iki_syntax_quote_single_s,
    f_iki_syntax_quote_single_s,
    f_iki_syntax_quote_single_s,
    f_iki_syntax_quote_single_s,
    f_iki_syntax_quote_grave_s,
    f_iki_syntax_quote_grave_s,
    f_iki_syntax_quote_grave_s,
    f_iki_syntax_quote_grave_s,
    f_iki_syntax_quote_grave_s,
    f_iki_syntax_quote_grave_s,
    f_iki_syntax_quote_grave_s,
    f_iki_syntax_quote_grave_s,
    f_iki_syntax_quote_grave_s,
    f_iki_syntax_quote_grave_s,
    f_iki_syntax_quote_grave_s,
    f_iki_syntax_quote_grave_s,
    f_iki_syntax_quote_grave_s,
    f_iki_syntax_quote_grave_s,
    f_iki_syntax_quote_grave_s,
    f_iki_syntax_quote_grave_s,
    f_iki_syntax_quote_grave_s,
    f_iki_syntax_quote_grave_s,
    f_iki_syntax_quote_grave_s,
    f_iki_syntax_quote_grave_s,
    f_iki_syntax_quote_grave_s,
    f_iki_syntax_quote_grave_s,
    f_iki_syntax_quote_grave_s,
    f_iki_syntax_quote_grave_s,
    f_iki_syntax_quote_grave_s,
    f_iki_syntax_quote_grave_s,
    f_iki_syntax_quote_grave_s,
    f_iki_syntax_quote_grave_s,
    f_iki_syntax_quote_grave_s,
    f_iki_syntax_quote_grave_s,
    f_iki_syntax_quote_grave_s,
    f_iki_syntax_quote_grave_s,
    f_iki_syntax_quote_grave_s,
    f_iki_syntax_quote_double_s,
    f_iki_syntax_quote_double_s,
    f_iki_syntax_quote_double_s,
    f_iki_syntax_quote_double_s,
    f_iki_syntax_quote_double_s,
    f_iki_syntax_quote_double_s,
    f_iki_syntax_quote_double_s,
    f_iki_syntax_quote_double_s,
    f_iki_syntax_quote_double_s,
    f_iki_syntax_quote_double_s,
    f_iki_syntax_quote_double_s,
    f_iki_syntax_quote_double_s,
    f_iki_syntax_quote_double_s,
    f_iki_syntax_quote_double_s,
    f_iki_syntax_quote_double_s,
    f_iki_syntax_quote_double_s,
    f_iki_syntax_quote_double_s,
    f_iki_syntax_quote_double_s,
    f_iki_syntax_quote_double_s,
    f_iki_syntax_quote_double_s,
    f_iki_syntax_quote_double_s,
    f_iki_syntax_quote_double_s,
    f_iki_syntax_quote_double_s,
    f_iki_syntax_quote_double_s,
    f_iki_syntax_quote_double_s,
    f_iki_syntax_quote_double_s,
    f_iki_syntax_quote_double_s,
    f_iki_syntax_quote_double_s,
    f_iki_syntax_quote_double_s,
    f_iki_syntax_quote_double_s,
    f_iki_syntax_quote_double_s,
    f_iki_syntax_quote_double_s,
    f_iki_syntax_quote_double_s,
    f_iki_syntax_quote_single_s,
    f_iki_syntax_quote_single_s,
    f_iki_syntax_quote_single_s,
    f_iki_syntax_quote_single_s,
    f_iki_syntax_quote_single_s,
    f_iki_syntax_quote_single_s,
    f_iki_syntax_quote_single_s,
    f_iki_syntax_quote_single_s,
    f_iki_syntax_quote_single_s,
    f_iki_syntax_quote_single_s,
    f_iki_syntax_quote_single_s,
    f_iki_syntax_quote_single_s,
    f_iki_syntax_quote_single_s,
    f_iki_syntax_quote_single_s,
    f_iki_syntax_quote_single_s,
    f_iki_syntax_quote_single_s,
    f_iki_syntax_quote_single_s,
    f_iki_syntax_quote_single_s,
    f_iki_syntax_quote_single_s,
    f_iki_syntax_quote_single_s,
    f_iki_syntax_quote_single_s,
    f_iki_syntax_quote_single_s,
    f_iki_syntax_quote_single_s,
    f_iki_syntax_quote_single_s,
    f_iki_syntax_quote_single_s,
    f_iki_syntax_quote_single_s,
    f_iki_syntax_quote_single_s,
    f_iki_syntax_quote_single_s,
    f_iki_syntax_quote_single_s,
    f_iki_syntax_quote_single_s,
    f_iki_syntax_quote_single_s,
    f_iki_syntax_quote_single_s,
    f_iki_syntax_quote_single_s,
    f_iki_syntax_quote_grave_s,
    f_iki_syntax_quote_grave_s,
    f_iki_syntax_quote_grave_s,
    f_iki_syntax_quote_grave_s,
    f_iki_syntax_quote_grave_s,
    f_iki_syntax_quote_grave_s,
    f_iki_syntax_quote_grave_s,
    f_iki_syntax_quote_grave_s,
    f_iki_syntax_quote_grave_s,
    f_iki_syntax_quote_grave_s,
    f_iki_syntax_quote_grave_s,
    f_iki_syntax_quote_grave_s,
    f_iki_syntax_quote_grave_s,
    f_iki_syntax_quote_grave_s,
    f_iki_syntax_quote_grave_s,
    f_iki_syntax_quote_grave_s,
    f_iki_syntax_quote_grave_s,
    f_iki_syntax_quote_grave_s,
    f_iki_syntax_quote_grave_s,
    f_iki_syntax_quote_grave_s,
    f_iki_syntax_quote_grave_s,
    f_iki_syntax_quote_grave_s,
    f_iki_syntax_quote_grave_s,
    f_iki_syntax_quote_grave_s,
    f_iki_syntax_quote_grave_s,
    f_iki_syntax_quote_grave_s,
    f_iki_syntax_quote_grave_s,
    f_iki_syntax_quote_grave_s,
    f_iki_syntax_quote_grave_s,
    f_iki_syntax_quote_grave_s,
    f_iki_syntax_quote_grave_s,
    f_iki_syntax_quote_grave_s,
    f_iki_syntax_quote_grave_s,
    f_iki_syntax_quote_double_s,
    f_iki_syntax_quote_double_s,
    f_iki_syntax_quote_double_s,
    f_iki_syntax_quote_double_s,
    f_iki_syntax_quote_double_s,
    f_iki_syntax_quote_double_s,
    f_iki_syntax_quote_double_s,
    f_iki_syntax_quote_double_s,
    f_iki_syntax_quote_double_s,
    f_iki_syntax_quote_double_s,
    f_iki_syntax_quote_double_s,
    f_iki_syntax_quote_double_s,
    f_iki_syntax_quote_double_s,
    f_iki_syntax_quote_double_s,
    f_iki_syntax_quote_double_s,
    f_iki_syntax_quote_double_s,
    f_iki_syntax_quote_double_s,
    f_iki_syntax_quote_double_s,
    f_iki_syntax_quote_double_s,
    f_iki_syntax_quote_double_s,
    f_iki_syntax_quote_double_s,
    f_iki_syntax_quote_double_s,
    f_iki_syntax_quote_double_s,
    f_iki_syntax_quote_double_s,
    f_iki_syntax_quote_double_s,
    f_iki_syntax_quote_double_s,
    f_iki_syntax_quote_double_s,
    f_iki_syntax_quote_double_s,
    f_iki_syntax_quote_double_s,
    f_iki_syntax_quote_double_s,
    f_iki_syntax_quote_double_s,
    f_iki_syntax_quote_double_s,
    f_iki_syntax_quote_double_s,
  };

  uint64_t flags[] = {
    0,
    0,
    0,
    0,
    0,
    0,
    0,
    0,
    0,
    0,
    0,
    0,
    0,
    0,
    0,
    0,
    0,
    0,
    0,
    0,
    0,
    0,
    0,
    0,
    0,
    0,
    0,
    0,
    0,
    0,
    0,
    0,
    0,
    0,
    0,
    0,
    0,
    0,
    0,
    0,
    0,
    0,
    0,
    0,
    0,
    0,
    0,
    0,
    0,
    0,
    0,
    0,
    0,
    0,
    0,
    0,
    0,
    0,
    0,
    0,
    0,
    0,
    0,
    0,
    0,
    0,
    0,
    0,
    0,
    0,
    0,
    0,
    0,
    0,
    0,
    0,
    0,
    0,
    0,
    0,
    0,
    0,
    0,
    0,
    0,
    0,
    0,
    0,
    0,
    0,
    0,
    0,
    0,
    0,
    0,
    0,
    0,
    0,
    0,
    f_iki_state_flag_force_wrap_e,
    f_iki_state_flag_force_wrap_e,
    f_iki_state_flag_force_wrap_e,
    f_iki_state_flag_force_wrap_e,
    f_iki_state_flag_force_wrap_e,
    f_iki_state_flag_force_wrap_e,
    f_iki_state_flag_force_wrap_e,
    f_iki_state_flag_force_wrap_e,
    f_iki_state_flag_force_wrap_e,
    f_iki_state_flag_force_wrap_e,
    f_iki_state_flag_force_wrap_e,
    f_iki_state_flag_force_wrap_e,
    f_iki_state_flag_force_wrap_e,
    f_iki_state_flag_force_wrap_e,
    f_iki_state_flag_force_wrap_e,
    f_iki_state_flag_force_wrap_e,
    f_iki_state_flag_force_wrap_e,
    f_iki_state_flag_force_wrap_e,
    f_iki_state_flag_force_wrap_e,
    f_iki_state_flag_force_wrap_e,
    f_iki_state_flag_force_wrap_e,
    f_iki_state_flag_force_wrap_e,
    f_iki_state_flag_force_wrap_e,
    f_iki_state_flag_force_wrap_e,
    f_iki_state_flag_force_wrap_e,
    f_iki_state_flag_force_wrap_e,
    f_iki_state_flag_force_wrap_e,
    f_iki_state_flag_force_wrap_e,
    f_iki_state_flag_force_wrap_e,
    f_iki_state_flag_force_wrap_e,
    f_iki_state_flag_force_wrap_e,
    f_iki_state_flag_force_wrap_e,
    f_iki_state_flag_force_wrap_e,
    f_iki_state_flag_force_wrap_e,
    f_iki_state_flag_force_wrap_e,
    f_iki_state_flag_force_wrap_e,
    f_iki_state_flag_force_wrap_e,
    f_iki_state_flag_force_wrap_e,
    f_iki_state_flag_force_wrap_e,
    f_iki_state_flag_force_wrap_e,
    f_iki_state_flag_force_wrap_e,
    f_iki_state_flag_force_wrap_e,
    f_iki_state_flag_force_wrap_e,
    f_iki_state_flag_force_wrap_e,
    f_iki_state_flag_force_wrap_e,
    f_iki_state_flag_force_wrap_e,
    f_iki_state_flag_force_wrap_e,
    f_iki_state_flag_force_wrap_e,
    f_iki_state_flag_force_wrap_e,
    f_iki_state_flag_force_wrap_e,
    f_iki_state_flag_force_wrap_e,
    f_iki_state_flag_force_wrap_e,
    f_iki_state_flag_force_wrap_e,
    f_iki_state_flag_force_wrap_e,
    f_iki_state_flag_force_wrap_e,
    f_iki_state_flag_force_wrap_e,
    f_iki_state_flag_force_wrap_e,
    f_iki_state_flag_force_wrap_e,
    f_iki_state_flag_force_wrap_e,
    f_iki_state_flag_force_wrap_e,
    f_iki_state_flag_force_wrap_e,
    f_iki_state_flag_force_wrap_e,
    f_iki_state_flag_force_wrap_e,
    f_iki_state_flag_force_wrap_e,
    f_iki_state_flag_force_wrap_e,
    f_iki_state_flag_force_wrap_e,
    f_iki_state_flag_force_wrap_e,
    f_iki_state_flag_force_wrap_e,
    f_iki_state_flag_force_wrap_e,
    f_iki_state_flag_force_wrap_e,
    f_iki_state_flag_force_wrap_e,
    f_iki_state_flag_force_wrap_e,
    f_iki_state_flag_force_wrap_e,
    f_iki_state_flag_force_wrap_e,
    f_iki_state_flag_force_wrap_e,
    f_iki_state_flag_force_wrap_e,
    f_iki_state_flag_force_wrap_e,
    f_iki_state_flag_force_wrap_e,
    f_iki_state_flag_force_wrap_e,
    f_iki_state_flag_force_wrap_e,
    f_iki_state_flag_force_wrap_e,
    f_iki_state_flag_force_wrap_e,
    f_iki_state_flag_force_wrap_e,
    f_iki_state_flag_force_wrap_e,
    f_iki_state_flag_force_wrap_e,
    f_iki_state_flag_force_wrap_e,
    f_iki_state_flag_force_wrap_e,
    f_iki_state_flag_force_wrap_e,
    f_iki_state_flag_force_wrap_e,
    f_iki_state_flag_force_wrap_e,
    f_iki_state_flag_force_wrap_e,
    f_iki_state_flag_force_wrap_e,
    f_iki_state_flag_force_wrap_e,
    f_iki_state_flag_force_wrap_e,
    f_iki_state_flag_force_wrap_e,
    f_iki_state_flag_force_wrap_e,
    f_iki_state_flag_force_wrap_e,
    f_iki_state_flag_force_wrap_e,
    f_iki_state_flag_force_wrap_e,
  };

  f_string_static_t ikis[] = {
    macro_f_string_static_t_initialize_1("a:b:''", 0, 6),
    macro_f_string_static_t_initialize_1("a:b:'b'", 0, 7),
    macro_f_string_static_t_initialize_1("a:b:'content'", 0, 13),
    macro_f_string_static_t_initialize_1("a:b:'\\'content\\''", 0, 17),
    macro_f_string_static_t_initialize_1("a:b:'quote\\'s'", 0, 14),
    macro_f_string_static_t_initialize_1("a:b:'quote`s'", 0, 13),
    macro_f_string_static_t_initialize_1("a:b:'quote\"s'", 0, 13),
    macro_f_string_static_t_initialize_1("a:b:'slash\\\\'", 0, 13),
    macro_f_string_static_t_initialize_1("a:b:'slash\\\\\\\\'", 0, 15),
    macro_f_string_static_t_initialize_1("a:b:'slash\\\\\\\\\\\\'", 0, 17),
    macro_f_string_static_t_initialize_1("a:b:'slash\\\\\\\\\\\\\\\\'", 0, 19),
    macro_f_string_static_t_initialize_1("iki:sea:''", 0, 10),
    macro_f_string_static_t_initialize_1("iki:sea:'b'", 0, 11),
    macro_f_string_static_t_initialize_1("iki:sea:'content'", 0, 17),
    macro_f_string_static_t_initialize_1("iki:sea:'\\'content\\''", 0, 21),
    macro_f_string_static_t_initialize_1("iki:sea:'quote\\'s'", 0, 18),
    macro_f_string_static_t_initialize_1("iki:sea:'quote`s'", 0, 17),
    macro_f_string_static_t_initialize_1("iki:sea:'quote\"s'", 0, 17),
    macro_f_string_static_t_initialize_1("iki:sea:'slash\\\\'", 0, 17),
    macro_f_string_static_t_initialize_1("iki:sea:'slash\\\\\\\\'", 0, 19),
    macro_f_string_static_t_initialize_1("iki:sea:'slash\\\\\\\\\\\\'", 0, 21),
    macro_f_string_static_t_initialize_1("iki:sea:'slash\\\\\\\\\\\\\\\\'", 0, 23),
    macro_f_string_static_t_initialize_1("has_CHARS+-:b:sea:''", 0, 20),
    macro_f_string_static_t_initialize_1("has_CHARS+-:b:sea:'b'", 0, 21),
    macro_f_string_static_t_initialize_1("has_CHARS+-:b:sea:'content'", 0, 27),
    macro_f_string_static_t_initialize_1("has_CHARS+-:b:sea:'\\'content\\''", 0, 31),
    macro_f_string_static_t_initialize_1("has_CHARS+-:b:sea:'quote\\'s'", 0, 28),
    macro_f_string_static_t_initialize_1("has_CHARS+-:b:sea:'quote`s'", 0, 27),
    macro_f_string_static_t_initialize_1("has_CHARS+-:b:sea:'quote\"s'", 0, 27),
    macro_f_string_static_t_initialize_1("has_CHARS+-:b:sea:'slash\\\\'", 0, 27),
    macro_f_string_static_t_initialize_1("has_CHARS+-:b:sea:'slash\\\\\\\\'", 0, 29),
    macro_f_string_static_t_initialize_1("has_CHARS+-:b:sea:'slash\\\\\\\\\\\\'", 0, 31),
    macro_f_string_static_t_initialize_1("has_CHARS+-:b:sea:'slash\\\\\\\\\\\\\\\\'", 0, 33),
    macro_f_string_static_t_initialize_1("a:b:``", 0, 6),
    macro_f_string_static_t_initialize_1("a:b:`b`", 0, 7),
    macro_f_string_static_t_initialize_1("a:b:`content`", 0, 13),
    macro_f_string_static_t_initialize_1("a:b:`\\`content\\``", 0, 17),
    macro_f_string_static_t_initialize_1("a:b:`quote's`", 0, 13),
    macro_f_string_static_t_initialize_1("a:b:`quote\\`s`", 0, 14),
    macro_f_string_static_t_initialize_1("a:b:`quote\"s`", 0, 13),
    macro_f_string_static_t_initialize_1("a:b:`slash\\\\`", 0, 13),
    macro_f_string_static_t_initialize_1("a:b:`slash\\\\\\\\`", 0, 15),
    macro_f_string_static_t_initialize_1("a:b:`slash\\\\\\\\\\\\`", 0, 17),
    macro_f_string_static_t_initialize_1("a:b:`slash\\\\\\\\\\\\\\\\`", 0, 19),
    macro_f_string_static_t_initialize_1("iki:sea:``", 0, 10),
    macro_f_string_static_t_initialize_1("iki:sea:`b`", 0, 11),
    macro_f_string_static_t_initialize_1("iki:sea:`content`", 0, 17),
    macro_f_string_static_t_initialize_1("iki:sea:`\\`content\\``", 0, 21),
    macro_f_string_static_t_initialize_1("iki:sea:`quote's`", 0, 17),
    macro_f_string_static_t_initialize_1("iki:sea:`quote\\`s`", 0, 18),
    macro_f_string_static_t_initialize_1("iki:sea:`quote\"s`", 0, 17),
    macro_f_string_static_t_initialize_1("iki:sea:`slash\\\\`", 0, 17),
    macro_f_string_static_t_initialize_1("iki:sea:`slash\\\\\\\\`", 0, 19),
    macro_f_string_static_t_initialize_1("iki:sea:`slash\\\\\\\\\\\\`", 0, 21),
    macro_f_string_static_t_initialize_1("iki:sea:`slash\\\\\\\\\\\\\\\\`", 0, 23),
    macro_f_string_static_t_initialize_1("has_CHARS+-:b:sea:``", 0, 20),
    macro_f_string_static_t_initialize_1("has_CHARS+-:b:sea:`b`", 0, 21),
    macro_f_string_static_t_initialize_1("has_CHARS+-:b:sea:`content`", 0, 27),
    macro_f_string_static_t_initialize_1("has_CHARS+-:b:sea:`\\`content\\``", 0, 31),
    macro_f_string_static_t_initialize_1("has_CHARS+-:b:sea:`quote's`", 0, 27),
    macro_f_string_static_t_initialize_1("has_CHARS+-:b:sea:`quote\\`s`", 0, 28),
    macro_f_string_static_t_initialize_1("has_CHARS+-:b:sea:`quote\"s`", 0, 27),
    macro_f_string_static_t_initialize_1("has_CHARS+-:b:sea:`slash\\\\`", 0, 27),
    macro_f_string_static_t_initialize_1("has_CHARS+-:b:sea:`slash\\\\\\\\`", 0, 29),
    macro_f_string_static_t_initialize_1("has_CHARS+-:b:sea:`slash\\\\\\\\\\\\`", 0, 31),
    macro_f_string_static_t_initialize_1("has_CHARS+-:b:sea:`slash\\\\\\\\\\\\\\\\`", 0, 33),
    macro_f_string_static_t_initialize_1("a:b:\"\"", 0, 6),
    macro_f_string_static_t_initialize_1("a:b:\"b\"", 0, 7),
    macro_f_string_static_t_initialize_1("a:b:\"content\"", 0, 13),
    macro_f_string_static_t_initialize_1("a:b:\"\\\"content\\\"\"", 0, 17),
    macro_f_string_static_t_initialize_1("a:b:\"quote's\"", 0, 13),
    macro_f_string_static_t_initialize_1("a:b:\"quote`s\"", 0, 13),
    macro_f_string_static_t_initialize_1("a:b:\"quote\\\"s\"", 0, 14),
    macro_f_string_static_t_initialize_1("a:b:\"slash\\\\\"", 0, 13),
    macro_f_string_static_t_initialize_1("a:b:\"slash\\\\\\\\\"", 0, 15),
    macro_f_string_static_t_initialize_1("a:b:\"slash\\\\\\\\\\\\\"", 0, 17),
    macro_f_string_static_t_initialize_1("a:b:\"slash\\\\\\\\\\\\\\\\\"", 0, 19),
    macro_f_string_static_t_initialize_1("iki:sea:\"\"", 0, 10),
    macro_f_string_static_t_initialize_1("iki:sea:\"b\"", 0, 11),
    macro_f_string_static_t_initialize_1("iki:sea:\"content\"", 0, 17),
    macro_f_string_static_t_initialize_1("iki:sea:\"\\\"content\\\"\"", 0, 21),
    macro_f_string_static_t_initialize_1("iki:sea:\"quote's\"", 0, 17),
    macro_f_string_static_t_initialize_1("iki:sea:\"quote`s\"", 0, 17),
    macro_f_string_static_t_initialize_1("iki:sea:\"quote\\\"s\"", 0, 18),
    macro_f_string_static_t_initialize_1("iki:sea:\"slash\\\\\"", 0, 17),
    macro_f_string_static_t_initialize_1("iki:sea:\"slash\\\\\\\\\"", 0, 19),
    macro_f_string_static_t_initialize_1("iki:sea:\"slash\\\\\\\\\\\\\"", 0, 21),
    macro_f_string_static_t_initialize_1("iki:sea:\"slash\\\\\\\\\\\\\\\\\"", 0, 23),
    macro_f_string_static_t_initialize_1("has_CHARS+-:b:sea:\"\"", 0, 20),
    macro_f_string_static_t_initialize_1("has_CHARS+-:b:sea:\"b\"", 0, 21),
    macro_f_string_static_t_initialize_1("has_CHARS+-:b:sea:\"content\"", 0, 27),
    macro_f_string_static_t_initialize_1("has_CHARS+-:b:sea:\"\\\"content\\\"\"", 0, 31),
    macro_f_string_static_t_initialize_1("has_CHARS+-:b:sea:\"quote's\"", 0, 27),
    macro_f_string_static_t_initialize_1("has_CHARS+-:b:sea:\"quote`s\"", 0, 27),
    macro_f_string_static_t_initialize_1("has_CHARS+-:b:sea:\"quote\\\"s\"", 0, 28),
    macro_f_string_static_t_initialize_1("has_CHARS+-:b:sea:\"slash\\\\\"", 0, 27),
    macro_f_string_static_t_initialize_1("has_CHARS+-:b:sea:\"slash\\\\\\\\\"", 0, 29),
    macro_f_string_static_t_initialize_1("has_CHARS+-:b:sea:\"slash\\\\\\\\\\\\\"", 0, 31),
    macro_f_string_static_t_initialize_1("has_CHARS+-:b:sea:\"slash\\\\\\\\\\\\\\\\\"", 0, 33),
    macro_f_string_static_t_initialize_1("[a:b]:''", 0, 8),
    macro_f_string_static_t_initialize_1("[a:b]:'b'", 0, 9),
    macro_f_string_static_t_initialize_1("[a:b]:'content'", 0, 15),
    macro_f_string_static_t_initialize_1("[a:b]:'\\'content\\''", 0, 19),
    macro_f_string_static_t_initialize_1("[a:b]:'quote\\'s'", 0, 16),
    macro_f_string_static_t_initialize_1("[a:b]:'quote`s'", 0, 15),
    macro_f_string_static_t_initialize_1("[a:b]:'quote\"s'", 0, 15),
    macro_f_string_static_t_initialize_1("[a:b]:'slash\\\\'", 0, 15),
    macro_f_string_static_t_initialize_1("[a:b]:'slash\\\\\\\\'", 0, 17),
    macro_f_string_static_t_initialize_1("[a:b]:'slash\\\\\\\\\\\\'", 0, 19),
    macro_f_string_static_t_initialize_1("[a:b]:'slash\\\\\\\\\\\\\\\\'", 0, 21),
    macro_f_string_static_t_initialize_1("[iki:sea]:''", 0, 12),
    macro_f_string_static_t_initialize_1("[iki:sea]:'b'", 0, 13),
    macro_f_string_static_t_initialize_1("[iki:sea]:'content'", 0, 19),
    macro_f_string_static_t_initialize_1("[iki:sea]:'\\'content\\''", 0, 23),
    macro_f_string_static_t_initialize_1("[iki:sea]:'quote\\'s'", 0, 20),
    macro_f_string_static_t_initialize_1("[iki:sea]:'quote`s'", 0, 19),
    macro_f_string_static_t_initialize_1("[iki:sea]:'quote\"s'", 0, 19),
    macro_f_string_static_t_initialize_1("[iki:sea]:'slash\\\\'", 0, 19),
    macro_f_string_static_t_initialize_1("[iki:sea]:'slash\\\\\\\\'", 0, 21),
    macro_f_string_static_t_initialize_1("[iki:sea]:'slash\\\\\\\\\\\\'", 0, 23),
    macro_f_string_static_t_initialize_1("[iki:sea]:'slash\\\\\\\\\\\\\\\\'", 0, 25),
    macro_f_string_static_t_initialize_1("[has_CHARS+-:b:sea]:''", 0, 22),
    macro_f_string_static_t_initialize_1("[has_CHARS+-:b:sea]:'b'", 0, 23),
    macro_f_string_static_t_initialize_1("[has_CHARS+-:b:sea]:'content'", 0, 29),
    macro_f_string_static_t_initialize_1("[has_CHARS+-:b:sea]:'\\'content\\''", 0, 33),
    macro_f_string_static_t_initialize_1("[has_CHARS+-:b:sea]:'quote\\'s'", 0, 30),
    macro_f_string_static_t_initialize_1("[has_CHARS+-:b:sea]:'quote`s'", 0, 29),
    macro_f_string_static_t_initialize_1("[has_CHARS+-:b:sea]:'quote\"s'", 0, 29),
    macro_f_string_static_t_initialize_1("[has_CHARS+-:b:sea]:'slash\\\\'", 0, 29),
    macro_f_string_static_t_initialize_1("[has_CHARS+-:b:sea]:'slash\\\\\\\\'", 0, 31),
    macro_f_string_static_t_initialize_1("[has_CHARS+-:b:sea]:'slash\\\\\\\\\\\\'", 0, 33),
    macro_f_string_static_t_initialize_1("[has_CHARS+-:b:sea]:'slash\\\\\\\\\\\\\\\\'", 0, 35),
    macro_f_string_static_t_initialize_1("[a:b]:``", 0, 8),
    macro_f_string_static_t_initialize_1("[a:b]:`b`", 0, 9),
    macro_f_string_static_t_initialize_1("[a:b]:`content`", 0, 15),
    macro_f_string_static_t_initialize_1("[a:b]:`\\`content\\``", 0, 19),
    macro_f_string_static_t_initialize_1("[a:b]:`quote's`", 0, 15),
    macro_f_string_static_t_initialize_1("[a:b]:`quote\\`s`", 0, 16),
    macro_f_string_static_t_initialize_1("[a:b]:`quote\"s`", 0, 15),
    macro_f_string_static_t_initialize_1("[a:b]:`slash\\\\`", 0, 15),
    macro_f_string_static_t_initialize_1("[a:b]:`slash\\\\\\\\`", 0, 17),
    macro_f_string_static_t_initialize_1("[a:b]:`slash\\\\\\\\\\\\`", 0, 19),
    macro_f_string_static_t_initialize_1("[a:b]:`slash\\\\\\\\\\\\\\\\`", 0, 21),
    macro_f_string_static_t_initialize_1("[iki:sea]:``", 0, 12),
    macro_f_string_static_t_initialize_1("[iki:sea]:`b`", 0, 13),
    macro_f_string_static_t_initialize_1("[iki:sea]:`content`", 0, 19),
    macro_f_string_static_t_initialize_1("[iki:sea]:`\\`content\\``", 0, 23),
    macro_f_string_static_t_initialize_1("[iki:sea]:`quote's`", 0, 19),
    macro_f_string_static_t_initialize_1("[iki:sea]:`quote\\`s`", 0, 20),
    macro_f_string_static_t_initialize_1("[iki:sea]:`quote\"s`", 0, 19),
    macro_f_string_static_t_initialize_1("[iki:sea]:`slash\\\\`", 0, 19),
    macro_f_string_static_t_initialize_1("[iki:sea]:`slash\\\\\\\\`", 0, 21),
    macro_f_string_static_t_initialize_1("[iki:sea]:`slash\\\\\\\\\\\\`", 0, 23),
    macro_f_string_static_t_initialize_1("[iki:sea]:`slash\\\\\\\\\\\\\\\\`", 0, 25),
    macro_f_string_static_t_initialize_1("[has_CHARS+-:b:sea]:``", 0, 22),
    macro_f_string_static_t_initialize_1("[has_CHARS+-:b:sea]:`b`", 0, 23),
    macro_f_string_static_t_initialize_1("[has_CHARS+-:b:sea]:`content`", 0, 29),
    macro_f_string_static_t_initialize_1("[has_CHARS+-:b:sea]:`\\`content\\``", 0, 33),
    macro_f_string_static_t_initialize_1("[has_CHARS+-:b:sea]:`quote's`", 0, 29),
    macro_f_string_static_t_initialize_1("[has_CHARS+-:b:sea]:`quote\\`s`", 0, 30),
    macro_f_string_static_t_initialize_1("[has_CHARS+-:b:sea]:`quote\"s`", 0, 29),
    macro_f_string_static_t_initialize_1("[has_CHARS+-:b:sea]:`slash\\\\`", 0, 29),
    macro_f_string_static_t_initialize_1("[has_CHARS+-:b:sea]:`slash\\\\\\\\`", 0, 31),
    macro_f_string_static_t_initialize_1("[has_CHARS+-:b:sea]:`slash\\\\\\\\\\\\`", 0, 33),
    macro_f_string_static_t_initialize_1("[has_CHARS+-:b:sea]:`slash\\\\\\\\\\\\\\\\`", 0, 35),
    macro_f_string_static_t_initialize_1("[a:b]:\"\"", 0, 8),
    macro_f_string_static_t_initialize_1("[a:b]:\"b\"", 0, 9),
    macro_f_string_static_t_initialize_1("[a:b]:\"content\"", 0, 15),
    macro_f_string_static_t_initialize_1("[a:b]:\"\\\"content\\\"\"", 0, 19),
    macro_f_string_static_t_initialize_1("[a:b]:\"quote's\"", 0, 15),
    macro_f_string_static_t_initialize_1("[a:b]:\"quote`s\"", 0, 15),
    macro_f_string_static_t_initialize_1("[a:b]:\"quote\\\"s\"", 0, 16),
    macro_f_string_static_t_initialize_1("[a:b]:\"slash\\\\\"", 0, 15),
    macro_f_string_static_t_initialize_1("[a:b]:\"slash\\\\\\\\\"", 0, 17),
    macro_f_string_static_t_initialize_1("[a:b]:\"slash\\\\\\\\\\\\\"", 0, 19),
    macro_f_string_static_t_initialize_1("[a:b]:\"slash\\\\\\\\\\\\\\\\\"", 0, 21),
    macro_f_string_static_t_initialize_1("[iki:sea]:\"\"", 0, 12),
    macro_f_string_static_t_initialize_1("[iki:sea]:\"b\"", 0, 13),
    macro_f_string_static_t_initialize_1("[iki:sea]:\"content\"", 0, 19),
    macro_f_string_static_t_initialize_1("[iki:sea]:\"\\\"content\\\"\"", 0, 23),
    macro_f_string_static_t_initialize_1("[iki:sea]:\"quote's\"", 0, 19),
    macro_f_string_static_t_initialize_1("[iki:sea]:\"quote`s\"", 0, 19),
    macro_f_string_static_t_initialize_1("[iki:sea]:\"quote\\\"s\"", 0, 20),
    macro_f_string_static_t_initialize_1("[iki:sea]:\"slash\\\\\"", 0, 19),
    macro_f_string_static_t_initialize_1("[iki:sea]:\"slash\\\\\\\\\"", 0, 21),
    macro_f_string_static_t_initialize_1("[iki:sea]:\"slash\\\\\\\\\\\\\"", 0, 23),
    macro_f_string_static_t_initialize_1("[iki:sea]:\"slash\\\\\\\\\\\\\\\\\"", 0, 25),
    macro_f_string_static_t_initialize_1("[has_CHARS+-:b:sea]:\"\"", 0, 22),
    macro_f_string_static_t_initialize_1("[has_CHARS+-:b:sea]:\"b\"", 0, 23),
    macro_f_string_static_t_initialize_1("[has_CHARS+-:b:sea]:\"content\"", 0, 29),
    macro_f_string_static_t_initialize_1("[has_CHARS+-:b:sea]:\"\\\"content\\\"\"", 0, 33),
    macro_f_string_static_t_initialize_1("[has_CHARS+-:b:sea]:\"quote's\"", 0, 29),
    macro_f_string_static_t_initialize_1("[has_CHARS+-:b:sea]:\"quote`s\"", 0, 29),
    macro_f_string_static_t_initialize_1("[has_CHARS+-:b:sea]:\"quote\\\"s\"", 0, 30),
    macro_f_string_static_t_initialize_1("[has_CHARS+-:b:sea]:\"slash\\\\\"", 0, 29),
    macro_f_string_static_t_initialize_1("[has_CHARS+-:b:sea]:\"slash\\\\\\\\\"", 0, 31),
    macro_f_string_static_t_initialize_1("[has_CHARS+-:b:sea]:\"slash\\\\\\\\\\\\\"", 0, 33),
    macro_f_string_static_t_initialize_1("[has_CHARS+-:b:sea]:\"slash\\\\\\\\\\\\\\\\\"", 0, 35),
  };

  const f_status_t statuss[] = {
    F_okay,
    F_okay,
    F_okay,
    F_okay,
    F_okay,
    F_okay,
    F_okay,
    F_okay,
    F_okay,
    F_okay,
    F_okay,
    F_okay,
    F_okay,
    F_okay,
    F_okay,
    F_okay,
    F_okay,
    F_okay,
    F_okay,
    F_okay,
    F_okay,
    F_okay,
    F_okay,
    F_okay,
    F_okay,
    F_okay,
    F_okay,
    F_okay,
    F_okay,
    F_okay,
    F_okay,
    F_okay,
    F_okay,
    F_okay,
    F_okay,
    F_okay,
    F_okay,
    F_okay,
    F_okay,
    F_okay,
    F_okay,
    F_okay,
    F_okay,
    F_okay,
    F_okay,
    F_okay,
    F_okay,
    F_okay,
    F_okay,
    F_okay,
    F_okay,
    F_okay,
    F_okay,
    F_okay,
    F_okay,
    F_okay,
    F_okay,
    F_okay,
    F_okay,
    F_okay,
    F_okay,
    F_okay,
    F_okay,
    F_okay,
    F_okay,
    F_okay,
    F_okay,
    F_okay,
    F_okay,
    F_okay,
    F_okay,
    F_okay,
    F_okay,
    F_okay,
    F_okay,
    F_okay,
    F_okay,
    F_okay,
    F_okay,
    F_okay,
    F_okay,
    F_okay,
    F_okay,
    F_okay,
    F_okay,
    F_okay,
    F_okay,
    F_okay,
    F_okay,
    F_okay,
    F_okay,
    F_okay,
    F_okay,
    F_okay,
    F_okay,
    F_okay,
    F_okay,
    F_okay,
    F_okay,
    F_okay,
    F_okay,
    F_okay,
    F_okay,
    F_okay,
    F_okay,
    F_okay,
    F_okay,
    F_okay,
    F_okay,
    F_okay,
    F_okay,
    F_okay,
    F_okay,
    F_okay,
    F_okay,
    F_okay,
    F_okay,
    F_okay,
    F_okay,
    F_okay,
    F_okay,
    F_okay,
    F_okay,
    F_okay,
    F_okay,
    F_okay,
    F_okay,
    F_okay,
    F_okay,
    F_okay,
    F_okay,
    F_okay,
    F_okay,
    F_okay,
    F_okay,
    F_okay,
    F_okay,
    F_okay,
    F_okay,
    F_okay,
    F_okay,
    F_okay,
    F_okay,
    F_okay,
    F_okay,
    F_okay,
    F_okay,
    F_okay,
    F_okay,
    F_okay,
    F_okay,
    F_okay,
    F_okay,
    F_okay,
    F_okay,
    F_okay,
    F_okay,
    F_okay,
    F_okay,
    F_okay,
    F_okay,
    F_okay,
    F_okay,
    F_okay,
    F_okay,
    F_okay,
    F_okay,
    F_okay,
    F_okay,
    F_okay,
    F_okay,
    F_okay,
    F_okay,
    F_okay,
    F_okay,
    F_okay,
    F_okay,
    F_okay,
    F_okay,
    F_okay,
    F_okay,
    F_okay,
    F_okay,
    F_okay,
    F_okay,
    F_okay,
    F_okay,
    F_okay,
    F_okay,
    F_okay,
    F_okay,
    F_okay,
    F_okay,
    F_okay,
    F_okay,
    F_okay,
    F_okay,
    F_okay,
  };

  for (uint8_t i = 0; i < _inline_macro_total_rows; ++i) {

    state_data.flag = flags[i];
    destination.used = 0;

    f_iki_eki_write(vocabularyss[i], contents[i], quotes[i], &destination, &state_data);

    assert_int_equal(state_data.status, statuss[i]);
    assert_int_equal(destination.used, ikis[i].used);
    assert_string_equal(destination.string, ikis[i].string);

    // Clear the string to make any debugging easier.
    memset(destination.string, 0, sizeof(destination.used));
  } // for

  if (destination.string) free(destination.string);

  #undef _inline_macro_total_rows
}

#ifdef __cplusplus
} // extern "C"
#endif
