#include "test-time.h"
#include "test-time-calendar_string.h"

#ifdef __cplusplus
extern "C" {
#endif

void test__f_time_calendar_string__fails(void **state) {

  const time_t time = { 0 };
  f_string_dynamic_t destination = f_string_dynamic_t_initialize;

  {
    int errnos[] = {
      EOVERFLOW,
      mock_errno_generic,
    };

    f_status_t statuss[] = {
      F_status_set_error(F_buffer_overflow),
      F_status_set_error(F_failure),
    };

    for (int i = 0; i < 2; ++i) {

      will_return(__wrap_ctime_r, true);
      will_return(__wrap_ctime_r, errnos[i]);

      const f_status_t status = f_time_calendar_string(time, &destination);

      assert_int_equal(status, statuss[i]);
    } // for
  }
}

void test__f_time_calendar_string__parameter_checking(void **state) {

  const time_t time = { 0 };

  {
    const f_status_t status = f_time_calendar_string(time, 0);

    assert_int_equal(status, F_status_set_error(F_parameter));
  }
}

void test__f_time_calendar_string__works(void **state) {

  mock_unwrap_f_string = 0;

  const time_t time = { 0 };
  const f_string_static_t example = macro_f_string_static_t_initialize_1("example", 0, 7);

  {
    f_string_dynamic_t destination = f_string_dynamic_t_initialize;

    f_char_t example_string[255];
    memset((void *) example_string, 0, 255);
    memcpy((void *) example_string, example.string, example.used);

    will_return(__wrap_ctime_r, false);
    will_return(__wrap_ctime_r, example_string);
    will_return(__wrap_f_string_append, false);
    will_return(__wrap_f_string_append, example.string);
    will_return(__wrap_f_string_append, example.used);
    will_return(__wrap_f_string_append, F_okay);

    const f_status_t status = f_time_calendar_string(time, &destination);

    assert_int_equal(status, F_okay);
    assert_int_equal(destination.used, example.used);
    assert_string_equal(destination.string, example.string);
  }
}

#ifdef __cplusplus
} // extern "C"
#endif
