#include "test-time.h"
#include "test-time-clock_get.h"

#ifdef __cplusplus
extern "C" {
#endif

void test__f_time_clock_get__fails(void **state) {

  {
    int errnos[] = {
      EACCES,
      EFAULT,
      ENODEV,
      ENOTSUP,
      EINVAL,
      EOVERFLOW,
      EPERM,
      mock_errno_generic,
    };

    f_status_t statuss[] = {
      F_status_set_error(F_access_denied),
      F_status_set_error(F_buffer),
      F_status_set_error(F_device_not),
      F_status_set_error(F_support_not),
      F_status_set_error(F_parameter),
      F_status_set_error(F_buffer_overflow),
      F_status_set_error(F_prohibited),
      F_status_set_error(F_failure),
    };

    for (int i = 0; i < 8; ++i) {

      f_time_spec_t time = f_time_spec_t_initialize;

      will_return(__wrap_clock_gettime, true);
      will_return(__wrap_clock_gettime, errnos[i]);

      const f_status_t status = f_time_clock_get(0, &time);

      assert_int_equal(status, statuss[i]);
    } // for
  }
}

void test__f_time_clock_get__parameter_checking(void **state) {

  {
    const f_status_t status = f_time_clock_get(0, 0);

    assert_int_equal(status, F_status_set_error(F_parameter));
  }
}

void test__f_time_clock_get__works(void **state) {

  const f_time_spec_t expect = { .tv_sec = 1, .tv_nsec = 2 };

  {
    f_time_spec_t time = f_time_spec_t_initialize;

    will_return(__wrap_clock_gettime, false);
    will_return(__wrap_clock_gettime, &expect);

    const f_status_t status = f_time_clock_get(0, &time);

    assert_int_equal(status, F_okay);
    assert_int_equal(time.tv_sec, expect.tv_sec);
    assert_int_equal(time.tv_nsec, expect.tv_nsec);
  }
}

#ifdef __cplusplus
} // extern "C"
#endif
