#include "test-time.h"
#include "test-time-local_get.h"

#ifdef __cplusplus
extern "C" {
#endif

void test__f_time_local_get__fails(void **state) {

  const time_t time = { 0 };
  f_time_part_t part = f_time_part_t_initialize;

  {
    int errnos[] = {
      EOVERFLOW,
      mock_errno_generic,
    };

    f_status_t statuss[] = {
      F_status_set_error(F_buffer_overflow),
      F_status_set_error(F_failure),
    };

    for (int i = 0; i < 2; ++i) {

      will_return(__wrap_localtime_r, true);
      will_return(__wrap_localtime_r, errnos[i]);

      const f_status_t status = f_time_local_get(time, &part);

      assert_int_equal(status, statuss[i]);
    } // for
  }
}

void test__f_time_local_get__parameter_checking(void **state) {

  const time_t time = { 0 };

  {
    const f_status_t status = f_time_local_get(time, 0);

    assert_int_equal(status, F_status_set_error(F_parameter));
  }
}

void test__f_time_local_get__works(void **state) {

  const time_t time = { 0 };
  const f_time_part_t expect = { .tm_sec = 1, .tm_min = 2, .tm_hour = 3 };

  {
    f_time_part_t part = f_time_part_t_initialize;

    will_return(__wrap_localtime_r, false);
    will_return(__wrap_localtime_r, &expect);

    const f_status_t status = f_time_local_get(time, &part);

    assert_int_equal(status, F_okay);
    assert_int_equal(part.tm_sec, expect.tm_sec);
    assert_int_equal(part.tm_min, expect.tm_min);
    assert_int_equal(part.tm_hour, expect.tm_hour);
  }
}

#ifdef __cplusplus
} // extern "C"
#endif
