#include "test-time.h"
#include "test-time-of_day_get.h"

#ifdef __cplusplus
extern "C" {
#endif

void test__f_time_of_day_get__fails(void **state) {

  {
    int errnos[] = {
      EFAULT,
      EINVAL,
      EPERM,
      mock_errno_generic,
    };

    f_status_t statuss[] = {
      F_status_set_error(F_buffer),
      F_status_set_error(F_parameter),
      F_status_set_error(F_prohibited),
      F_status_set_error(F_failure),
    };

    for (int i = 0; i < 4; ++i) {

      f_time_value_t time = f_time_value_t_initialize;

      will_return(__wrap_gettimeofday, true);
      will_return(__wrap_gettimeofday, errnos[i]);

      const f_status_t status = f_time_of_day_get(&time);

      assert_int_equal(status, statuss[i]);
    } // for
  }
}

void test__f_time_of_day_get__parameter_checking(void **state) {

  {
    const f_status_t status = f_time_of_day_get(0);

    assert_int_equal(status, F_status_set_error(F_parameter));
  }
}

void test__f_time_of_day_get__works(void **state) {

  const f_time_value_t expect = { .tv_sec = 1, .tv_usec = 2 };

  {
    f_time_value_t time = f_time_value_t_initialize;

    will_return(__wrap_gettimeofday, false);
    will_return(__wrap_gettimeofday, &expect);

    const f_status_t status = f_time_of_day_get(&time);

    assert_int_equal(status, F_okay);
    assert_int_equal(time.tv_sec, expect.tv_sec);
    assert_int_equal(time.tv_usec, expect.tv_usec);
  }
}

#ifdef __cplusplus
} // extern "C"
#endif
