#include "test-time.h"
#include "test-time-sleep_spec.h"

#ifdef __cplusplus
extern "C" {
#endif

void test__f_time_sleep_spec__fails(void **state) {

  const f_time_spec_t spec = f_time_spec_t_initialize;

  {
    int errnos[] = {
      EFAULT,
      EINTR,
      EINVAL,
      mock_errno_generic,
    };

    f_status_t statuss[] = {
      F_status_set_error(F_buffer),
      F_interrupt,
      F_status_set_error(F_parameter),
      F_status_set_error(F_failure),
    };

    for (int i = 0; i < 4; ++i) {

      will_return(__wrap_nanosleep, true);
      will_return(__wrap_nanosleep, errnos[i]);

      const f_status_t status = f_time_sleep_spec(spec, 0);

      assert_int_equal(status, statuss[i]);
    } // for
  }
}

void test__f_time_sleep_spec__works(void **state) {

  const f_time_spec_t spec = { .tv_sec = 1, .tv_nsec = 1 };
  const f_time_spec_t expect = { .tv_sec = 3, .tv_nsec = 4 };

  {
    will_return(__wrap_nanosleep, false);

    const f_status_t status = f_time_sleep_spec(spec, 0);

    assert_int_equal(status, F_okay);
  }

  {
    f_time_spec_t remaining = f_time_spec_t_initialize;

    will_return(__wrap_nanosleep, false);
    will_return(__wrap_nanosleep, &expect);

    const f_status_t status = f_time_sleep_spec(spec, &remaining);

    assert_int_equal(status, F_okay);
    assert_int_equal(remaining.tv_sec, expect.tv_sec);
    assert_int_equal(remaining.tv_nsec, expect.tv_nsec);
  }
}

#ifdef __cplusplus
} // extern "C"
#endif
